#!/bin/ksh
#
#
#
# execNLMECmd.sh : Runs NLME on Linux from command line.
#
# Args :
#        %1 RUN_MODE   = COMPILE_AND_RUN COMPILE RUN
#        %2 MODELFILE  = PML file to use for the model
#        %3 LCWD       = Full path Working directory to start on local host
#        %4 MPIFLAG    = MPIYES | MPINO
#        %5 LOCAL_HOST =  YES | NO
#        %6 NUM_NODES  = Number of mpi nodes
#        %7 SHARED_DRIVE = Location of shared drive for #ote MPI
#        %8 RCWD         = Name of the working directory on #ote/shared drive
#        
#
#                          This directory will be created on either :
#                          --  Shared Drive OR
#                          --  $HOME
#                              on #ote nodes and will be cleaned after the
#                              run is finished.
# 
#        %9 FILES_TO_COPY= List of files to copy to #ote node or shared directory
#        %10 NLME_ARGS    = Arguments passed on to xxx.exe
# 


RUN_MODE=$1
MODELFILE=$2
WORKING_DIR=$3
MPIFLAG=$4
LOCAL_HOST=$5
NUM_NODES=$6
SHARED_DRIVE=$7
RCWD=${8}_$RANDOM
FILES=$9
NLME_ARGS=${10}
CMD_HASHCODE=${11}
NLME_EXE_POSTFIX=${12}

####### pushd $WORKING_DIR
cd $WORKING_DIR


echo WORKING_DIR=$WORKING_DIR,MPIFLAG=$MPIFLAG, LOCAL_HOST=$LOCAL_HOST,NUM_NODES=$NUM_NODES,SHARED_DRIVE=$SHARED_DRIVE

EXECUTION_DIR=

HASH_TO_USE=

if [ -z "${CMD_HASHCODE}" ]  
then
    HASH=${NLME_HASH}
    if [ -z "${HASH}" ]  
    then
        HASH_TO_USE=
    else
        HASH_TO_USE=" /hash ${HASH} "
    fi
else
    HASH_TO_USE=" /hash ${CMD_HASHCODE} "
fi


#
# First thing is to translate the model file and create NLME.exe
# 


if [ -z "${INSTALLDIR}" ]  
then
   INSTALLDIR=$PhoenixDir/application/lib/NLME/Executables
fi


if [ -z "${NLME_BUILD_DEBUG}" ]
then
    DEBUG_FLAG=-O2
else
    DEBUG_FLAG=-g
fi

PLAT="_linux"
ARCH=""
if [ "${NLME_PLATFORM}" == "INTEL" ]
then
    LIBEXT="i"
    BUILD_FLAGS="-I.. -malign-double -fverbose-asm -UWIN32 -c"
    GPP="icc  ${BUILD_FLAGS} ${DEBUG_FLAG}  "
    G77="icc -Bstatic --fno-underscoring --enable-stdcall-fixup -g -u main -u _SETNP -u _DGPADM -u _DVERK "
    if [[ -z "$INTEL_HOME" ]]; then
	. ${INSTALLDIR}/NLME7/Build/IntelEnv.sh
    fi
else
    LIBEXT="g"
    BUILD_FLAGS="-I.. -malign-double -fverbose-asm -UWIN32 -c"
    GPP="gcc  ${BUILD_FLAGS} ${DEBUG_FLAG}  "
#    G77="gfortran -Bstatic --fno-underscoring --enable-stdcall-fixup ${DEBUG_FLAG}  -u main -u _SETNP -u _DGPADM -u _DVERK "
	G77="gfortran -Bstatic  ${DEBUG_FLAG}  -u main -u _SETNP -u _DGPADM -u _DVERK "
fi


#  Define any undefined args - note if OUTERRESULTFILE is not defined (normally the case in the examples)
#  it is hard coded to out.txt below.  

if [ "${OUTERRESULTFILE}" == ""  ]
then
    OUTERRESULTFILE=out.txt
fi

if [ "${SUBMODELS}" == ""  ]
then
    SUBMODELS=0
fi

if [ "${PhoenixMPIDir64}" == ""  ]
then
    PhoenixMPIDir64=/usr/
fi


if [ "${RUN_MODE}" != "RUN" ]
then

echo      model=${MODELFILE}, nlmeDir=${INSTALLDIR}
#------------------------------------------------------
echo      Deleting files

if [ ! -d Work ]
then
    mkdir Work
fi

rm -f Work/*


#------------------------------------------------------
echo "-------------------------------------------------------------"
echo "--------------------  Translating  --------------------------"

#"${INSTALLDIR}/TDL4" /hash 1539912481 /L ./${MODELFILE} ./Work  > log.txt
#"${INSTALLDIR}/TDL4" /hash 1521876431 /L ./${MODELFILE} ./Work  > log.txt
echo "${INSTALLDIR}/TDL4" ${HASH_TO_USE} /L ./${MODELFILE} ./Work  
"${INSTALLDIR}/TDL4" ${HASH_TO_USE} /L ./${MODELFILE} ./Work  

echo      Done

if [ $? != 0 ]
then
    echo "ERROR in model translation"
    cat log.txt
    exit 1 
fi

if [ ! -f Work/Model.c ]
then
    echo "ERROR in generating Model.c"
    exit 1
fi
	cp  "${INSTALLDIR}/ModelAPI.h" ./Work
	cp  "${INSTALLDIR}/MUtil.h" ./Work

#------------------------------------------------------
echo "-------------------------------------------------------------"
echo "------------------- Compliling *.c  -------------------------"


	${GPP} ./Work/Model.c
	mv  Model.o ./Work 
	cp  "${INSTALLDIR}/MUtil.c" ./Work
	${GPP} ./Work/MUtil.c
	mv  MUtil.o ./Work

#------------------------------------------------------

echo "-------------------------------------------------------------"
echo "----------------------- Linking -----------------------------"

#if [ "${NLME_PLATFORM}" == "INTEL" ]
#then
# 	icc -g -u main -u _SETNP -u _DGPADM -u _DVERK \
# "${INSTALLDIR}/NLME7/Lib/NLME7${LIBEXT}${PLAT}${ARCH}.a" \
# ./Work/Model.o ./Work/MUtil.o \
# "${INSTALLDIR}/LAPACK/Lib/lapack${LIBEXT}${PLAT}${ARCH}.a" \
# "${INSTALLDIR}/LAPACK/Lib/blas${LIBEXT}${PLAT}${ARCH}.a" \
# -lstdc++ \
# -L${INTEL_FC_COMPILER_LIBS} -lifcore -lifport \
# -o NLME7.exe
#else
# 	gfortran -Bstatic --fno-underscoring --enable-stdcall-fixup -u main -u _SETNP -u _DGPADM -u _DVERK "${INSTALLDIR}/NLME7/Lib/NLME7${LIBEXT}${PLAT}${ARCH}.a" ./Work/Model.o ./Work/MUtil.o "${INSTALLDIR}/LAPACK/Lib/lapack${LIBEXT}${PLAT}${ARCH}.a" "${INSTALLDIR}/LAPACK/Lib/blas${LIBEXT}${PLAT}${ARCH}.a" -lstdc++ -o NLME7${NLME_EXE_POSTFIX}.exe
#fi
#


if [ "${MPIFLAG}" == "MPINO" ]
then
    ${G77} "${INSTALLDIR}//NLME7${LIBEXT}${PLAT}${ARCH}.a" ./Work/Model.o ./Work/MUtil.o "${INSTALLDIR}/lapack${LIBEXT}${PLAT}${ARCH}.a" "${INSTALLDIR}/blas${LIBEXT}${PLAT}${ARCH}.a" -lstdc++ -o NLME7${NLME_EXE_POSTFIX}.exe
else
    ${PhoenixMPIDir64}//bin/mpif90 -Bstatic -g -u main -u _SETNP -u _DGPADM -u _DVERK  "${INSTALLDIR}/mpiNLME7${LIBEXT}${PLAT}${ARCH}.a" ./Work/Model.o ./Work/MUtil.o "${INSTALLDIR}/lapack${LIBEXT}${PLAT}${ARCH}.a" "${INSTALLDIR}/blas${LIBEXT}${PLAT}${ARCH}.a" -lstdc++ -o mpiNLME7${NLME_EXE_POSTFIX}.exe
fi

if [ $? != 0 ]
then
    echo "ERROR in generating NLME7.exe"
    cat log.txt
    exit 1 
fi
fi
echo "-------------------------------------------------------------"

if [ "${RUN_MODE}" == "COMPILE" ]
then
    exit 0 
fi

#------------------------------------------------------
	RESULTFILE=out.txt
echo      Running
if [ "${MPIFLAG}" == "MPINO" ]
then
    echo ./NLME7${NLME_EXE_POSTFIX}.exe ${NLME_ARGS} 
    ./NLME7${NLME_EXE_POSTFIX}.exe ${NLME_ARGS} 1> err1.txt 2>err2.txt
else
    if [ "${LOCAL_HOST}" == "YES" ]
    then
        LOCALONLY="-localonly"
        MAPALLFLAG=
        EXECUTABLE=./mpiNLME7${NLME_EXE_POSTFIX}.exe
        FILES_TO_COPY=
    else
    #
    # Figure out what hosts we have
    #

            LOCALONLY=""
            MAPALLFLAG=
            EXECUTABLE=./mpiNLME7${NLME_EXE_POSTFIX}.exe
            FILES_TO_COPY=
            WD="-wdir "`pwd`  
    fi
#    NLME_ARGS="${NLME_ARGS//EXECUTION_DIR/$EXECUTION_DIR}"
    echo "${PhoenixMPIDir64}/bin/mpiexec"  -n ${NUM_NODES} ${LOCALONLY}  ${WD} ${EXECUTABLE} ${NLME_ARGS} 
#   "${PhoenixMPIDir64}/bin/mpiexec" -channel NEMESIS -n ${NUM_NODES} ${LOCALONLY}  ${WD} ${EXECUTABLE} ${NLME_ARGS}  2>err1.txt 2>err2.txt
	"${PhoenixMPIDir64}/bin/mpiexec" -n ${NUM_NODES} ${LOCALONLY}  ${WD} ${EXECUTABLE} ${NLME_ARGS}  2>err1.txt 2>err2.txt
fi

if [ $? != 0 ]
then
    echo      ERROR in running model fitting engine
    cat log.txt
    exit 1 
fi

grep "exit from engine" err1.txt

#
# Clean up temporary files
#
if [ -z "${SHARED_DRIVE}" ]
then
    rm -rf ${EXECUTION_DIR}
else
    rm -rf ${EXECUTION_DIR}
fi
