// MUtil.h

#ifdef min
	#undef min
#endif

#ifdef max
	#undef max
#endif

typedef int BOOL;
#define FALSE 0
#define TRUE 1

#define INTERP_MODE_LL		0
#define INTERP_MODE_PRED	1
#define INTERP_MODE_SIM		2
#define INTERP_MODE_INIT1	3

#define _UNK 3.3e-151
#define _NA 3.3e-151

extern double curTime, curTimeLastDose;
extern int iCurWhichDose;
extern int iCurWhichReset;
extern double _dMaxTime;
extern int _bOnTrajectory;
extern int _bDroppingOut;

typedef struct _statemachineTag {
	int _iState;
	void (*_func)(void* zzp, double t, double zzY[]);
} _statemachine;

// table stuff
void _tableGetXPos(int _n, double _x, int* _ix0, int* _ix1, double* _xFrac, ...);
void _tableSetYVal(int _n, double* _y, int _ix0, int _ix1, double _xFrac, ...);

// delay stuff
#define COUNTOF(a) (sizeof(a)/sizeof((a)[0]))
#define CDF_TABLE_SIZE 128
//#define CDF_TABLE_SIZE 8
#define GROW_DELAYTABLE 1024
#define MAX_DELAYTABLE (GROW_DELAYTABLE * 1024)
typedef struct _delayTableTag {
	double delt, c, lgammc, gammc;
	int iDoseMode;
	// table of past values of expression
	int nt;
	int mxt;
	double* t;
	double* y;
	// historical y value prior to table
	double _yHist;
	// past dose functions
	int ntDose;
	int mxtDose;
	double* tDose; // time
	double* yDose; // amount
	double* rDose; // rate (or _NA)
	// gamma distribution function
	double cq;	// c for current distribution function table
	int nq;
	double q[CDF_TABLE_SIZE+1];
	double p[CDF_TABLE_SIZE+1];
	//// values of t translated for current gamma table
	//double latestt;
	//double deltq; // delt for current translation table
	//double transt[CDF_TABLE_SIZE+1];
} _delayTable_t;

void _initDelayTable(_delayTable_t* pt);
double _delayFunc(double _tt, double _y, double _delt, double _shape, double _threshold, _delayTable_t* _pt);
void _delayAddIncrements(double* py, double _ta, double _tb, double _delt, double _shape, double _threshold, _delayTable_t* _pt);
void _insertDose(double _tt, double _y, double _r, _delayTable_t* _pt);

// peak stuff
typedef struct _peakTableTag {
	double v[10];
} _peakTable_t;

void _initPeakTable(_peakTable_t* pt);
double _peakFunc(double _tt, double _y, double _bMin, double *_cmax, _peakTable_t* _pt);

// declare generic closed-form state machine structure
typedef struct _cf2machineTag {
	double dTimeOfState;
	double dTimeOfGetVal;
	double dRateA;	// infusion rate into absorption cpt
	double dRate1;	// infusion rate into cpt 1
	int na;			// number of exponentials, max 5
	int err;

	double aaa[5];							// residues for compartment A from a(0)
	double a1a[5], a11[5], a12[5], a13[5];	// residues for compartment 1 from a(0), y1(0), y2(0), y3(0)
	double a2a[5], a21[5], a22[5], a23[5];	// residues for compartment 2 from a(0), y1(0), y2(0), y3(0)
	double a3a[5], a31[5], a32[5], a33[5];	// residues for compartment 3 from a(0), y1(0), y2(0), y3(0)

	double x[5];	// X's (negative rate constants)

	double staa[5];	// state of each exponential at time t (compartment a from a)

	double st1a[5];	// state of each exponential at time t (compartment 1 from a)
	double st11[5];	// state of each exponential at time t (compartment 1 from 1)
	double st12[5];	// state of each exponential at time t (compartment 1 from 2)
	double st13[5];	// state of each exponential at time t (compartment 1 from 3)

	double st2a[5];	// state of each exponential at time t (compartment 2 from a)
	double st21[5];	// state of each exponential at time t (compartment 2 from 1)
	double st22[5];	// state of each exponential at time t (compartment 2 from 2)
	double st23[5];	// state of each exponential at time t (compartment 2 from 3)

	double st3a[5];	// state of each exponential at time t (compartment 3 from a)
	double st31[5];	// state of each exponential at time t (compartment 3 from 1)
	double st32[5];	// state of each exponential at time t (compartment 3 from 2)
	double st33[5];	// state of each exponential at time t (compartment 3 from 3)

} _cf2machine;

void CF2Clear(_cf2machine* _p);
void CF2InitMacro1(_cf2machine* _p, double _al0);
void CF2InitMacro2(_cf2machine* _p, double _al0, double _a1, double _al1);
void CF2InitMacro3(_cf2machine* _p, double _al0, double _a1, double _al1, double _a2, double _al2);

void CF2InitMacro1New(_cf2machine* _p, double _a0, double _al0);
void CF2InitMacro2New(_cf2machine* _p, double _a0, double _al0, double _a1, double _al1);
void CF2InitMacro3New(_cf2machine* _p, double _a0, double _al0, double _a1, double _al1, double _a2, double _al2);
void CF2AppendKaNew(_cf2machine* _p, double Ka);

void CF2InitMicro1(_cf2machine* _p, double _k10);
void CF2InitMicro2(_cf2machine* _p, double _k10, double _k12, double _k21);
void CF2InitMicro3(_cf2machine* _p, double _k10, double _k12, double _k21, double _k13, double _k31);
void CF2Convolve(
	  int _na, double _a[], double _al[]
	, int _nb, double _b[], double _bl[]
	, double _u[], double _v[]
	);
void CF2Add1stOrd(_cf2machine* _p, double _scale, double _ka);

void CF2Reset(_cf2machine* _p);

void CF2Advance(_cf2machine* _p, double _dt);

double CF2GetValA(_cf2machine* _p);
double CF2GetVal1(_cf2machine* _p);
double CF2GetVal2(_cf2machine* _p);
double CF2GetVal3(_cf2machine* _p);
double CF2GetRateA(_cf2machine* _p);
double CF2GetRate1(_cf2machine* _p);

void CF2SetValA(_cf2machine* _p, double _dv);
void CF2SetVal1(_cf2machine* _p, double _dv);
void CF2SetVal2(_cf2machine* _p, double _dv);
void CF2SetVal3(_cf2machine* _p, double _dv);
void CF2SetRateA(_cf2machine* _p, double _dr);
void CF2SetRate1(_cf2machine* _p, double _dr);

void CF2AddValA(_cf2machine* _p, double _dv);
void CF2AddVal1(_cf2machine* _p, double _dv);
void CF2AddVal2(_cf2machine* _p, double _dv);
void CF2AddVal3(_cf2machine* _p, double _dv);
void CF2AddRateA(_cf2machine* _p, double _dr);
void CF2AddRate1(_cf2machine* _p, double _dr);

double _TransitCorrectionFactor(double ntr);
double _interpolate(double x, double x0, double y0, double x1, double y1);
double factorial(double _x);
//double lnorm(double _x, double _std);
double dLNormdEta(double _x, double _s, double _dXdEta, double _dSdEta);
double gammCDF(double x, double c, double b, double lb, double lgammc, double gammc);
double gammPDF(double x, double c, double b, double lb, double lgammc);
double lnegbin_rp(double r, double p, double y);
double lnegbin(double mean, double beta, double power, double y);
double pnegbin(double mean, double beta, double power, double y);
int rnegbin_rp(double r, double p);
int rnegbin(double mean, double beta, double power);
int test_rnegbin();
double lpois(double _mean, int _x);
double ppois(double _mean, int _x);
int rpois(double lambda);
double lphi(double _x, double _v);
double phi(double _x);
double lambertw(const double _z);
double ilogit(double _x);	// inverse logit link function
double iloglog(double _x);	// inverse log-log link function
double icloglog(double _x);	// inverse complementary log-log link function
double lgamm(double _x);	// log gamma function
double erfunc(double _x);	// error function
double iprobit(double _x);	// inverse probit function (gaussian distribution function)
int unifToPoisson(double _mean, double _r);	// convert uniform distribution to poisson
double min(double _x, double _y);
double max(double _x, double _y);
double CalcTMax(double _A, double _a, double _B, double _b, double _C, double _c);
// there's always somebody who asks why we don't have "ln", so here it is
double ln(double _x);
double logfl(double _x);
//double log10(double _x);	// conflicts with math.h
double _exp_cached(double _x, double* _tempArg, double* _tempResult);
double _log_cached(double _x, double* _tempArg, double* _tempResult);
double _ln_cached(double _x, double* _tempArg, double* _tempResult);
double _pow_cached(double _x, double _y, double* _tempArg1, double* _tempArg2, double* _tempResult);
char* __NumberString(double _d);
char* _units_Mul(const char* _a, const char* _b);
char* _units_Div(const char* _a, const char* _b);
char* _units_Exp(const char* _a);
char* _units_Log(const char* _a);
//double abs(double x); // conflicts with math.h
double vfwt(double _f, double _p);

void _VecAddTo(int _n, double* _a, double* _b);
void _MatMulTo(int _na, int _ma, double* _a, int _mb, double* _b, double* _c);

// action table
#define ZZMAX_ACTION 10000

#define ZZACTION_BOLUS1		1
#define ZZACTION_INFSTRT1	2
#define ZZACTION_INFEND1	3
#define ZZACTION_BOLUS2		4
#define ZZACTION_INFSTRT2	5
#define ZZACTION_INFEND2	6

#define ZZACTION_SLEEP		7
#define ZZACTION_TIME		8
#define ZZACTION_DOSETIME	9

#define ZZMARK_BOLUS1		21
#define ZZMARK_INFSTRT1		22
#define ZZMARK_INFEND1		23
#define ZZMARK_BOLUS2		24
#define ZZMARK_INFSTRT2		25
#define ZZMARK_INFEND2		26
#define ZZMARK_ENDROW		27

#ifdef IN_MUTIL
	#define DEFVAR(typ, nam, init) typ nam = init
	#define DEFARRAY(typ, nam, siz) typ nam[siz]
	#define EXTERN
#else
	#define DEFVAR(typ, nam, init) extern typ nam
	#define DEFARRAY(typ, nam, siz) extern typ nam[]
	#define EXTERN extern
#endif

DEFVAR(int, iODELevel, 10);
DEFVAR(int, bModelAnagrad, 0);
void SetModelAnagrad(int b);

EXTERN void* _dbg;	// pointer to debug callback routine

EXTERN int zzNAction;
DEFARRAY(double, zzDT, ZZMAX_ACTION);			// time remaining to future action
DEFARRAY(int,   zzPriority, ZZMAX_ACTION);		// sort order of actions at same time
DEFARRAY(int,   zzActionCode, ZZMAX_ACTION);	// what to do at that time
DEFARRAY(int,   zzActionIArg, ZZMAX_ACTION);	// action argument integer
DEFARRAY(double, zzActionDArg1, ZZMAX_ACTION);	// action argument double
DEFARRAY(double, zzActionDArg2, ZZMAX_ACTION);	// action argument double
DEFARRAY(void*,  zzActionPArg, ZZMAX_ACTION);	// action argument pointer

// backup copy of the action queue
EXTERN int zzNAction_Bak;
DEFARRAY(double, zzDT_Bak, ZZMAX_ACTION);			// time remaining to future action
DEFARRAY(int,   zzPriority_Bak, ZZMAX_ACTION);		// sort order of actions at same time
DEFARRAY(int,   zzActionCode_Bak, ZZMAX_ACTION);	// what to do at that time
DEFARRAY(int,   zzActionIArg_Bak, ZZMAX_ACTION);	// action argument integer
DEFARRAY(double, zzActionDArg1_Bak, ZZMAX_ACTION);	// action argument double
DEFARRAY(double, zzActionDArg2_Bak, ZZMAX_ACTION);	// action argument double
DEFARRAY(void*,  zzActionPArg_Bak, ZZMAX_ACTION);	// action argument pointer

EXTERN int* _piState;	// state of ODE solver

EXTERN int bEnableLogTransform;	// turn on log transform if it's allowed
#ifdef IN_MUTIL
	double _logFloor = 1e-5;
#else
	extern double _logFloor;
#endif

#undef EXTERN
#undef DEFARRAY

// routine for debugging that calls back into NLME7
void CallDebug(int i, double d);

void InsertAction(double _dt, int _priority, int _code, int _iarg, double _darg1, double _darg2, void* _parg);
void InitSubject();
void InitDelays(double _t);
void InitCF(double _t);
void ReInitCF(double _t);
void SetTimeOfStateCF(double _t);
void SetTimeOfGetValCF(double _t);

void CF2Reset(_cf2machine* _p);
void CF2SetTimeOfState(_cf2machine* _p, double _t);
void CF2SetTimeOfGetVal(_cf2machine* _p, double _t);
void CF2Advance(_cf2machine* _p, double _dt);

void ResetSubject();
void ResetTime();
void ResetActions();
void ResetInteg();
void ResetCF();
void ResetUrine();
void ResetEvent();
//void ClearObservationVector();

void ScheduleBolus1(int _mark, double _t, int _iCpt, double _amt, int _addl);
void ScheduleBolus2(int _mark, double _t, int _iCpt, double _amt, int _addl);
void ScheduleIV1(int _mark, double _t, int _iCpt, double _amt, double _rate, int _addl);
void ScheduleIV2(int _mark, double _t, int _iCpt, double _amt, double _rate, int _addl);

void PerformBolus1(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv, double _rate);
void PerformBolus2(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv, double _rate);

void PerformInfStrt1(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv);
void PerformInfStrt2(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv);

void PerformInfEnd1(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv);
void PerformInfEnd2(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv);

void StartDroppingOut();
void Interp(
	  int _bGenGradients
	, double _dLL[]
	, double _dGrad[]
	, double _dId1, double _dId2, double _dId3, double _dId4, double _dId5
	, int* _pnObs
	, double** _args
	, int* _pbOK
	, int _iInterpMode
	, int bSampleEpsilon
	, int bForSimTbl
	, int bForVPC
	);

#ifdef _WIN32
#else
	#define _finite finite
#endif

void RecordObservation(double _dXPred, double _tLastDose, int _iWhichDose, int _iWhichReset, double _dYPred, double _dYObs, double _dVFunc, int _iWhichObs);
void RecordObservationGradient(double _dYdK, double _dSdK, int _iWhichK);

#ifdef __cplusplus
extern "C" {
#endif
void SetErrorMsg(const char*);
void ClearErrorMsg();
void SetSSFailedMessage();
#ifdef __cplusplus
}
#endif


void AdvanceByMatrixExponent(double _dt);
void SetGradArrays(int __nEta, double* __daYdK, double* __daSdK);
void LLOneSubject1(
	  int bGenGradients
	, int nEta
	, int* pnObs
	, const unsigned char* *pb
	, double* pdId1, double* pdId2, double* pdId3, double* pdId4, double* pdId5
	, double _dLL[]
	, double _dGrad[]
	, double** args
	, int* pbOK
	);
void NewPredOneSubject1(
	  int _bGenGradients
	, int _nEta
	, int* _pnObs
	, const unsigned char* *_ppb
	, double* _pdId1, double* _pdId2, double* _pdId3, double* _pdId4, double* _pdId5
	, double** _args
	, int* _pbOK
	, int* __nPrediction
	, double* __daXPrediction, double* __taLastDose, int* __iaWhichDose, int* __iaWhichReset, double* __daYPrediction, double* __daYObservation, double* __daVarianceFunction
	, double* __daYdK
	, double* __daSdK
	, int* __iaWhichObs
	);

double probit(double p);
