// Model.c
#include "stdio.h" // for sprintf and possibly fprintf
static char _msgbuf[1024];

// following line stops math.h from defining old bessel functions like y1, etc.
#define _NO_OLDNAMES
#define  NO_OLDNAMES
#include <math.h>
#include <float.h>
#include <string.h>
#include <memory.h>
#define IN_MODEL
	#include "MUtil.h"
	#include "ModelAPI.h"
	#include "Model.h"
#undef  _NO_OLDNAMES
#undef   NO_OLDNAMES

#undef  IN_MODEL

// this allows sigma of current epsilon to be referred to in the model
#define sigma() _std

static double t;
static double wt;
static double _initial_wt;
static double apgr;
static double _initial_apgr;
 // zzY[0] is extra integrator for infusions

#define _ISENABLED(i)(!_pocb || _pocb->_pfIsEnabled(i))
int _i_tvV = -1;	// index of tvV in _fixef
static double ___tvV = 0;		// value of tvV if it is disabled
#define tvV (*(_i_tvV >= 0 ? &_fixef[_i_tvV] : &___tvV))
int _i_tvKe = -1;	// index of tvKe in _fixef
static double ___tvKe = 0;		// value of tvKe if it is disabled
#define tvKe (*(_i_tvKe >= 0 ? &_fixef[_i_tvKe] : &___tvKe))
int _i_dVdwt = -1;	// index of dVdwt in _fixef
static double ___dVdwt = 0;		// value of dVdwt if it is disabled
#define dVdwt (*(_i_dVdwt >= 0 ? &_fixef[_i_dVdwt] : &___dVdwt))
int _i_dKedwt = -1;	// index of dKedwt in _fixef
static double ___dKedwt = 0;		// value of dKedwt if it is disabled
#define dKedwt (*(_i_dKedwt >= 0 ? &_fixef[_i_dKedwt] : &___dKedwt))
int _i_dVdapgr = -1;	// index of dVdapgr in _fixef
static double ___dVdapgr = 0;		// value of dVdapgr if it is disabled
#define dVdapgr (*(_i_dVdapgr >= 0 ? &_fixef[_i_dVdapgr] : &___dVdapgr))
void InitializeIndirectionTable(){
	_i_tvV = 0;
	_i_tvKe = 1;
	_i_dVdwt = (_ISENABLED(0) ? 2 : -1);
	_i_dKedwt = (_ISENABLED(1) ? 3 : -1);
	_i_dVdapgr = (_ISENABLED(2) ? 4 : -1);
}
#define nV _ranef5[0]
#define nKe _ranef5[1]
static double V;
static double _initial_V;
static double Ke;
static double _initial_Ke;
static double C;

void GetFixefName(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0){
	} else if (*_pi==0){ strcpy(_nm, "tvV");
	} else if (*_pi==1){ strcpy(_nm, "tvKe");
	} else if (*_pi==2){ strcpy(_nm, "dVdwt");
	} else if (*_pi==3){ strcpy(_nm, "dKedwt");
	} else if (*_pi==4){ strcpy(_nm, "dVdapgr");
	} else if (*_pi==5){ strcpy(_nm, "CEps");
	}
}

void GetFixefUnits(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0){
	} else if (*_pi==0){ strcpy(_nm, "");
	} else if (*_pi==1){ strcpy(_nm, "");
	} else if (*_pi==2){ strcpy(_nm, "");
	} else if (*_pi==3){ strcpy(_nm, "");
	} else if (*_pi==4){ strcpy(_nm, "");
	}
}

void GetSecondaryName(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0){
	}
}

void InitEnable(){	// Initialize Enablement
	InitializeIndirectionTable();
}

// if gaussian alg is allowed, but not being used, set this to 0
// so that the main standard deviation is treated as a theta
int _bUsingGaussianAlgorithm = 1;
void SetUsingGaussianAlgorithm(int* _pBool){
	_bUsingGaussianAlgorithm = *_pBool;
}

int GetNThetaOld(){	// get number of theta minus possibly 1 sigma to estimate
	int _n = 5 + 1; // number of fixefs + number of epsilons
	// don't count any frozen or disabled thetas
	if (!_ISENABLED(0)) _n--; // dVdwt disabled?
	if (!_ISENABLED(1)) _n--; // dKedwt disabled?
	if (!_ISENABLED(2)) _n--; // dVdapgr disabled?
	// if using a gaussian alg, don't treat the last stdev as a theta
	if (_bUsingGaussianAlgorithm)
		_n--;
	return _n;
}

int GetNTheta(){	// get number of theta to estimate
	int _n = 5 + 1; // number of fixefs + number of epsilons
	// don't count any frozen or disabled fixefs
	if (!_ISENABLED(0)) _n--; // dVdwt disabled?
	if (!_ISENABLED(1)) _n--; // dKedwt disabled?
	if (!_ISENABLED(2)) _n--; // dVdapgr disabled?
	// don't count any frozen errors
	return _n;
}

int GetNSigma(){	// get number of sigma to estimate, 0 or 1
	int _n = 1; // number of free epsilons
	// if not using a gaussian alg, treat the last stdev as a theta
	if (! _bUsingGaussianAlgorithm)
		_n--;
	return (_n > 0 ? 1 : 0);
}

void GetRanef1Name(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0){
	}
}

void GetRanef2Name(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0);
}

void GetRanef3Name(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0);
}

void GetRanef4Name(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0);
}

void GetRanef5Name(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0);
	else if (*_pi==0) strcpy(_nm, "nV");
	else if (*_pi==1) strcpy(_nm, "nKe");
}

void GetErrorName(int *_pi, char* _nm){
	_nm[0] = 0;
	if (0);
	else if (*_pi==0) strcpy(_nm, "CEps");
}


static double** _args;
static int* _pbOK;

void SetArgsOK(double** __args, int *__pbOK, int *__piState){
	_args = __args;
	_pbOK = __pbOK;
	_piState = __piState;
}

void SetODECallbacks(ODECallbackTable* _p){
	_pocb = _p;
}
void SetCallbacks(CallbackTable* _p){
	_pcb = _p;
}

double unif(){
	double result = 0.5;
	if (_pcb) result = (*_pcb->_pfGetUniform)();
	return result;
}

double norm(){
	double result = 0.0;
	if (_pcb) result = (*_pcb->_pfGetNormal)();
	return result;
}

const char* _t_Units;
const char* _CObs_Units;
const char* _A1_Units;
const char* _wt_Units;
const char* _apgr_Units;
void ClearUnits(){
	_t_Units = "";
	_CObs_Units = "";
	_A1_Units = "";
	_wt_Units = "";
	_apgr_Units = "";
}

void SetVarUnits(const char* _nm, const char* _units){
	if (0);
	else if (_nm[0]=='t' && strcmp(_nm, "t")==0) _t_Units = _units;
	else if (_nm[0]=='C' && strcmp(_nm, "CObs")==0) _CObs_Units = _units;
	else if (_nm[0]=='A' && strcmp(_nm, "A1")==0) _A1_Units = _units;
	else if (_nm[0]=='w' && strcmp(_nm, "wt")==0) _wt_Units = _units;
	else if (_nm[0]=='a' && strcmp(_nm, "apgr")==0) _apgr_Units = _units;
}

void GetPlotVarInfo(int *_pi, int* _pflags, char* _nmObs, char* _nmPred){
	*_pflags = 0;
	*_nmObs = '\0';
	*_nmPred = '\0';
	if (0){
	} else if (*_pi == 0){
		*_pflags = 0x011f;
		strcpy(_nmObs, "CObs");
		strcpy(_nmPred, "C");
	} else if (*_pi == 1){
		*_pflags = 0x0019;
		strcpy(_nmObs, "");
		strcpy(_nmPred, "V");
	} else if (*_pi == 2){
		*_pflags = 0x0019;
		strcpy(_nmObs, "");
		strcpy(_nmPred, "Ke");
	} else if (*_pi == 3){
		*_pflags = 0x0019;
		strcpy(_nmObs, "");
		strcpy(_nmPred, "t");
	} else if (*_pi == 4){
		*_pflags = 0x011b;
		strcpy(_nmObs, "");
		strcpy(_nmPred, "wt");
	} else if (*_pi == 5){
		*_pflags = 0x011b;
		strcpy(_nmObs, "");
		strcpy(_nmPred, "apgr");
	}
}

int GetNPlotVar(){return 6;}

void AddToCpt(double zzY[], int _iCpt, double _dv);	// forward declaration

int GetNumCovariates(){
	return 2;
}

int GetCovariateNumber(const char* _nm){
	int _i = -1;
	if (0);
	else if (_nm[0]=='w' && strcmp(_nm, "wt")==0) _i = 0;
	else if (_nm[0]=='a' && strcmp(_nm, "apgr")==0) _i = 1;
	return _i;
}

void GetCovariateName(int* _pi, char* _nm){
	if (0);
	else if (*_pi == 0) strcpy(_nm, "wt");
	else if (*_pi == 1) strcpy(_nm, "apgr");
}

void GetCovariateCategorical(int* _pi, int* _cat){
	if (0);
	else if (*_pi == 0) *_cat = 0;
	else if (*_pi == 1) *_cat = 0;
}

void GetCovariateDirection(int* _pi, int* _dir){
	// 0=backward, 1=forward, 2=interpolate
	if (0);
	else if (*_pi == 0) *_dir = 1;
	else if (*_pi == 1) *_dir = 1;
}

void SetCovariate(double _t, const char* _nm, double _v){
	if (0){
	} else if (_nm[0]=='w' && strcmp(_nm, "wt")==0){
		if (wt == _NA){
			_initial_wt = _v;
		}
		wt = _v;
	} else if (_nm[0]=='a' && strcmp(_nm, "apgr")==0){
		if (apgr == _NA){
			_initial_apgr = _v;
		}
		apgr = _v;
	}
}

void SetCovariateInterp(double _t, const char* _nm, double _v1, double _dt, double _v2, int _bForward){
	if (0){
	} else if (_nm[0]=='w' && strcmp(_nm, "wt")==0){
		wt = (_bForward ? _v1 : _v2);
	} else if (_nm[0]=='a' && strcmp(_nm, "apgr")==0){
		apgr = (_bForward ? _v1 : _v2);
	}
}

int GetStripDoseName(char* _nm){
	int _rvalue = 0;
	if (0){
	}
	return _rvalue;
}

int GetNumStructural(){
	return 2;
}

int GetStructuralNumber(const char* _nm){
	int _i = -1;
	if (0);
	else if (_nm[0]=='V' && strcmp(_nm, "V")==0) _i = 0;
	else if (_nm[0]=='K' && strcmp(_nm, "Ke")==0) _i = 1;
	return _i;
}

void GetStructuralName(int* _pi, char* _nm){
	if (0);
	else if (*_pi == 0) strcpy(_nm, "V");
	else if (*_pi == 1) strcpy(_nm, "Ke");
}

void GetVarUnits(const char* _nm, char* _units){
	const char* _p = (const char*)0L;
	_units[0] = 0;
	if (0){
	} else if (_nm[0]=='K' && strcmp(_nm, "Ke")==0){
		_p = _units_Div(__NumberString(1),_t_Units);
	} else if (_nm[0]=='A' && strcmp(_nm, "A1")==0){
		_p = _A1_Units;
	} else if (_nm[0]=='C' && strcmp(_nm, "CObs")==0){
		_p = _CObs_Units;
	} else if (_nm[0]=='w' && strcmp(_nm, "wt")==0){
		_p = _wt_Units;
	} else if (_nm[0]=='a' && strcmp(_nm, "apgr")==0){
		_p = _apgr_Units;
	} else if (_nm[0]=='t' && strcmp(_nm, "tvKe")==0){
		_p = _units_Div(__NumberString(1),_t_Units);
	} else if (_nm[0]=='C' && strcmp(_nm, "C")==0){
		_p = _CObs_Units;
	} else if (_nm[0]=='C' && strcmp(_nm, "CEps")==0){
		_p = _CObs_Units;
	} else if (_nm[0]=='V' && strcmp(_nm, "V")==0){
		_p = _units_Div(_A1_Units,_CObs_Units);
	} else if (_nm[0]=='n' && strcmp(_nm, "nKe")==0){
		_p = __NumberString(0);
	} else if (_nm[0]=='t' && strcmp(_nm, "tvV")==0){
		_p = _units_Div(_A1_Units,_CObs_Units);
	} else if (_nm[0]=='n' && strcmp(_nm, "nV")==0){
		_p = __NumberString(0);
	} else if (_nm[0]=='t' && strcmp(_nm, "t")==0){
		_p = _t_Units;
	}
	if (_p != (const char*)0L) strcpy(_units, _p);
}

_cf2machine _cf_A1;
#define A1 CF2GetVal1(&_cf_A1)
// routine to read out all the CF state vector
void GetAllCFState(double* _y){
	_y[0] = CF2GetValA(&_cf_A1);
	_y[1] = CF2GetVal1(&_cf_A1);
	_y[2] = CF2GetVal2(&_cf_A1);
	_y[3] = CF2GetVal3(&_cf_A1);
	_y[4] = CF2GetRateA(&_cf_A1);
	_y[5] = CF2GetRate1(&_cf_A1);
}
void SetAllCFState(double* _y){
	CF2SetValA(&_cf_A1, _y[0]);
	CF2SetVal1(&_cf_A1, _y[1]);
	CF2SetVal2(&_cf_A1, _y[2]);
	CF2SetVal3(&_cf_A1, _y[3]);
	CF2SetRateA(&_cf_A1, _y[4]);
	CF2SetRate1(&_cf_A1, _y[5]);
}
int GetNInteg(){
	return NINTEG_ALL;
}
void ClearSSIrrelevantState(double* _y){
}
void GetIntegrators(double* _y){
	int _i;
}
void SetIntegrators(double* _y){
	int _i;
}
void SetTimeOfStateCF(double _t){
	CF2SetTimeOfState(&_cf_A1, _t);
}
void SetTimeOfGetValCF(double _t){
	CF2SetTimeOfGetVal(&_cf_A1, _t);
}
#define USE_SETTIMECF
void ResetSubject(){
	ResetTime();
	ResetActions();
	ResetInteg();
	ResetCF();
	ResetUrine();
	TDL4ResetEvent();
	ResetGradient();
}
void InitDelays(double _t){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
}
void InitCF(double _t){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	CF2Clear(&_cf_A1);
	CF2InitMicro1(&_cf_A1, Ke);
	#if defined(USE_SETTIMECF)
		SetTimeOfStateCF(_t);
		SetTimeOfGetValCF(_t);
	#endif
}
// routine to capture CF state, change covariates, and re-initialize CF machines
// make sure all covariate-derived values are up to date before calling
void ReInitCF(double _t){
	double _state[NCFMACRO*6+1];
	GetAllCFState(_state); // get the compartment values
	InitCF(_t); // regenerate all the As
	SetAllCFState(_state); // put the values back in all the compartments
}
void GetNumCVMean(int* _pn){*_pn = 0;}
void GetCVMeanName(int* _pi, char* _nm){
	if (0);
}
void GetCVMean(int* _pi, double* _pv){
	if (0);
}
void SetCVMean(int* _pi, double* _pv){
	if (0);
}
void GetNumCVMedian(int* _pn){*_pn = 0;}
void GetCVMedianName(int* _pi, char* _nm){
	if (0);
}
void GetCVMedian(int* _pi, double* _pv){
	if (0);
}
void SetCVMedian(int* _pi, double* _pv){
	if (0);
}
void InitSubject(){
	ResetSubject();
	wt = _NA;
	_initial_wt = _NA;
	apgr = _NA;
	_initial_apgr = _NA;
	V = _NA;
	_initial_V = _NA;
	Ke = _NA;
	_initial_Ke = _NA;
	ClearErrorMsg();
}
void RestartSubjectSequences(){
}
void RestartSubjectEventSims(){
}
void EndSubject(int bSimulating){
	if (bSimulating){
	}
	// set covariates to their first settings, in case needed for secondary parameters
	SetCovariate(curTime, "wt", _initial_wt);
	SetCovariate(curTime, "apgr", _initial_apgr);
	V = _initial_V;
	Ke = _initial_Ke;
}
static double __tvV_low  = 3.3e-151;
static double __tvV_init = 1.393229999999999968;
static double __tvV_high = 3.3e-151;
static double __tvKe_low  = 3.3e-151;
static double __tvKe_init = 0.004874110000000000074;
static double __tvKe_high = 3.3e-151;
static double __dVdwt_low  = 3.3e-151;
static double __dVdwt_init = 0.0;
static double __dVdwt_high = 3.3e-151;
static double __dKedwt_low  = 3.3e-151;
static double __dKedwt_init = 0.0;
static double __dKedwt_high = 3.3e-151;
static double __dVdapgr_low  = 3.3e-151;
static double __dVdapgr_init = 0.0;
static double __dVdapgr_high = 3.3e-151;
void RestoreFixefLowInitHighFromModel(){
	__tvV_low  = 3.3e-151;
	__tvV_init = 1.393229999999999968;
	__tvV_high = 3.3e-151;
	__tvKe_low  = 3.3e-151;
	__tvKe_init = 0.004874110000000000074;
	__tvKe_high = 3.3e-151;
	__dVdwt_low  = 3.3e-151;
	__dVdwt_init = 0.0;
	__dVdwt_high = 3.3e-151;
	__dKedwt_low  = 3.3e-151;
	__dKedwt_init = 0.0;
	__dKedwt_high = 3.3e-151;
	__dVdapgr_low  = 3.3e-151;
	__dVdapgr_init = 0.0;
	__dVdapgr_high = 3.3e-151;
}
void SetFixefLowInitHigh(const char* _nm, double* _fixefLow, double* _fixefInit, double* _fixefHigh){
	if (0){
	} else if (strcmp(_nm, "tvV")==0){
		__tvV_low  = *_fixefLow;
		__tvV_init = *_fixefInit;
		__tvV_high = *_fixefHigh;
	} else if (strcmp(_nm, "tvKe")==0){
		__tvKe_low  = *_fixefLow;
		__tvKe_init = *_fixefInit;
		__tvKe_high = *_fixefHigh;
	} else if (strcmp(_nm, "dVdwt")==0){
		__dVdwt_low  = *_fixefLow;
		__dVdwt_init = *_fixefInit;
		__dVdwt_high = *_fixefHigh;
	} else if (strcmp(_nm, "dKedwt")==0){
		__dKedwt_low  = *_fixefLow;
		__dKedwt_init = *_fixefInit;
		__dKedwt_high = *_fixefHigh;
	} else if (strcmp(_nm, "dVdapgr")==0){
		__dVdapgr_low  = *_fixefLow;
		__dVdapgr_init = *_fixefInit;
		__dVdapgr_high = *_fixefHigh;
	}
}
void GetFixefLowInitHigh(const char* _nm, double* _fixefLow, double* _fixefInit, double* _fixefHigh){
	if (0){
	} else if (strcmp(_nm, "tvV")==0){
		*_fixefLow  = __tvV_low;
		*_fixefInit = __tvV_init;
		*_fixefHigh = __tvV_high;
	} else if (strcmp(_nm, "tvKe")==0){
		*_fixefLow  = __tvKe_low;
		*_fixefInit = __tvKe_init;
		*_fixefHigh = __tvKe_high;
	} else if (strcmp(_nm, "dVdwt")==0){
		*_fixefLow  = __dVdwt_low;
		*_fixefInit = __dVdwt_init;
		*_fixefHigh = __dVdwt_high;
	} else if (strcmp(_nm, "dKedwt")==0){
		*_fixefLow  = __dKedwt_low;
		*_fixefInit = __dKedwt_init;
		*_fixefHigh = __dKedwt_high;
	} else if (strcmp(_nm, "dVdapgr")==0){
		*_fixefLow  = __dVdapgr_low;
		*_fixefInit = __dVdapgr_init;
		*_fixefHigh = __dVdapgr_high;
	}
}
void GetThetaLimits(double* _fixefLow, double* _fixefHigh){
	int _n = 0;
	_fixefLow[_n] = __tvV_low; // set tvV
	_fixefHigh[_n] = __tvV_high;
	_n++;
	_fixefLow[_n] = __tvKe_low; // set tvKe
	_fixefHigh[_n] = __tvKe_high;
	_n++;
	if (_ISENABLED(0)){ // dVdwt enabled?
		_fixefLow[_n] = __dVdwt_low; // set dVdwt
		_fixefHigh[_n] = __dVdwt_high;
		_n++;
	}
	if (_ISENABLED(1)){ // dKedwt enabled?
		_fixefLow[_n] = __dKedwt_low; // set dKedwt
		_fixefHigh[_n] = __dKedwt_high;
		_n++;
	}
	if (_ISENABLED(2)){ // dVdapgr enabled?
		_fixefLow[_n] = __dVdapgr_low; // set dVdapgr
		_fixefHigh[_n] = __dVdapgr_high;
		_n++;
	}
	// if not using a gaussian alg, last theta is the main stdev
	if (! _bUsingGaussianAlgorithm){
		_fixefLow[_n] = 0; // stdev 0
		_fixefHigh[_n] = 1e100;
		_n++;
	}
}

void InitFixefErrorEstimate(){
	double* _fixef = _args[0];
	_fixef[0] = __tvV_init; // set tvV
	_fixef[1] = __tvKe_init; // set tvKe
	_fixef[2] = __dVdwt_init; // set dVdwt
	_fixef[3] = __dKedwt_init; // set dKedwt
	_fixef[4] = __dVdapgr_init; // set dVdapgr
	_fixef[5] = 2.85299; // stdev 0
}

void GetFixefErrorInitial(double* _fixef){
	_fixef[0] = __tvV_init; // set tvV
	_fixef[1] = __tvKe_init; // set tvKe
	_fixef[2] = __dVdwt_init; // set dVdwt
	_fixef[3] = __dKedwt_init; // set dKedwt
	_fixef[4] = __dVdapgr_init; // set dVdapgr
	_fixef[5] = 2.85299; // stdev 0
}

void GetFixefErrorLimits(double* _low, double* _high){
	_low[0] = __tvV_low; // set tvV
	_high[0] = __tvV_high;
	_low[1] = __tvKe_low; // set tvKe
	_high[1] = __tvKe_high;
	_low[2] = __dVdwt_low; // set dVdwt
	_high[2] = __dVdwt_high;
	_low[3] = __dKedwt_low; // set dKedwt
	_high[3] = __dKedwt_high;
	_low[4] = __dVdapgr_low; // set dVdapgr
	_high[4] = __dVdapgr_high;
	_low[5] = 0; // stdev 0
	_high[5] = 1e100; // stdev 0
}

void GetFixefValue(int* _iFixef, double* _dValue){
	*_dValue = _args[0][*_iFixef];
	return;
}

double GetObsSigma(int* _piWhichObs){
	double* _fixef = _args[0];
	double _std = 0;
	if (0){
	} else if (*_piWhichObs == 0){
		_std = _fixef[5];
	}
	return _std;
}

void GetInitialStdev(double* _svec){
	int _i = 0;
	double _temp = 0;
	_svec[0] = 2.852990000000000137;
}

void GetInitialOmega5(double* _omat){
	int _i = 0;
	double _temp = 0;
	for (_i=0; _i<4; _i++) _omat[_i] = 0;
	_temp = 0.1551800200000000018;
	_omat[0+0+0*2] = _temp;
	_temp = 0.03583175100000000196;
	_omat[0+1+0*2] = _temp;
	_omat[0+0+1*2] = _temp;
	_temp = 0.01115678800000000082;
	_omat[0+1+1*2] = _temp;
}

double GetEtaVariance5(int _i){
	double _temp = 1;
	if (0){
	} else if (_i == 0){
		_temp = 0.1551800200000000018;
	} else if (_i == 1){
		_temp = 0.01115678800000000082;
	}
	return _temp;
}

void TruncateOmega5(double* _omat){
	int _i = 0;
	double _temp = 0;
	double _mtemp[NRANEF5*NRANEF5+1];
	memcpy(_mtemp, _omat, sizeof(double)*NRANEF5*NRANEF5);
	memset(_omat, 0, sizeof(double)*NRANEF5*NRANEF5);
	_temp = _mtemp[0+0+0*2];
	_omat[0+0+0*2] = _temp;
	_temp = _mtemp[0+1+0*2];
	_omat[0+1+0*2] = _temp;
	_omat[0+0+1*2] = _temp;
	_temp = _mtemp[0+1+1*2];
	_omat[0+1+1*2] = _temp;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void UnpackParameters(double* _thvec, double* _omat, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _i1 = 0, _j1 = 0, _ldo = *_ldOmat;
	double _temp = 0;
	// assert NRANEF5 <= ldo
	_i1 = GetNTheta();
	for (_i = 0; _i < _i1; _i++){
		_thvec[_n++] = _pack[_i];
	}
	_i = 0 + 0;
	_j = 0 + 0;
	_omat[_j * _ldo + _i] = _pack[_n++];
	_i = 0 + 1;
	_j = 0 + 0;
	_omat[_j * _ldo + _i] = _pack[_n++];
	_i = 0 + 1;
	_j = 0 + 1;
	_omat[_j * _ldo + _i] = _pack[_n++];
	// assert *_nPack == _n;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void UnpackParameters1(double* _thvec, double* _omat, double* _svec, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _i1 = 0, _j1 = 0, _ldo = *_ldOmat;
	double _temp = 0;
	// assert NRANEF5 <= ldo
	_i1 = GetNTheta();
	for (_i = 0; _i < _i1; _i++){
		_thvec[_n++] = _pack[_i];
	}
	if (_i1 > 0){
		_svec[0] = _pack[_i1-1];
	}
	_i = 0 + 0;
	_j = 0 + 0;
	_omat[_j * _ldo + _i] = _pack[_n++];
	_i = 0 + 1;
	_j = 0 + 0;
	_omat[_j * _ldo + _i] = _pack[_n++];
	_i = 0 + 1;
	_j = 0 + 1;
	_omat[_j * _ldo + _i] = _pack[_n++];
	// assert *_nPack == _n;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void   PackParameters(double* _thvec, double* _omat, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _i1 = 0, _ldo = *_ldOmat;
	// assert NTHETA == *nth
	_i1 = GetNTheta();
	for (_i = 0; _i < _i1; _i++){
		_pack[_i] = _thvec[_n++];
	}
	{
		_i = 0 + 0;
		_j = 0 + 0;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	{
		_i = 0 + 1;
		_j = 0 + 0;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	{
		_i = 0 + 1;
		_j = 0 + 1;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	*_nPack = _n;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void   PackParameters1(double* _thvec, double* _omat, double* _svec, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _i1 = 0, _ldo = *_ldOmat;
	// assert NTHETA == *nth
	_i1 = GetNTheta();
	for (_i = 0; _i < _i1; _i++){
		_pack[_i] = _thvec[_n++];
	}
	if (_i1 > 0){
		_pack[_i1-1] = _svec[0];
	}
	{
		_i = 0 + 0;
		_j = 0 + 0;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	{
		_i = 0 + 1;
		_j = 0 + 0;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	{
		_i = 0 + 1;
		_j = 0 + 1;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	*_nPack = _n;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void   TypXParameters(double* _thvec, double* _omat, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _i1 = 0, _ldo = *_ldOmat;
	// assert NTHETA == *nth
	_i1 = GetNTheta();
	for (_i = 0; _i < _i1; _i++){
		_pack[_i] = fabs(_thvec[_n++]);
	}
	{
		_i = 0 + 0;
		_j = 0 + 0;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	{
		_i = 0 + 1;
		_j = 0 + 0;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	{
		_i = 0 + 1;
		_j = 0 + 1;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	*_nPack = _n;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void   TypXParameters1(double* _thvec, double* _omat, double* _svec, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _i1 = 0, _ldo = *_ldOmat;
	// assert NTHETA == *nth
	_i1 = GetNTheta();
	for (_i = 0; _i < _i1; _i++){
		_pack[_i] = fabs(_thvec[_n++]);
	}
	if (_i1 > 0){
		_pack[_i1-1] = fabs(_svec[0]);
	}
	{
		_i = 0 + 0;
		_j = 0 + 0;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	{
		_i = 0 + 1;
		_j = 0 + 0;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	{
		_i = 0 + 1;
		_j = 0 + 1;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	*_nPack = _n;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void UnpackOmega(double* _omat, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _i1 = 0, _j1 = 0, _ldo = *_ldOmat;
	double _temp = 0;
	// assert NRANEF5 <= ldo
	_i = 0 + 0;
	_j = 0 + 0;
	_omat[_j * _ldo + _i] = _pack[_n++];
	_i = 0 + 1;
	_j = 0 + 0;
	_omat[_j * _ldo + _i] = _pack[_n++];
	_i = 0 + 1;
	_j = 0 + 1;
	_omat[_j * _ldo + _i] = _pack[_n++];
	// assert *_nPack == _n;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void   PackOmega(double* _omat, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _ldo = *_ldOmat;
	// assert NTHETA == *nth
	{
		_i = 0 + 0;
		_j = 0 + 0;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	{
		_i = 0 + 1;
		_j = 0 + 0;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	{
		_i = 0 + 1;
		_j = 0 + 1;
		_pack[_n++] = _omat[_j * _ldo + _i];
	}
	*_nPack = _n;
}

// Caution: omega matrix is assumed to be in lower cholesky form
void   TypXOmega(double* _omat, double* _pack, int* _ldOmat, int* _nPack){
	int _i = 0, _j, _n = 0, _ldo = *_ldOmat;
	// assert NTHETA == *nth
	{
		_i = 0 + 0;
		_j = 0 + 0;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	{
		_i = 0 + 1;
		_j = 0 + 0;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	{
		_i = 0 + 1;
		_j = 0 + 1;
		_pack[_n++] = sqrt(fabs(_omat[_j * _ldo + _j]) * fabs(_omat[_i * _ldo + _i]));
	}
	*_nPack = _n;
}

void GetFixefErrToThetaMap(int* _iaTheta, int* _nth){
	int _n = 0;
	_iaTheta[0] = _n++; // tvV
	_iaTheta[1] = _n++; // tvKe
	_iaTheta[2] = (_ISENABLED(0) ? _n++ : -1); // dVdwt
	_iaTheta[3] = (_ISENABLED(1) ? _n++ : -1); // dKedwt
	_iaTheta[4] = (_ISENABLED(2) ? _n++ : -1); // dVdapgr
	// unfrozen error sigmas 
	_iaTheta[5] = _n++; // stdev 0
	*_nth = _n;
}

void GetRanefToEtaMap(int* _iaEta, int* _nth){
	int _n = 0;
	_iaEta[0] = _n++; // ranef 0
	_iaEta[1] = _n++; // ranef 1
	*_nth = _n;
}

void UnpackTheta(double* _fixef, double* _thvec){
	int _n = 0;
	// copy theta to fixef+error
	// unfrozen, enabled fixed effects 
	_fixef[0] = _thvec[_n++];
	_fixef[1] = _thvec[_n++];
	if (_ISENABLED(0)){
		_fixef[2] = _thvec[_n++];
	}
	if (_ISENABLED(1)){
		_fixef[3] = _thvec[_n++];
	}
	if (_ISENABLED(2)){
		_fixef[4] = _thvec[_n++];
	}
	// unfrozen error sigmas 
	_fixef[5] = _thvec[_n++];
}

void   PackTheta(double* _fixef, double* _thvec, int* _nth){
	int _n = 0;
	// copy fixef+error to theta
	// unfrozen, enabled fixed effects 
	_thvec[_n++] = _fixef[0];
	_thvec[_n++] = _fixef[1];
	if (_ISENABLED(0)){
		_thvec[_n++] = _fixef[2];
	}
	if (_ISENABLED(1)){
		_thvec[_n++] = _fixef[3];
	}
	if (_ISENABLED(2)){
		_thvec[_n++] = _fixef[4];
	}
	// unfrozen error sigmas 
	_thvec[_n++] = _fixef[5];
	*_nth = _n;
}

void   TypXTheta(double* _fixef, double* _thvec, int* _nth){
	int _n = 0;
	// copy fabs(fixef+error) to theta
	// unfrozen, enabled fixed effects 
	_thvec[_n++] = fabs(_fixef[0]);
	_thvec[_n++] = fabs(_fixef[1]);
	if (_ISENABLED(0)){
		_thvec[_n++] = fabs(_fixef[2]);
	}
	if (_ISENABLED(1)){
		_thvec[_n++] = fabs(_fixef[3]);
	}
	if (_ISENABLED(2)){
		_thvec[_n++] = fabs(_fixef[4]);
	}
	// unfrozen error sigmas 
	_thvec[_n++] = fabs(_fixef[5]);
	*_nth = _n;
}

// Caution: live portion of omega matrix is assumed to be packed in lower cholesky form
void   GetNPack(int* _nPack){
	int _n = 0;
	_n++; // tvV
	_n++; // tvKe
	if (_ISENABLED(0)) _n++; // dVdwt if enabled
	if (_ISENABLED(1)) _n++; // dKedwt if enabled
	if (_ISENABLED(2)) _n++; // dVdapgr if enabled
	_n++; // CEps
	{
		_n++;
	}
	{
		_n++;
	}
	{
		_n++;
	}
	*_nPack = _n;
}

// Caution: live portion of omega matrix is assumed to be packed in lower cholesky form
void   GetNPackOmega(int* _nPack){
	int _n = 0;
	{
		_n++;
	}
	{
		_n++;
	}
	{
		_n++;
	}
	*_nPack = _n;
}

void   GetNSeqOmega(int* _nSeq){
	int _n = 0;
	_n++; // block 0
	*_nSeq = _n;
}

void   GetOmegaSequenceInfo(int* _iSeq1, int* _nEta, int* _nSame){
	int _iSeqCount = 0;
	*_nEta = *_nSame = 0;
	if ((*_iSeq1-1) == _iSeqCount){
		*_nEta = 2;
		*_nSame = 0;
		return;
	}
	_iSeqCount++;
}

// Caution: live portion of omega matrix is assumed to be packed in lower cholesky form
void   GetNPackRanCov(int* _nPack){
	int _n = 0;
	{
		_n++;
	}
	{
		_n++;
	}
	{
		_n++;
	}
	*_nPack = _n;
}

// Caution: live portion of omega matrix is assumed to be packed in lower cholesky form
void   GetPackedIndexRanCov(int* _iRan, int* _jRan, int* _index){
	int _n = 0;
	*_index = -1;
	_n++; // tvV
	_n++; // tvKe
	if (_ISENABLED(0)){
		_n++; // dVdwt if enabled
	}
	if (_ISENABLED(1)){
		_n++; // dKedwt if enabled
	}
	if (_ISENABLED(2)){
		_n++; // dVdapgr if enabled
	}
	_n++; // CEps
	{
		if (*_iRan==0 && *_jRan==0){*_index = _n; return;}
		_n++;
	}
	{
		if (*_iRan==1 && *_jRan==0){*_index = _n; return;}
		_n++;
	}
	{
		if (*_iRan==1 && *_jRan==1){*_index = _n; return;}
		_n++;
	}
}

static double _dMainSigma;
void SetMainSigma(double* _sig){
	_dMainSigma = *_sig;
}

void NotOK(){
	if (_pbOK != 0L)(*_pbOK) = 0;
}
static double _tempResult[5];
static double _tempArg[8];
int _bTempValid = 0;
void InitTemp(){ int _i;
	for (_i = 0; _i < 5; _i++) _tempResult[_i] = -999999999.0;
	for (_i = 0; _i < 8; _i++) _tempArg[_i]    = -999999999.0;
	_bTempValid = 1;
}
void EvalSParms(double* zzY){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _ix0 = 0, _ix1 = 0;	// used for table statement
	double _xFrac = 0;
	if (!_bTempValid) InitTemp();
	EvalGroup(zzY);
	if (wt == _NA){
		SetErrorMsg("Covariate 'wt' not set");
		NotOK();
	}
	if (apgr == _NA){
		SetErrorMsg("Covariate 'apgr' not set");
		NotOK();
	}
	V = (((tvV * _pow_cached(wt, dVdwt, &_tempArg[0], &_tempArg[1], &_tempResult[0])) * _pow_cached(apgr, dVdapgr, &_tempArg[2], &_tempArg[3], &_tempResult[1])) * _exp_cached(nV, &_tempArg[4], &_tempResult[2]));
	if (_initial_V == _NA) _initial_V = V;
	Ke = ((tvKe * _pow_cached(wt, dKedwt, &_tempArg[5], &_tempArg[6], &_tempResult[3])) * _exp_cached(nKe, &_tempArg[7], &_tempResult[4]));
	if (_initial_Ke == _NA) _initial_Ke = Ke;
	EvalAssign(zzY);
}
void EvalGroup(double* zzY){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _ix0 = 0, _ix1 = 0;	// used for table statement
	double _xFrac = 0;
	if (!_bTempValid) InitTemp();
}
void EvalAssign(double* zzY){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _ix0 = 0, _ix1 = 0;	// used for table statement
	double _xFrac = 0;
	if (!_bTempValid) InitTemp();
	C = (A1 / V);
}
void CalcSecondary(){
	double* _fixef = _args[0];
};

void GetSecondary(int* _pn, double* _sec){
	CalcSecondary();
	*_pn = 0;
};

void SetModelTime(double _t){
	t = _t;
}
void AddIncrements(double zzY[], double _ta, double _tb){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
}
void Deriv(int* _pN, double* _pt, double zzY[], double zzR[]){
	int _i = 0;
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	t = *_pt;
	#if defined(USE_SETTIMECF)
		SetTimeOfGetValCF(*_pt);
	#endif
	#if defined(THERE_ARE_INTERPOLATED_COVARIATES)
		EvalSParms(zzY);
	#else
		EvalAssign(zzY);
	#endif
	zzR[0] = 0; // extra null equation
}
void GradientJacobian(int* pN, double* pt, double zzY[], int* pML, int* pMU, double* _gjacobian, int* pNRJ); // forward
void Jacobian(int* pN, double* pt, double zzY[], int* pML, int* pMU, double* _jacobian, int* pNRJ){
}
void ResetTime(){
	curTime = 0;
	curTimeLastDose = curTime;
	iCurWhichDose = 0;
	t = curTime;
}
void ResetActions(){
	zzNAction = 0;
}
void ResetInteg(){
	int i;
	memset(zzY, 0, sizeof(zzY));
	memset(zzIRate, 0, sizeof(zzIRate));
	zzY[0] = 1; // init dummy integrator for infusions
}
void ResetCF(){
	CF2Reset(&_cf_A1);
}
void ResetUrine(){
	int i;
	for (i=NINTEG_ALL - NURINE - NEVENT; i<NINTEG_ALL - NEVENT; i++) zzY[i] = 0;
}
void TDL4ResetEvent(){
	int i;
	for (i=NINTEG_ALL - NEVENT; i<NINTEG_ALL; i++) zzY[i] = 0;
}
void ResetGradient(){
//	int _i = NINTEG_ALL + 1, _n = sizeof(zzY)/sizeof(zzY[0]) - _i;
//	memset(&zzY[_i], 0, _n * sizeof(zzY[0]));
}
void Advance(int bGenGradients, int iODELevel, double *pTimeOuter, double _dt, int bTimeAdvances){
	#if defined(USE_SETTIMECF)
		SetTimeOfStateCF(*pTimeOuter);
	#endif
	CF2Advance(&_cf_A1, _dt);
	EvalSParms(zzY);
}
int GetNumCompartments(){
	return 2;
}
int GetCompartmentNumber(const char* _nm){
	int _i = -1;
	if (0);
	else if (_nm[0]=='A' && strcmp(_nm, "A1")==0) _i = 0;
	return _i;
}
void GetCompartmentName(int* _pi, char* _nm){
	_nm[0] = 0;
	if (0);
	else if (*_pi == 0) strcpy(_nm, "A1");
}
void ScheduleBolus1(int _mark, double _tRel, int _iCpt, double _amt, int _addl){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _tlag = 0;
	double _rate = 1, _dt = 1, _bio = 1;
	if (0){
	} else if (_iCpt==0){
		if (_addl) InsertAction(_tRel        , 0,   ZZMARK_ENDROW,  0, 0, 0, (void*)0L);		// trigger endrow at time dose requested
		           InsertAction(_tRel        , 0, ZZACTION_DOSETIME, 0, 0, 0, (void*)0L);		// mark dose requested time
		           InsertAction(_tRel + _tlag, 0, ZZACTION_BOLUS1, _iCpt, _amt * _bio, _NA, (void*)0L);		// bolus
		if (_mark) InsertAction(_tRel        , 0,   ZZMARK_BOLUS1, _iCpt, _amt, 0, (void*)0L);		// mark bolus requested time
	}
}
void ScheduleIV1(int _mark, double _tRel, int _iCpt, double _amt, double _rate, int _addl){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _tlag = 0, _bio = 1, _dt;
	if (_rate == 0) return;
	_dt = _amt / _rate;
	if (0){
	} else if (_iCpt==0){
		if (_addl) InsertAction(_tRel              , 0,   ZZMARK_ENDROW,   0, 0, 0, (void*)0L);		// trigger endrow at time dose requested
		           InsertAction(_tRel              , 0, ZZACTION_DOSETIME, 0, 0, 0, (void*)0L);		// mark dose time
		           InsertAction(_tRel + _tlag      , 0, ZZACTION_INFSTRT1, _iCpt, _rate * _bio, 0, (void*)0L);		// start/end inf
		if (_mark) InsertAction(_tRel              , 0,   ZZMARK_INFSTRT1, _iCpt, _amt, _rate, (void*)0L);		// start/end inf
		           InsertAction(_tRel + _tlag + _dt, 0, ZZACTION_INFEND1, _iCpt, _rate * _bio, 0, (void*)0L);
		if (_mark) InsertAction(_tRel         + _dt, 0,   ZZMARK_INFEND1, _iCpt, 0, 0, (void*)0L);
	}
}
void PerformBolus1(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv, double _rate){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _neq = 0;
	if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
	if (0){
	} else if (_iCpt==0){
		CF2AddVal1(&_cf_A1, _dv);
	}
}
void PerformInfStrt1(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _neq = 0;
	if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
	if (0){
	} else if (_iCpt==0){
		CF2AddRate1(&_cf_A1, _dv);
	}
}
void PerformInfEnd1(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _neq = 0;
	if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
	if (0){
	} else if (_iCpt==0){
		CF2AddRate1(&_cf_A1, - _dv);
	}
}
void ScheduleBolus2(int _mark, double _tRel, int _iCpt, double _amt, int _addl){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _tlag = 0;
	double _rate = 1, _dt = 1, _bio = 1;
	if (0){
	}
}
void ScheduleIV2(int _mark, double _tRel, int _iCpt, double _amt, double _rate, int _addl){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _tlag = 0, _bio = 1, _dt;
	if (_rate == 0) return;
	_dt = _amt / _rate;
	if (0){
	}
}
void PerformBolus2(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv, double _rate){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _neq = 0;
	if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
	if (0){
	}
}
void PerformInfStrt2(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _neq = 0;
	if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
	if (0){
	}
}
void PerformInfEnd2(double* _pt, double zzY[], double zzR[], int _iCpt, double _dv){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _neq = 0;
	if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
	if (0){
	}
}
void AddToCpt(double zzY[], int _iCpt, double _dv){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
	if (0){
	} else if (_iCpt==0){
		CF2AddVal1(&_cf_A1, _dv);
	}
}
int GetVarValue(const char* _nm, double* _pv){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	EvalSParms(zzY);
	if (0);
	else if (_nm[0]=='t' && strcmp(_nm, "t")==0) *_pv = t;
	else if (_nm[0]=='w' && strcmp(_nm, "wt")==0) *_pv = wt;
	else if (_nm[0]=='a' && strcmp(_nm, "apgr")==0) *_pv = apgr;
	else if (_nm[0]=='A' && strcmp(_nm, "A1")==0) *_pv = A1;
	else if (_nm[0]=='t' && strcmp(_nm, "tvV")==0) *_pv = tvV;
	else if (_nm[0]=='t' && strcmp(_nm, "tvKe")==0) *_pv = tvKe;
	else if (_nm[0]=='d' && strcmp(_nm, "dVdwt")==0) *_pv = dVdwt;
	else if (_nm[0]=='d' && strcmp(_nm, "dKedwt")==0) *_pv = dKedwt;
	else if (_nm[0]=='d' && strcmp(_nm, "dVdapgr")==0) *_pv = dVdapgr;
	else if (_nm[0]=='n' && strcmp(_nm, "nV")==0) *_pv = nV;
	else if (_nm[0]=='n' && strcmp(_nm, "nKe")==0) *_pv = nKe;
	else if (_nm[0]=='V' && strcmp(_nm, "V")==0) *_pv = V;
	else if (_nm[0]=='K' && strcmp(_nm, "Ke")==0) *_pv = Ke;
	else if (_nm[0]=='C' && strcmp(_nm, "C")==0) *_pv = C;
	else if (_nm[0]=='t' && strcmp(_nm, "t")==0) *_pv = t;
	else return 0;
	return 1;
}

void SetVarValue(const char* _nm, double* _pv){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	EvalSParms(zzY);
	if (0);
	else if (_nm[0]=='w' && strcmp(_nm, "wt")==0) wt = *_pv;
	else if (_nm[0]=='a' && strcmp(_nm, "apgr")==0) apgr = *_pv;
	else if (_nm[0]=='A' && strcmp(_nm, "A1")==0) ; // A1 = *_pv;
	else if (_nm[0]=='t' && strcmp(_nm, "tvV")==0) tvV = *_pv;
	else if (_nm[0]=='t' && strcmp(_nm, "tvKe")==0) tvKe = *_pv;
	else if (_nm[0]=='d' && strcmp(_nm, "dVdwt")==0) dVdwt = *_pv;
	else if (_nm[0]=='d' && strcmp(_nm, "dKedwt")==0) dKedwt = *_pv;
	else if (_nm[0]=='d' && strcmp(_nm, "dVdapgr")==0) dVdapgr = *_pv;
	else if (_nm[0]=='n' && strcmp(_nm, "nV")==0) nV = *_pv;
	else if (_nm[0]=='n' && strcmp(_nm, "nKe")==0) nKe = *_pv;
	else if (_nm[0]=='V' && strcmp(_nm, "V")==0) V = *_pv;
	else if (_nm[0]=='K' && strcmp(_nm, "Ke")==0) Ke = *_pv;
	else if (_nm[0]=='C' && strcmp(_nm, "C")==0) C = *_pv;
	else if (_nm[0]=='t' && strcmp(_nm, "t")==0) t = *_pv;
	EvalSParms(zzY);
}

void GetSaemCMatrixSize(int* nBaseEta, int* nExtraEta, int* nTheta){
	*nBaseEta = *nTheta = *nExtraEta = 0;

	(*nTheta)++; // tvV
	if (_ISENABLED(0)) (*nTheta)++; // dVdwt
	if (_ISENABLED(2)) (*nTheta)++; // dVdapgr
	(*nBaseEta)++; // nV

	(*nTheta)++; // tvKe
	if (_ISENABLED(1)) (*nTheta)++; // dKedwt
	(*nBaseEta)++; // nKe

}
void GetSaemCMatrix(double* daCMat){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _nBaseEta, _nExtraEta, _nTheta, _nEta;
	int _iEta = 0, _jTheta = 0;
	GetSaemCMatrixSize(&_nBaseEta, &_nExtraEta, &_nTheta);
	_nEta = _nBaseEta + _nExtraEta;
	// clear the C matrix
	memset(daCMat, 0, sizeof(double)*_nEta*_nTheta);

	// for structural parameter log_V
	daCMat[_iEta + _jTheta * _nEta] = 1; // eta nV, theta log_tvV
	_jTheta++;
	if (_ISENABLED(0)){
		daCMat[_iEta + _jTheta * _nEta] = log(wt); // dVdwt
		_jTheta++;
	}
	if (_ISENABLED(2)){
		daCMat[_iEta + _jTheta * _nEta] = log(apgr); // dVdapgr
		_jTheta++;
	}
	_iEta++;

	// for structural parameter log_Ke
	daCMat[_iEta + _jTheta * _nEta] = 1; // eta nKe, theta log_tvKe
	_jTheta++;
	if (_ISENABLED(1)){
		daCMat[_iEta + _jTheta * _nEta] = log(wt); // dKedwt
		_jTheta++;
	}
	_iEta++;

	// assert _iEta == _nEta
	// assert _jTheta == _nTheta
}
int GetCovariateIsInCMatrix(const char* name){
	int rval = 0;
	if (0);
	else if (strcmp(name, "wt")==0) rval = 1;
	else if (strcmp(name, "apgr")==0) rval = 1;
	return rval;
}
void GetSaemThetaInfo(int* iaCColToITheta, int* baCColIsLog){
	int _jCTheta = 0;
	int _nTheta = 0;
	int _iaIFixefErrToITheta[5+5];
	int _i;
	for (_i = 0; _i < 5+5; _i++) _iaIFixefErrToITheta[_i] = -1;
	GetFixefErrToThetaMap(_iaIFixefErrToITheta, &_nTheta);

	iaCColToITheta[_jCTheta] = _iaIFixefErrToITheta[0]; // tvV
	baCColIsLog[_jCTheta] = 1;
	_jCTheta++;
	if (_ISENABLED(0)){
		iaCColToITheta[_jCTheta] = _iaIFixefErrToITheta[2]; // dVdwt
		baCColIsLog[_jCTheta] = 0;
		_jCTheta++;
	}
	if (_ISENABLED(2)){
		iaCColToITheta[_jCTheta] = _iaIFixefErrToITheta[4]; // dVdapgr
		baCColIsLog[_jCTheta] = 0;
		_jCTheta++;
	}

	iaCColToITheta[_jCTheta] = _iaIFixefErrToITheta[1]; // tvKe
	baCColIsLog[_jCTheta] = 1;
	_jCTheta++;
	if (_ISENABLED(1)){
		iaCColToITheta[_jCTheta] = _iaIFixefErrToITheta[3]; // dKedwt
		baCColIsLog[_jCTheta] = 0;
		_jCTheta++;
	}

}
void GetSaemNaivePoolThetas(int* nNPTheta, int* iaTheta){
	int _jTheta = 0;

	*nNPTheta = _jTheta;
}
int CanUseSAEM(){
	return 1;
}
void DoEndObsActions(){
}
#define NINTEG_GRADIENT_DERIV 1


int GetNIntegGradientDeriv(){return NINTEG_GRADIENT_DERIV;}


void GradientDeriv(int* _pN, double* _pt, double zzY[], double zzR[]){
	int _i = 0;
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	t = *_pt;
	#if defined(USE_SETTIMECF)
		SetTimeOfGetValCF(*_pt);
	#endif
	#if defined(THERE_ARE_INTERPOLATED_COVARIATES)
		EvalSParms(zzY);
	#else
		EvalAssign(zzY);
	#endif
	zzR[0] = 0; // extra null equation
}
void GradientJacobian(int* pN, double* pt, double zzY[], int* pML, int* pMU, double* _gjacobian, int* pNRJ){
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _nrj = *pNRJ;
	memset(_gjacobian, 0, sizeof(double)*_nrj*_nrj);
	// rate terms for zzR[0]
}
void NewGetLL(int bGenGradients, double _dLL[], double _dLLdEta[], int* pnObs, double _t, const char* _nm, double _v1, int bBql){
	double _yval[12];
	double _probge[12];
	double _prob[12];
	double _probge1[12];
	double _prob1[12];
	double _dyde[12];
	double _dldy[12];
	double _beta, _power, _zprob;
	#define t (_t + 0)
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _ypred = 0, _var = 1, _sig = 1, _sqrtvf = 1, _std = 1;
	double _dLL1 = 0, _dLL2;
	double _dLLdEta1[2];
	double _dYdEta = 0, _dSdEta = 0, _dLLdY = 0, _temp = 0, _haz0 = 0, _haz1 = 0, _del = 1e-6;
//	double _dKdEta[2 * 2];
	int _i = 0;
	memset(_dLLdEta1, 0, sizeof(_dLLdEta1));
	memset(_yval, 0, sizeof(_yval));
	memset(_probge, 0, sizeof(_probge));
	memset(_prob, 0, sizeof(_prob));
	memset(_probge1, 0, sizeof(_probge1));
	memset(_prob1, 0, sizeof(_prob1));
	memset(_dyde, 0, sizeof(_dyde));
	memset(_dldy, 0, sizeof(_dldy));
	if (0){
	} else if (_nm[0]=='C' && strcmp(_nm, "CObs")==0){
		_std = _fixef[5];
		_ypred = C;
		if (!_finite(_ypred)){
			SetErrorMsg("Error (Model.c): non-finite _ypred");
		}
		_sqrtvf = 1.0;
		/* handle 0 observations and proportional models */
		if (_sqrtvf == 0 || (_sqrtvf < 1e-12 && _v1 == 0)){_ypred = 0; _sqrtvf = 1;}
		_sig = _sqrtvf;
		if (!_finite(_sig)){
			SetErrorMsg("Error (Model.c): non-finite _sig");
		}
		if (_sig == 0){
			SetErrorMsg("Error (Model.c): zero _sig");
		}
		// get base LL
		if (ALLOWLOGTRANSFORM && bEnableLogTransform){
			if (bBql){
				_dLL1 = lphi(logfl(_v1) - logfl(_ypred), 1 * _std); // add current LL to log(P)
			} else {
				_dLL1 = lnorm(logfl(_v1) - logfl(_ypred), 1 * _std); // add current LL to log(P)
			}
		} else {
			if (bBql){
				_dLL1 = lphi((_v1 - _ypred), _sig * _std); // add current LL to log(P)
			} else {
				_dLL1 = lnorm((_v1 - _ypred), _sig * _std); // add current LL to log(P)
			}
		}

		if (bGenGradients){

			if (ALLOWLOGTRANSFORM && bEnableLogTransform){
				_dYdEta = ( -((A1 * V)) / (V * V)) / _ypred; // Get dY / dnV
				_dSdEta = 0; // Get dS / dnV
			} else {
				_dYdEta = ( -((A1 * V)) / (V * V)); // Get dY / dnV
				_dSdEta = 0.0; // Get dS / dnV
			}
			if (ALLOWLOGTRANSFORM && bEnableLogTransform){
				if (bBql){
					_dLL2 = lphi(logfl(_v1) - logfl(_ypred + _dYdEta*_del), (1+_dSdEta*_del)*_std);
					_dLLdEta1[0] = (_dLL2 - _dLL1)/_del;
				} else {
					_dLLdEta1[0]  = dLNormdEta(logfl(_v1) - logfl(_ypred), 1*_std, -_dYdEta, _dSdEta*_std);
				}
			} else {
				if (bBql){
					_dLL2 = lphi((_v1 - (_ypred + _dYdEta*_del)), (_sig+_dSdEta*_del)*_std);
					_dLLdEta1[0] = (_dLL2 - _dLL1)/_del;
				} else {
					_dLLdEta1[0]  = dLNormdEta((_v1 - _ypred), _sig*_std, -_dYdEta, _dSdEta*_std);
				}
			}

			_VecAddTo(2, _dLLdEta1, _dLLdEta);
		}
		_dLL[0] += _dLL1;

		(*pnObs)++; // count all observations
	}
	#undef t
}

void NewGetPred(int bGenGradients, int* pnObs, double _t, const char* _nm, double _v1){
	double _dVFCorrectionFactor = 1;
	#define t (_t + 0)
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _ypred = 0; // predicted value
	double _var = 1;
	double _sig = 1; // current standard deviation of Epsilon
	double _std = 1; // current standard deviation of Epsilon
	double _sqrtvf = 1; // dObs / dEpsilon (additive = 1, proportional = _ypred)
	double _vf = 1; // _sqrtvf*_sqrtvf (additive = 1, proportional = _ypred^2)
	double _dYdEta[2];
	double _dSdEta[2];
	_dYdEta[0] = _dYdEta[1] = 0;
	_dSdEta[0] = _dSdEta[1] = 0;
	if (0){
	} else if (_nm[0]=='C' && strcmp(_nm, "CObs")==0){
		_std = _fixef[5];
		_ypred = C;
		if (!_finite(_ypred)){
			SetErrorMsg("Error (Model.c): non-finite _ypred");
		}
		_sqrtvf = sqrt(1.0);
		/* handle 0 observations and proportional models */
		if (_sqrtvf == 0 || (_sqrtvf < 1e-12 && _v1 == 0)){_ypred = 0; _sqrtvf = 1;}
		_sig = _sqrtvf;
		_dVFCorrectionFactor = GetPredVFCorrectionFactor(0);
		if (_dVFCorrectionFactor != 0) _sig /= _dVFCorrectionFactor;
		if (!_finite(_sig)){
			SetErrorMsg("Error (Model.c): non-finite _sig");
		}
		if (_sig == 0){
			SetErrorMsg("Error (Model.c): zero _sig");
		}
		_var = _sig * _sig;
		if (ALLOWLOGTRANSFORM && bEnableLogTransform){
			RecordObservation(t, curTimeLastDose, iCurWhichDose, iCurWhichReset, logfl(_ypred), logfl(_v1), 1, 0);
		} else {
			RecordObservation(t, curTimeLastDose, iCurWhichDose, iCurWhichReset, _ypred, _v1, _var, 0);
		}

		if (bGenGradients){
			_dYdEta[0] = ( -((A1 * V)) / (V * V)); // Get dY / dnV
			_dSdEta[0] = 0.0; // Get dS / dnV
			if (ALLOWLOGTRANSFORM && bEnableLogTransform){
				RecordObservationGradient(_dYdEta[0] / (_ypred + 1e-30), 0, 0);
				RecordObservationGradient(_dYdEta[1] / (_ypred + 1e-30), 0, 1);
			} else {
				RecordObservationGradient(_dYdEta[0], _dSdEta[0], 0);
				RecordObservationGradient(_dYdEta[1], _dSdEta[1], 1);
			}
		}
		(*pnObs)++; // count all observations

	}
	#undef t
}

void GetLLGradientEta(
	  int* _pnObs
	, const unsigned char* *ppb
	, double* _pdId1, double* _pdId2, double* _pdId3, double* _pdId4, double* _pdId5
	, double _dLL[]
	, double _dGrad[]
	)
{
	double _dEta = 1e-6;
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	int _nEta = 2;
	int _nObs = 0;
	double _dLLLocal[1];
	double _dGradLocal[1];
	memset(_dGrad, 0, sizeof(double)*2);
	_dLLLocal[0] = 0;
	memset(_dGradLocal, 0, sizeof(_dGradLocal));
	const unsigned char* pb0 = *ppb;
	*ppb = pb0;
	_nObs = 0;
	LLOneSubject1(TRUE, _nEta, &_nObs, ppb, _pdId1, _pdId2, _pdId3, _pdId4, _pdId5, _dLL, _dGrad, _args, _pbOK);
	_dEta = max(fabs(nKe), sqrt(GetEtaVariance5(1))) * 1e-6;
	nKe += _dEta;
	*ppb = pb0;
	_nObs = 0;
	_dLLLocal[0] = 0;
	LLOneSubject1(FALSE, _nEta, &_nObs, ppb, _pdId1, _pdId2, _pdId3, _pdId4, _pdId5, _dLLLocal, _dGradLocal, _args, _pbOK);
	_dGrad[1] = (_dLLLocal[0] - _dLL[0]) / _dEta; // dLLdnKe
	nKe -= _dEta;
}
void NewPredOneSubject(
	  int* _pnObs
	, const unsigned char* *ppb
	, double* _dId1, double* _dId2, double* _dId3, double* _dId4, double* _dId5
	, double** _args
	, int* _pbOK
	, int* _nPrediction
	, double* __daXPrediction
	, double* __taLastDose
	, int  * __iaWhichDose
	, int  * __iaWhichReset
	, double* __daYPrediction
	, double* __daYObservation
	, double* __daVarianceFunction
	, double* _dYdK // comes out as _daYdK(iEta, iObs)
	, double* _dSdK // comes out as _daSdK(iEta, iObs)
	, int* _iaWhichObs
	)
{
	int _nEta = 2;
	int _nObs = 0;
	double _dEta = 1e-6;
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	const unsigned char* pb0 = *ppb;
	int _i;
	double __daYPredictionLocal[1000];
	double __daYVarianceLocal[1000];
//	memset(__daYPredictionLocal, 0, sizeof(__daYPredictionLocal));
//	memset(__daYVarianceLocal, 0, sizeof(__daYVarianceLocal));
	NewPredOneSubject1(TRUE, _nEta, &_nObs, ppb, _dId1, _dId2, _dId3, _dId4, _dId5, _args, _pbOK, _nPrediction, __daXPrediction, __taLastDose, __iaWhichDose, __iaWhichReset, __daYPrediction, __daYObservation, __daVarianceFunction, _dYdK, _dSdK, _iaWhichObs);
	// for each eta handled the hard way, do it by finite difference
	_dEta = max(fabs(nKe), sqrt(GetEtaVariance5(1))) * 1e-6;
	nKe += _dEta;
	*ppb = pb0;
	*_nPrediction = _nObs = 0;
	NewPredOneSubject1(FALSE, _nEta, &_nObs, ppb, _dId1, _dId2, _dId3, _dId4, _dId5, _args, _pbOK, _nPrediction, __daXPrediction, __taLastDose, __iaWhichDose, __iaWhichReset, __daYPredictionLocal, __daYObservation, __daYVarianceLocal, _dYdK, _dSdK, _iaWhichObs);
	// for each observation
	for (_i = 0; _i < _nObs; _i++){
		// get prediction and sigma gradient
		double _dChangeY = __daYPredictionLocal[_i] - __daYPrediction[_i];
		double _dChangeS = sqrt(__daYVarianceLocal[_i]) - sqrt(__daVarianceFunction[_i]);
		_dYdK[1 + _i*_nEta] = _dChangeY / _dEta;
		_dSdK[1 + _i*_nEta] = _dChangeS / _dEta;
	}
	nKe -= _dEta;
}
// jacobian info: 2=can matexp, 1=can stiffjac, 0=no jac, -1=no odes
void GetJacobianInfo(int* bUsingSyntheticGradients, int* _piJacobianInfo){
	if (*bUsingSyntheticGradients){
		* _piJacobianInfo = -1;
	} else {
		* _piJacobianInfo = -1;
	}
}
void GetMaxODELevel(int* bUsingSyntheticGradients, int* _piMaxODELevel){
	if (*bUsingSyntheticGradients){
		* _piMaxODELevel = ODELEVEL_MATEXP;
	} else {
		* _piMaxODELevel = ODELEVEL_MATEXP;
	}
}
int GetNumObservations(){
	return 1;
}

int GetObservationNumber(const char* _nm){
	int _i = -1;
	if (0);
	else if (strcmp(_nm, "CObs")==0) _i = 0;
	return _i;
}

void GetObservationName(int* _pi, char* _nm){
	if (0);
	else if (*_pi == 0) strcpy(_nm, "CObs");
}

int GetObservationType(int* _pi){
	int _type = -1;
	if (0);
	else if (*_pi == 0) _type = 0;
	return _type;
}

void GetObservationBqlLimit(int* _pi, double* _bqllimit){
	*_bqllimit = 0;
	if (0);
}

void GetLL(int bGenGradients, double _dLL[], double _dGrad[], int* pnObs, double _t, const char* _nm, double _v1, int bBql){
	double _dLL1 = 0;
	double _probge[12];
	#define t (_t + 0)
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _ypred = 0, _sig = 1, _std = 1, _var = 1, _sqrtvf = 1;
	int _i = 0;
	if (0){
	} else if (_nm[0]=='C' && strcmp(_nm, "CObs")==0){
		_std = _fixef[5];
		_ypred = C;
		if (!_finite(_ypred)){
			SetErrorMsg("Error (Model.c): non-finite _ypred");
		}
		if (ALLOWLOGTRANSFORM && bEnableLogTransform){
			_sqrtvf = 1;
		} else {
			_sqrtvf = sqrt(1.0);
		}
		/* handle 0 observations and proportional models */
		if (_sqrtvf == 0 || (_sqrtvf < 1e-12 && _v1 == 0)){_ypred = 0; _sqrtvf = 1;}
		_sig = _sqrtvf;
		_sig *= _fixef[5];
		if (!_finite(_sig)){
			SetErrorMsg("Error (Model.c): non-finite _sig");
		}
		if (_sig == 0){
			SetErrorMsg("Error (Model.c): zero _sig");
		}
		_var = _sig * _sig;
		if (ALLOWLOGTRANSFORM && bEnableLogTransform){
			if (bBql){
				_dLL1 += lphi(logfl(_v1) - logfl(_ypred), _sig); // add current LL to log(P)
			}
			else {
				_dLL1 += lnorm(logfl(_v1) - logfl(_ypred), _sig); // add current LL to log(P)
			}
		} else {
			if (bBql){
				_dLL1 += lphi((_v1 - _ypred), _sig); // add current LL to log(P)
			} else {
				_dLL1 += lnorm((_v1 - _ypred), _sig); // add current LL to log(P)
			}
		}
		_dLL[0] += _dLL1;
		(*pnObs)++;
		if (_pcb){
			if (ALLOWLOGTRANSFORM && bEnableLogTransform){
				(*_pcb->_pfOnObserve)(curTime, curTimeLastDose, _nm, -1, logfl(_ypred), 0, logfl(_v1), _sig, _sqrtvf, bBql);
			} else {
				(*_pcb->_pfOnObserve)(curTime, curTimeLastDose, _nm, -1, _ypred, 0, _v1, _sig, _sqrtvf, bBql);
			}
		}
	}
	#undef t
}

void GetPred(int* pnObs, double _t, const char* _nm, double _v1){
	double _dVFCorrectionFactor = 1;
	double _probge[12];
	#define t (_t + 0)
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _ypred = 0, _sig = 1, _std = 1, _var = 1, _sqrtvf = 1;
	int _i = 0;
	if (0){
	} else if (_nm[0]=='C' && strcmp(_nm, "CObs")==0){
		_std = _fixef[5];
		_ypred = C;
		if (!_finite(_ypred)){
			SetErrorMsg("Error (Model.c): non-finite _ypred");
		}
		_sqrtvf = sqrt(1.0);
		/* handle 0 observations and proportional models */
		if (_sqrtvf == 0 || (_sqrtvf < 1e-12 && _v1 == 0)){_ypred = 0; _sqrtvf = 1;}
		_sig = _sqrtvf;
		_dVFCorrectionFactor = GetPredVFCorrectionFactor(0);
		if (_dVFCorrectionFactor != 0) _sig /= _dVFCorrectionFactor;
		if (!_finite(_sig)){
			SetErrorMsg("Error (Model.c): non-finite _sig");
		}
		if (_sig == 0){
			SetErrorMsg("Error (Model.c): zero _sig");
		}
		_var = _sig * _sig;
		if (ALLOWLOGTRANSFORM && bEnableLogTransform){
			RecordObservation(t, curTimeLastDose, iCurWhichDose, iCurWhichReset, logfl(_ypred), logfl(_v1), 1, 0);
		} else {
			RecordObservation(t, curTimeLastDose, iCurWhichDose, iCurWhichReset, _ypred, _v1, _var, 0);
		}
		(*pnObs)++;
	}
	#undef t
}

int GetPredObsError(int _iObs){
	int _iError = -1;
	if (0){
	} else if (_iObs == 0){
		_iError = 0;
	}
	return _iError;
}
double GetPredVFCorrectionFactor(int _iError){
	double* _fixef = _args[0];
	int _nFixef = 5;
	int _iLastFreeError = 0;
	int _bErrorFrozen = 0;
	double _vfCorrectionFactor = 1;
	if (0){} // code to detect if _iError is frozen
	if (!_bErrorFrozen && _iError != _iLastFreeError){
		double _dThisSigma = _fixef[_nFixef + _iError];
		if (_dThisSigma != 0){
			_vfCorrectionFactor = _dMainSigma / _dThisSigma;
		}
	}
	return _vfCorrectionFactor;
}
void GetPredName(int* _iWhichObs, char* _nm){
	_nm[0] = '\0';
	if (0);
	else if (*_iWhichObs == 0) strcpy(_nm, "C");
}
void GetObsName(int* _iWhichObs, char* _nm){
	_nm[0] = '\0';
	if (0);
	else if (*_iWhichObs == 0) strcpy(_nm, "CObs");
}
void GetIVarName(int* _iWhichObs, char* _nm){
	_nm[0] = '\0';
	if (0);
	else if (*_iWhichObs == 0) strcpy(_nm, "t");
}
void GetObsVF(int* _iWhichObs, double* _vf){
	#define t (_t + 0)
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	*_vf = 1;
	if (0);
	else if (*_iWhichObs == 0){
		double _std = _fixef[5];
		*_vf = 1.0;
	}
	#undef t
}
void GetNumGaussianObs(int* _nGaussianObs){
	(*_nGaussianObs) = 0;
	(*_nGaussianObs)++; /* CObs */
}
double _unifControlled(BOOL bSample){
	return ((bSample && _pcb) ? (*_pcb->_pfGetUniform)() : 0.5);
}
double _normControlled(BOOL bSample){
	return ((bSample && _pcb) ? (*_pcb->_pfGetNormal)() : 0.0);
}
void Simulate(double _t, const char* _nm, double _v1, int bBql, int bSampleEpsilon, int bForVPC){
	double _probge[12];
	#define t (_t + 0)
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _ypred = 0, _sig = 1, _std = 1, _var = 1, _sqrtvf = 1;
	int _i = 0;
	EvalSParms(zzY);
	if (0){
	} else if (_nm[0]=='C' && strcmp(_nm, "CObs")==0){
		_std = _fixef[5];
		_ypred = C;
		if (!_finite(_ypred)){
			SetErrorMsg("Error (Model.c): non-finite _ypred");
		}
		if (ALLOWLOGTRANSFORM && bEnableLogTransform){
			_sqrtvf = 1;
		} else {
			_sqrtvf = sqrt(1.0);
		}
		/* handle 0 observations and proportional models */
		if (_sqrtvf == 0 || (_sqrtvf < 1e-12 && _v1 == 0)){_ypred = 0; _sqrtvf = 1;}
		_sig = _sqrtvf;
		if (!_finite(_sig)){
			SetErrorMsg("Error (Model.c): non-finite _sig");
		}
		if (_sig == 0){
			SetErrorMsg("Error (Model.c): zero _sig");
		}
		_var = _sig * _sig;
		if (_pcb != 0){
			double _eps = (bSampleEpsilon ? (*_pcb->_pfGetNormal)() : 0); // get a N(0,1)
			double _ypred1 = 0;
			_eps *= _fixef[5];
			_ypred1 = (C + _eps);
			if (ALLOWLOGTRANSFORM && bEnableLogTransform){
				(*_pcb->_pfOnObserve)(curTime, curTimeLastDose, _nm, -1, logfl(_ypred), logfl(_ypred1), logfl(_v1), _sig, _sqrtvf, bBql);
			} else {
				(*_pcb->_pfOnObserve)(curTime, curTimeLastDose, _nm, -1, _ypred, _ypred1, _v1, _sig, _sqrtvf, bBql);
			}
		}
	}
	#undef t
}

void GetNumCategories(const char* _nm, int* pNCat){
	*pNCat = 0;
	if (0){
	}
	#undef t
}

void SimulateForSimTbl(double _t, const char* _nm, double* _pObs, double* _pPred){
	#define t (_t + 0)
	double* _fixef = _args[0];
	double* _ranef1 = _args[1];
	double* _ranef2 = _args[2];
	double* _ranef3 = _args[3];
	double* _ranef4 = _args[4];
	double* _ranef5 = _args[5];
	double _eps = 0;
	EvalSParms(zzY);
	if (_pcb == 0) return;
	if (0){
	} else if (_nm[0]=='C' && strcmp(_nm, "CObs")==0){
		double _std = 0;
		_std = _fixef[5];
		// MD 151026: Tentative change: Turn off log transformation for sim tables
		if (FALSE && ALLOWLOGTRANSFORM && bEnableLogTransform){
			*_pPred = logfl((C + _eps));
			_eps = (*_pcb->_pfGetNormal)() * _fixef[5]; // get a N(0,stdev)
			*_pObs = logfl((C + _eps));
		} else {
			*_pPred = (C + _eps);
			_eps = (*_pcb->_pfGetNormal)() * _fixef[5]; // get a N(0,stdev)
			*_pObs = (C + _eps);
		}
	}
	#undef t
}


