// MUtil.c

#include <math.h>
#include <float.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <memory.h>

#ifdef _WIN32
	int __stdcall DllMain(
		  unsigned int hModule
		, unsigned int ul_reason_for_call
		, void* lpReserved
		)
	{
		return 1;
	}
#else
	int DllMain(
		  unsigned int hModule
		, unsigned int ul_reason_for_call
		, void* lpReserved
		)
	{
		return 1;
	}
#endif

#define IN_MUTIL
#include "MUtil.h"

#include "ModelAPI.h"

#include "Model.h"

extern int GetCompartmentNumber(const char* name);

void SetDbgCallback(fDebug* _p){
	_dbg = (void*)_p;
}

void CallDebug(int i, double d){
	if (_dbg) (*(fDebug*)_dbg)(i, d);
}

void SetModelAnagrad(int b){
	bModelAnagrad = b;
}

void SetODELevel(int* _piODELevel){
	iODELevel = * _piODELevel;
}

// This do-nothing routine can be called from multiple places in the model and/or engine
// It is intended to contain a breakpoint which is disabled.
// The breakpoint can be manually enabled while the app is running so as to get a pseudo-random halt
// for the purpose of performance tuning.
void ModelBreakPoint(){
}

int GetNFixef(){return NFIXEF;}
int GetNSecondary(){return NSECONDARY;}
int GetNError(){return NERROR;}
int GetNRanef1(){return NRANEF1;}
int GetNRanef2(){return NRANEF2;}
int GetNRanef3(){return NRANEF3;}
int GetNRanef4(){return NRANEF4;}
int GetNRanef5(){return NRANEF5;}
int MultiLevelRanef(){return MULTI_LEVEL_RANEF;}
int AllowGaussianFit(){return ALLOWGAUSSIANFIT;}
int AllowLogTransform(){return ALLOWLOGTRANSFORM;}
void SetEnableLogTransform(int* _pbEnableLogTransform){
	bEnableLogTransform = * _pbEnableLogTransform;
}
int TrulyTimeBased(){return TRULYTIMEBASED;}
int ModelUniqueId(){return MODELUNIQUEID;}
void SetBreakTime(double* pdTime){
	double dTime = *pdTime;
	// priority 2 so it comes after other actions
	InsertAction(dTime - curTime, 2, ZZACTION_TIME, 0, 0, 0, NULL);
}

void ASSERT(BOOL b){
	if (!b){
		int breakhere = 1;
	}
}

double _TransitCorrectionFactor(double ntr){
#define NOLD 5
#define MAX_NTR 1000
	// cache up to NOLD old values
	static int iold = 0;
	static double old_ntr[NOLD];
	static double old_result[NOLD];
	double ntr0, f, temp, result = 1;
	// limit 0 to MAX_NTR
	ntr = (ntr < 0 ? 0 : ntr > MAX_NTR ? MAX_NTR : ntr);
	if (ntr == 0) return 1;
	else {
		// if the value is in the cache, return it
		int j;
		for (j = NOLD; --j >= 0;){
			if (ntr == old_ntr[j]) return old_result[j];
		}
	}
	ntr0 = floor(ntr);
	f = ntr - ntr0;
	temp = lgamm(ntr0 + 1) - lgamm(ntr + 1) + f * log(ntr0 + 1);
	result = exp(temp);
	// since had to calculate new value, add this value to the cache
	iold--; if (iold < 0) iold = NOLD-1;
	old_ntr[iold] = ntr;
	old_result[iold] = result;
	return result;
#undef MAX_NTR
#undef NOLD
}

double _interpolate(double x, double x0, double y0, double x1, double y1){
	double dx, dy, s;
	if (x <= x0) return y0;
	if (x >= x1) return y1;
	dx = x1 - x0;
	if (dx == 0) return y0;
	dy = y1 - y0;
	s = (x - x0)/dx;
	return y0 + s * dy;
}

double factorial(double x){
	double i, n;
	double y = 1;
	if (x < 0) x = -x;
	n = (int)(x + 0.5);
	for (i = 2; i <= n; i++){
		y *= i;
	}
	return y;
}

//double lnorm(double x, double s){
//	const double log2pi = 1.8378770664093454835606594728112;
//	double v = s * s;
//	double ll = -0.5 * (log2pi + log(v) + x*x/v);
//	return ll;
//}

double dLNormdEta(double x, double s, double dXdEta, double dSdEta){
	/***
	LL = -0.5 * (log2pi + log(s^2) + (x/s)^2)
	-2 dLL = d(log(s^2)) + d((x/s)^2)
	       = 1/(s^2)d(s^2) + 2(x/s)d(x/s)
		                           d(x/s) = (sdx - xds)/s^2 = (1/s dx - x/s^2 dx)
		   = 1/(s^2)2sds + 2(x/s)(1/s dx - x/s^2 dx)
		   = 1/(s^2)2sds + 2((x/s)/s dx - (x/s)x/s^2 dx)
    -dLL   = 1/(s^2)sds + (x/s)/s dx - (x/s)x/s^2 dx
	dLL    = -(1/s)ds - (x/s^2)dx + (x^2/s^3)ds
	***/
	double s2 = s * s;
	double s3 = s2 * s;
	double x2 = x * x;
	double result = 0 - 1/s*dSdEta - x/s2*dXdEta + x2/s3*dSdEta;
	return result;
}

double lgamm(double x);
double lpois(double mean, int x);

#if 0
	// MD 150410: Needed for rgamma. Got from here:
	// http://en.wikipedia.org/wiki/Gamma_distribution#Generating_gamma-distributed_random_variables
	double AhrensDieter(double del){ // 0 < del < 1
		double e = exp(1.0);
		double vv = e/(e+del);
		double chi = 0, eta = 0;
		double temp = 0;
		int m;
		for (m = 1; ; m++){
			double v2 = unif(), v1 = unif(), v0 = unif();
			if (vv >= v2){
				chi = pow(v1, 1/del);
				eta = v0 * pow(chi, del-1);
			} else {
				chi = 1 - log(v1);
				eta = v0 * exp(-chi);
			}
			temp = pow(chi, del-1)*exp(-chi);
			if (eta <= temp) break;
		}
		return chi;
	}

	// sample the gamma distribution with given shape
	double rgamma(double k, double scale){
		int i, kk = (int)floor(k);
		double del = k - kk;
		double sum, ad, result;
		if (k <= 0) return -1;
		for (i = 0, sum = 0; i < kk; i++){
			sum += -log(unif());
		}
		ad = 0;
		if (del > 0){
			ad = AhrensDieter(del);
		}
		sum += ad;
		result = sum * scale;
		return result;
	}

#else
	double unif_rand(){return unif();}
	double norm_rand(){return norm();}
	double exp_rand(){return -log(unif());}
	double fsign(double x, double y){double tempx = fabs(x); return (y >= 0 ? tempx : -tempx);}
//	double fmax(double x, double y){return (x < y) ? y : x;}
//	double fmin(double x, double y){return (x < y) ? x : y;}
	int imax2(int x, int y){return (x < y) ? y : x;}
	int imin2(int x, int y){return (x < y) ? x : y;}
//	double expm1(double x){return exp(x) - 1;}
	#define M_1_SQRT_2PI	0.398942280401432677939946059934	/* 1/sqrt(2pi) */
	#define ML_ERR_return_NAN return -1
	/*
	 *  Mathlib : A C Library of Special Functions
	 *  Copyright (C) 1998 Ross Ihaka
	 *  Copyright (C) 2000--2008 The R Core Team
	 *
	 *  This program is free software; you can redistribute it and/or modify
	 *  it under the terms of the GNU General Public License as published by
	 *  the Free Software Foundation; either version 2 of the License, or
	 *  (at your option) any later version.
	 *
	 *  This program is distributed in the hope that it will be useful,
	 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
	 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 *  GNU General Public License for more details.
	 *
	 *  You should have received a copy of the GNU General Public License
	 *  along with this program; if not, a copy is available at
	 *  http://www.r-project.org/Licenses/
	 *
	 *  SYNOPSIS
	 *
	 *    #include <Rmath.h>
	 *    double rgamma(double a, double scale);
	 *
	 *  DESCRIPTION
	 *
	 *    Random variates from the gamma distribution.
	 *
	 *  REFERENCES
	 *
	 *    [1] Shape parameter a >= 1.  Algorithm GD in:
	 *
	 *	  Ahrens, J.H. and Dieter, U. (1982).
	 *	  Generating gamma variates by a modified
	 *	  rejection technique.
	 *	  Comm. ACM, 25, 47-54.
	 *
	 *
	 *    [2] Shape parameter 0 < a < 1. Algorithm GS in:
	 *
	 *	  Ahrens, J.H. and Dieter, U. (1974).
	 *	  Computer methods for sampling from gamma, beta,
	 *	  poisson and binomial distributions.
	 *	  Computing, 12, 223-246.
	 *
	 *    Input: a = parameter (mean) of the standard gamma distribution.
	 *    Output: a variate from the gamma(a)-distribution
	 */

	//#include "nmath.h"

	#define R_FINITE _finite

	double rgamma(double a, double scale)
	{
	/* Constants : */
		const static double sqrt32 = 5.656854;
		const static double exp_m1 = 0.36787944117144232159;/* exp(-1) = 1/e */

		/* Coefficients q[k] - for q0 = sum(q[k]*a^(-k))
		 * Coefficients a[k] - for q = q0+(t*t/2)*sum(a[k]*v^k)
		 * Coefficients e[k] - for exp(q)-1 = sum(e[k]*q^k)
		 */
		const static double q1 = 0.04166669;
		const static double q2 = 0.02083148;
		const static double q3 = 0.00801191;
		const static double q4 = 0.00144121;
		const static double q5 = -7.388e-5;
		const static double q6 = 2.4511e-4;
		const static double q7 = 2.424e-4;

		const static double a1 = 0.3333333;
		const static double a2 = -0.250003;
		const static double a3 = 0.2000062;
		const static double a4 = -0.1662921;
		const static double a5 = 0.1423657;
		const static double a6 = -0.1367177;
		const static double a7 = 0.1233795;

		/* State variables [FIXME for threading!] :*/
		static double aa = 0.;
		static double aaa = 0.;
		static double s, s2, d;    /* no. 1 (step 1) */
		static double q0, b, si, c;/* no. 2 (step 4) */

		double e, p, q, r, t, u, v, w, x, ret_val;

		if (!R_FINITE(a) || !R_FINITE(scale) || a < 0.0 || scale <= 0.0) {
		if(scale == 0.) return 0.;
		ML_ERR_return_NAN;
		}

		if (a < 1.) { /* GS algorithm for parameters a < 1 */
		if(a == 0)
			return 0.;
		e = 1.0 + exp_m1 * a;
		for(;;) {
			p = e * unif_rand();
			if (p >= 1.0) {
			x = -log((e - p) / a);
			if (exp_rand() >= (1.0 - a) * log(x))
				break;
			} else {
			x = exp(log(p) / a);
			if (exp_rand() >= x)
				break;
			}
		}
		return scale * x;
		}

		/* --- a >= 1 : GD algorithm --- */

		/* Step 1: Recalculations of s2, s, d if a has changed */
		if (a != aa) {
		aa = a;
		s2 = a - 0.5;
		s = sqrt(s2);
		d = sqrt32 - s * 12.0;
		}
		/* Step 2: t = standard normal deviate,
				   x = (s,1/2) -normal deviate. */

		/* immediate acceptance (i) */
		t = norm_rand();
		x = s + 0.5 * t;
		ret_val = x * x;
		if (t >= 0.0)
		return scale * ret_val;

		/* Step 3: u = 0,1 - uniform sample. squeeze acceptance (s) */
		u = unif_rand();
		if (d * u <= t * t * t)
		return scale * ret_val;

		/* Step 4: recalculations of q0, b, si, c if necessary */

		if (a != aaa) {
		aaa = a;
		r = 1.0 / a;
		q0 = ((((((q7 * r + q6) * r + q5) * r + q4) * r + q3) * r
			   + q2) * r + q1) * r;

		/* Approximation depending on size of parameter a */
		/* The constants in the expressions for b, si and c */
		/* were established by numerical experiments */

		if (a <= 3.686) {
			b = 0.463 + s + 0.178 * s2;
			si = 1.235;
			c = 0.195 / s - 0.079 + 0.16 * s;
		} else if (a <= 13.022) {
			b = 1.654 + 0.0076 * s2;
			si = 1.68 / s + 0.275;
			c = 0.062 / s + 0.024;
		} else {
			b = 1.77;
			si = 0.75;
			c = 0.1515 / s;
		}
		}
		/* Step 5: no quotient test if x not positive */

		if (x > 0.0) {
		/* Step 6: calculation of v and quotient q */
		v = t / (s + s);
		if (fabs(v) <= 0.25)
			q = q0 + 0.5 * t * t * ((((((a7 * v + a6) * v + a5) * v + a4) * v
						  + a3) * v + a2) * v + a1) * v;
		else
			q = q0 - s * t + 0.25 * t * t + (s2 + s2) * log(1.0 + v);


		/* Step 7: quotient acceptance (q) */
		if (log(1.0 - u) <= q)
			return scale * ret_val;
		}

		for(;;) {
		/* Step 8: e = standard exponential deviate
		 *	u =  0,1 -uniform deviate
		 *	t = (b,si)-double exponential (laplace) sample */
		e = exp_rand();
		u = unif_rand();
		u = u + u - 1.0;
		if (u < 0.0)
			t = b - si * e;
		else
			t = b + si * e;
		/* Step	 9:  rejection if t < tau(1) = -0.71874483771719 */
		if (t >= -0.71874483771719) {
			/* Step 10:	 calculation of v and quotient q */
			v = t / (s + s);
			if (fabs(v) <= 0.25)
			q = q0 + 0.5 * t * t *
				((((((a7 * v + a6) * v + a5) * v + a4) * v + a3) * v
				  + a2) * v + a1) * v;
			else
			q = q0 - s * t + 0.25 * t * t + (s2 + s2) * log(1.0 + v);
			/* Step 11:	 hat acceptance (h) */
			/* (if q not positive go to step 8) */
			if (q > 0.0) {
			w = expm1(q);
			/*  ^^^^^ original code had approximation with rel.err < 2e-7 */
			/* if t is rejected sample again at step 8 */
			if (c * fabs(u) <= w * exp(e - 0.5 * t * t))
				break;
			}
		}
		} /* for(;;) .. until  `t' is accepted */
		x = s + 0.5 * t;
		return scale * x * x;
	}

#endif

double erlangCDF(double x, int intc){
	int i;
	double di = 0;
	double iFact = 1;
	double xPowI = 1;
	double sum = 0;
	for (i = 0; i < intc;){
		sum += xPowI/iFact;
		i++; di++;
		iFact *= di;
		xPowI *= x;
	}
	double expMinusX = exp(-x);
	double p = 1 - expMinusX * sum;
	return p;
}

double normCDF(double x, double mean, double std){
	double result = (1+erfunc((x-mean)/std))/2;
	return result;
}

// incomplete gamma via series
// old version
static void gserxxx(double* gammsr, double x, double c, double lgammc, double gammc){
	double sum = 0;
	double powxn = 1;
	double gamc1n = gammc * c;
	int n; double dcn = c;
	double term;
	for (n = 0; n < 100;){
		term = gammc/gamc1n * powxn;
		sum += term;
		if (term > -1e-12 && term < 1e-12) break;
		n++; dcn++;
		powxn *= x;
		gamc1n += dcn;
	}
	*gammsr = exp(log(x)*c - x) * sum;
}

// version copied from Numerical Recipes in C
static void gser(double* gammsr, double x, double c, double lgammc, double gammc){
	int n;
	double sum, del, cp;
	cp = c;
	del = sum = 1/c;
	for (n = 1; n < 100; n++){
		cp++;
		del *= x/cp;
		sum += del;
		if (del > -1e-12 && del < 1e-12){
			*gammsr = sum * exp(-x + c*log(x) - lgammc);
			return;
		}
	}
	// n too large
	*gammsr = 1e100;
}

// incomplete gamma via continued fraction
// Numerical Recipes in C, pp 174
static void gcf(double* gammcf, double x, double c, double lgammc, double gammc){
	double gold = 0, g, fac = 1, b1 = 1;
	double b0 = 0, anf, ana, an, a1, a0 = 1;
	double term;
	int n;
	if (x <= 0){*gammcf = 1; return;}
	a1 = x;
	for (n = 1; n < 1000; n++){
		an = n;
		ana = an - c;
		a0 = (a1 + a0*ana) * fac;
		b0 = (b1 + b0*ana) * fac;
		anf = an * fac;
		a1 = x*a0 + anf*a1;
		b1 = x*b0 + anf*b1;
		if (a1 == 0) continue;
		fac = 1/a1;
		g = b1*fac;
		term = (g-gold)/g;
		if (term < 1e-12 && term > -1e-12){
			*gammcf = exp(-x + c*log(x) - (lgammc)) * g;
			return;
		}
		gold = g;
	}
	ASSERT(FALSE);
}

// gammp function from Numerical Recipes in C, pp 172.
// Note I revised the gser function. The gser function on pp 173
// was giving incorrect results for non-integer a.
// The following was extensively tested against the pgamma function
// in R and agrees well with it.
// Parameter x is the quantile >= 0.
// Parameter c is the shape parameter > 0.
// Note the scale is assumed to be 1.
// To get a scale of 10, for example, pass in x/10.
// Following static variables are for memoizing the routine
static double __x = -1000, __c = -1000, __b = -1000, __lb = -1000, __lgammc = -1000, __gammc = -1000,  __result = -1000;
double gammCDF(double x, double c, double b, double lb, double lgammc, double gammc){
	double gamser, gammcf, result = 0;
	// if the arguments are the same as before, return prior result
	if (x == __x && c == __c && b == __b && lb == __lb && lgammc == __lgammc && gammc == __gammc){
		return __result;
	}
	int intc = (int)floor(c);
	ASSERT(c > 0);
	if (0){
	} if (x < 0){
		result = 0;
	} else if (x == 0 && c < 1){
		result = 0;
	//} else if (c >= 80){	// commented out because it's still not a very close match
	//	// treat it as a sum of normals with mean cb, variance cb^2
	//	result = normCDF(x, c*b, sqrt(c)*b);
	} else if (c == (double)intc && intc < 50){	// this blows up if c is big
		result = erlangCDF(x/b, intc);
	} else if (x/b < (c + 1)){
		gser(&gamser, x/b, c, lgammc, gammc);
		//result = gamser / gammc;
		result = gamser;
	} else {
		gcf( &gammcf, x/b, c, lgammc, gammc);
		result = 1 - gammcf;
	}
	// memoize the arguments and result
	__x = x;
	__c = c;
	__b = b;
	__lb = lb;
	__lgammc = lgammc;
	__gammc = gammc;
	__result = result;
	return result;
}

// the density function is G(x; b, c) = (x/b)^(c-1)*exp(-x/b)/(b*gamma(c))
// where c>0 is shape, b>0 = delt/c, and x goes from 0 to inf, mean = b*c = delt
// b passed in real and log form
double gammPDF(double x, double c, double b, double lb, double lgammc){
	double lx, lres = 0;
	if (x < 0) return 0;
	if (x == 0){
		if (c == 1) return 1/b;
		if (c > 1) return 0;
		// deal with x==0, c < 1 case
		x = 1e-8;	// try this // that doesn't help much either 170125
	}
	lx = log(x);
	lres += (lx - lb)*(c - 1);
	lres += -x / b;
	lres -= lb;
	lres -= lgammc;
	return exp(lres);
}

void testGammaDistr(){
	double m = 10;	// mean
	double c = 2.9999;	// shape
	double b = m / c;	// scale
	double lgammc = lgamm(c);
	double gammc = exp(lgammc);
	double lb = log(b);
	int iq;
	double d[20];
	double p[20];
	for (iq = 0; iq < 20; iq++){
		d[iq] = gammPDF((double)iq, c, b, lb, lgammc);
		p[iq] = gammCDF((double)iq, c, b, lb, lgammc, gammc);
	}
	// check d[] against R dgamma(seq(0,20), c, scale=b) // MD 161209: precise agreement
	// check p[] against R pgamma(seq(0,20), c, scale=b) //     "            "
}

//double lnegbin_rp(double r, double p, double y){
//	double q = 1 - p;
//	double lp = lgamm(r + y) - lgamm(r) - lgamm(y+1) + r*log(p) + y*log(q);
//	return lp;
//}
double lnegbin_rp(double r, double p, double y){
	double i;
	double q = 1 - p;
	double logr = log(r);
	double logp = log(p);
	double logq = log(q);
	double yy = r * logp;
	if (q < 1e-5){
		yy = 0;
		yy -= exp(logr + logq); //r * q;
		yy -= exp(logr + logq*2)/2; //r * q*q/2;
		yy -= exp(logr + logq*3)/3; //r * q*q*q/3;

//		yy = r * (-q - q*q/2 - q*q*q/3);
	}
	if (y == 0) return yy;
	yy += logr + logq; //log(r * q);
	if (y == 1) return yy;
	for (i = 2; i <= y; i++){
		yy += logq + log(1 + (r-1)/i);
	}
	return yy;
}
/***** negative binomial parameter conversions
convert r,p -> m,a
	m = r*q/p
	a = 1/r
convert m,a -> r,p
	r = 1/a
	p = 1/(1+a*m)

Reference: Statistical Distributions 2nd edition, Evans, Hastings, Peacock, pp 110-113
n ~ NB:r,p        where r > 0
n = number of failures before rth success in series of bernoulli trials with P(success) = p,
    and P(failure) = q = (1-p)
mean = r*q/p
var  = r*q/p/p

When parameterized by mean m, and overdispersion parameter a
var  = m*(1+a*m)  where a > 0

algebraic validation:

1) show that mean = m
mean = r*q/p
substitute r = 1/a, p = 1/(1+a*m)
mean = 1/a * (1 - 1/(1+a*m)) / (1/(1+a*m))
     = 1/a * (1 - 1/(1+a*m)) * (1+a*m)
     = 1/a * ((1+a*m)-1)/(1+a*m) * (1+a*m)
     = 1/a * ((1+a*m)-1)
     = 1/a * (1+a*m - 1)
     = 1/a * a*m
     = m

2) show that var = m*(1+a*m)
var = r*q/p/p
    = m/p
substitute p = 1/(1+a*m)
var = m/(1/(1+a*m))
    = m*(1+a*m)

Example: r=2, p=0.1
Convert: a = 0.5, m = 2*0.9/0.1 = 1.8/0.1 = 18
In r,p parameterization
mean = r*q/p = 18
var  = mean/p = 18/0.1 = 180
In m,a parameterization
mean = m = 18
var  = m*(1+a*m) = 18(1+0.5*18) = 18*(1+9) = 18*10 = 180

*****/
double lnegbin(double mean, double beta, double power, double y){
	double alpha = exp(beta * power);
	double r = 1/alpha;		// corrected code MD 150406
	double p = 1 / (1 + alpha*mean);
	double q = 1 - p;
	const double tol = 1e-11;
	if (q < tol || p < tol){
		return lpois(mean, (int)y);
	} else {
		double yy = lnegbin_rp(r, p, y);
		return yy;
	}
}
double pnegbin(double mean, double beta, double power, double y){
	return exp(lnegbin(mean, beta, power, y));
}

// number of failures to reach rth success (p = P(success))
// rejection method
// MD 150410: no longer use this routine. It only works if r is an int.
int rnegbin_rp_reject(double r, double p){
	int i, nltp = 0, ngtp = 0;
	for (i = 0; i < 1000000000; i++){
		double u = unif();
		if (u < p) nltp++;
		else ngtp++;
		if (nltp >= r) break;
	}
	return ngtp;
}
// adding geometric variables method
// MD 150410: Note this is only used if r is an int and is not large
int rnegbin_rp_geom(int r, double p){
	double logq = log(1-p);
	double sum = 0;
	int i;
	for (i = 0; i < r; i++){
		double g = log(unif())/logq - 1;
		sum += ceil(g);
	}
	return (int)sum;
}
// MD 150410: gamma-poisson method that works with fractional or integer r
// https://en.wikipedia.org/wiki/Negative_binomial_distribution#Gamma.E2.80.93Poisson_mixture
// (Although note the confusion there between p and (1-p).)
double rnegbin_rp_gamma_poisson(double r, double p){
	double k = r;
	double scale = (1-p)/p;
	double yy = rgamma(k, scale);
	double nn = rpois(yy);
	return nn;
}
int rnegbin_rp(double r, double p){
	// if r is integer and not too large, use geometric method
	if (r == floor(r) && r < 100){
		return rnegbin_rp_geom((int)floor(r), p);
	// else use gamma-poisson method
	} else {
		return (int)rnegbin_rp_gamma_poisson(r, p);
	}
}

int rnegbin(double mean, double beta, double power){ // beta is log(alpha), where alpha is dispersion parameter
	double alpha = exp(beta * power);
	double r = 1/alpha;		// corrected code MD 150406
	double p = 1 / (1 + alpha*mean);
	// note: variance = mean * (1 + alpha*mean)
	return rnegbin_rp(r, p);
}

int test_rnegbin(){
	double mean = 2.71;
	double beta = 1;
	double power = 1.0;
	double alpha = exp(beta*power);
	double r = 1/alpha;
	double p = 1/(1+alpha*mean);
	double mm = r*(1-p)/p;	// true mean (should = 2.71)
	double vv = mm/p;		// true variance;
	int i, maxn = 100*1000*1000;
	double sum = 0, sumsq = 0;
	double mhat = 0, vhat = 0;
	double merr = 0, verr = 0;
	for (i = 0; i < maxn; i++){
		double x = rnegbin(mean, beta, power);
		sum += x;
		sumsq += x*x;
	}
	mhat = sum/maxn;
	vhat = sumsq/maxn - mhat*mhat;
	merr = fabs(1 - mhat/mm);
	verr = fabs(1 - vhat/vv);
	if (merr < 1e-3 && verr < 1e-2){
		return 1;
	} else {
		return 0;
	}
}

double lpois(double mean, int x){
	// P:mean,x = mean^x * exp(-mean) / x!
	// log(P:mean,x) = x*log(mean) - mean - sum(i=1:x) log(i)
	int i;
	double tol = 1e-7;
	double logmean = log(mean < tol ? tol : mean);
	double logxfact = 0;
	for (i = 2; i <= x; i++) logxfact += log((double)i);
	return x * logmean - mean - logxfact;
}

double ppois(double mean, int x){
	// P:mean,x = mean^x * exp(-mean) / x!
	// log(P:mean,x) = x*log(mean) - mean - sum(i=1:x) log(i)
	return exp(lpois(mean, x));
}

#if 0
	int rpois(double lambda){
		int i;
		const int nUseNormal = 30;
		// if lambda large, approximate by normal with mean and variance lambda
		if (lambda >= nUseNormal){
			double r = norm() * sqrt(lambda) + lambda;
			i = (int)floor(r + 0.5);
		} else {
			double u = unif();
	#if 1
			double f = exp(-lambda);
			double sum = f;
			i = 0;
			while (i < nUseNormal+1){
				if (u < sum) break;
				i++;
				f *= lambda/i;
				sum += f;
			}
	#else
			double sum = 0, term = 1;
			double exp_lambda = exp(-lambda);
			for (i = 0; i < nUseNormal+1; i++, term *= lambda/i){
				sum += term;
				if (u < sum * exp_lambda) break;
			}
	#endif
		}
		return i;
	}

#else
	/*
	 *  Mathlib : A C Library of Special Functions
	 *  Copyright (C) 1998 Ross Ihaka
	 *  Copyright (C) 2000-2011 The R Core Team
	 *
	 *  This program is free software; you can redistribute it and/or modify
	 *  it under the terms of the GNU General Public License as published by
	 *  the Free Software Foundation; either version 2 of the License, or
	 *  (at your option) any later version.
	 *
	 *  This program is distributed in the hope that it will be useful,
	 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
	 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 *  GNU General Public License for more details.
	 *
	 *  You should have received a copy of the GNU General Public License
	 *  along with this program; if not, a copy is available at
	 *  http://www.r-project.org/Licenses/
	 *
	 *  SYNOPSIS
	 *
	 *    #include <Rmath.h>
	 *    double rpois(double lambda)
	 *
	 *  DESCRIPTION
	 *
	 *    Random variates from the Poisson distribution.
	 *
	 *  REFERENCE
	 *
	 *    Ahrens, J.H. and Dieter, U. (1982).
	 *    Computer generation of Poisson deviates
	 *    from modified normal distributions.
	 *    ACM Trans. Math. Software 8, 163-179.
	 */

	//#include "nmath.h"

	int rpois(double mu)
	{
		const static double a0 =	-0.5;
		const static double a1 =	 0.3333333;
		const static double a2 =	-0.2500068;
		const static double a3 =	 0.2000118;
		const static double a4 =	-0.1661269;
		const static double a5 =	 0.1421878;
		const static double a6 =	-0.1384794;
		const static double a7 =	 0.1250060;

		const static double one_7 =	0.1428571428571428571;
		const static double one_12 =	0.0833333333333333333;
		const static double one_24 =	0.0416666666666666667;

		/* Factorial Table (0:9)! */
		const static double fact[10] =
		{
		1., 1., 2., 6., 24., 120., 720., 5040., 40320., 362880.
		};

		/* These are static --- persistent between calls for same mu : */
		static int l, m;

		static double b1, b2, c, c0, c1, c2, c3;
		static double pp[36], p0, p, q, s, d, omega;
		static double big_l;/* integer "w/o overflow" */
		static double muprev = 0., muprev2 = 0.;/*, muold	 = 0.*/

		/* Local Vars  [initialize some for -Wall]: */
		double del, difmuk= 0., E= 0., fk= 0., fx, fy, g, px, py, t, u= 0., v, x;
		double pois = -1.;
		int k, kflag, big_mu, new_big_mu = FALSE;

		if (!R_FINITE(mu) || mu < 0)
		ML_ERR_return_NAN;

		if (mu <= 0.)
		return 0.;

		big_mu = mu >= 10.;
		if(big_mu)
		new_big_mu = FALSE;

		if (!(big_mu && mu == muprev)) {/* maybe compute new persistent par.s */

		if (big_mu) {
			new_big_mu = TRUE;
			/* Case A. (recalculation of s,d,l	because mu has changed):
			 * The poisson probabilities pk exceed the discrete normal
			 * probabilities fk whenever k >= m(mu).
			 */
			muprev = mu;
			s = sqrt(mu);
			d = 6. * mu * mu;
			big_l = floor(mu - 1.1484);
			/* = an upper bound to m(mu) for all mu >= 10.*/
		}
		else { /* Small mu ( < 10) -- not using normal approx. */

			/* Case B. (start new table and calculate p0 if necessary) */

			/*muprev = 0.;-* such that next time, mu != muprev ..*/
			if (mu != muprev) {
			muprev = mu;
			m = imax2(1, (int) mu);
			l = 0; /* pp[] is already ok up to pp[l] */
			q = p0 = p = exp(-mu);
			}

			for(;;) {
			/* Step U. uniform sample for inversion method */
			u = unif_rand();
			if (u <= p0)
				return 0;

			/* Step T. table comparison until the end pp[l] of the
			   pp-table of cumulative poisson probabilities
			   (0.458 > ~= pp[9](= 0.45792971447) for mu=10 ) */
			if (l != 0) {
				for (k = (u <= 0.458) ? 1 : imin2(l, m);  k <= l; k++)
				if (u <= pp[k])
					return k;
				if (l == 35) /* u > pp[35] */
				continue;
			}
			/* Step C. creation of new poisson
			   probabilities p[l..] and their cumulatives q =: pp[k] */
			l++;
			for (k = l; k <= 35; k++) {
				p *= mu / k;
				q += p;
				pp[k] = q;
				if (u <= q) {
				l = k;
				return k;
				}
			}
			l = 35;
			} /* end(for(;;)) */
		}/* mu < 10 */

		} /* end {initialize persistent vars} */

	/* Only if mu >= 10 : ----------------------- */

		/* Step N. normal sample */
		g = mu + s * norm_rand();/* norm_rand() ~ N(0,1), standard normal */

		if (g >= 0.) {
		pois = floor(g);
		/* Step I. immediate acceptance if pois is large enough */
		if (pois >= big_l)
			return (int)pois;
		/* Step S. squeeze acceptance */
		fk = pois;
		difmuk = mu - fk;
		u = unif_rand(); /* ~ U(0,1) - sample */
		if (d * u >= difmuk * difmuk * difmuk)
			return (int)pois;
		}

		/* Step P. preparations for steps Q and H.
		   (recalculations of parameters if necessary) */

		if (new_big_mu || mu != muprev2) {
			/* Careful! muprev2 is not always == muprev
		   because one might have exited in step I or S
		   */
			muprev2 = mu;
		omega = M_1_SQRT_2PI / s;
		/* The quantities b1, b2, c3, c2, c1, c0 are for the Hermite
		 * approximations to the discrete normal probabilities fk. */

		b1 = one_24 / mu;
		b2 = 0.3 * b1 * b1;
		c3 = one_7 * b1 * b2;
		c2 = b2 - 15. * c3;
		c1 = b1 - 6. * b2 + 45. * c3;
		c0 = 1. - b1 + 3. * b2 - 15. * c3;
		c = 0.1069 / mu; /* guarantees majorization by the 'hat'-function. */
		}

		if (g >= 0.) {
		/* 'Subroutine' F is called (kflag=0 for correct return) */
		kflag = 0;
		goto Step_F;
		}


		for(;;) {
		/* Step E. Exponential Sample */

		E = exp_rand();	/* ~ Exp(1) (standard exponential) */

		/*  sample t from the laplace 'hat'
			(if t <= -0.6744 then pk < fk for all mu >= 10.) */
		u = 2 * unif_rand() - 1.;
		t = 1.8 + fsign(E, u);
		if (t > -0.6744) {
			pois = floor(mu + s * t);
			fk = pois;
			difmuk = mu - fk;

			/* 'subroutine' F is called (kflag=1 for correct return) */
			kflag = 1;

		  Step_F: /* 'subroutine' F : calculation of px,py,fx,fy. */

			if (pois < 10) { /* use factorials from table fact[] */
			px = -mu;
			py = pow(mu, pois) / fact[(int)pois];
			}
			else {
			/* Case pois >= 10 uses polynomial approximation
			   a0-a7 for accuracy when advisable */
			del = one_12 / fk;
			del = del * (1. - 4.8 * del * del);
			v = difmuk / fk;
			if (fabs(v) <= 0.25)
				px = fk * v * v * (((((((a7 * v + a6) * v + a5) * v + a4) *
						  v + a3) * v + a2) * v + a1) * v + a0)
				- del;
			else /* |v| > 1/4 */
				px = fk * log(1. + v) - difmuk - del;
			py = M_1_SQRT_2PI / sqrt(fk);
			}
			x = (0.5 - difmuk) / s;
			x *= x;/* x^2 */
			fx = -0.5 * x;
			fy = omega * (((c3 * x + c2) * x + c1) * x + c0);
			if (kflag > 0) {
			/* Step H. Hat acceptance (E is repeated on rejection) */
			if (c * fabs(u) <= py * exp(px + E) - fy * exp(fx + E))
				break;
			} else
			/* Step Q. Quotient acceptance (rare case) */
			if (fy - u * fy <= py * exp(px - fx))
				break;
		}/* t > -.67.. */
		}
		return (int)pois;
	}

#endif

double phi(double x){	
	double cdf = iprobit(x);	
	return cdf;
}

double lphi(double x, double stdev){
	double dx = x / stdev;
	double phi = iprobit(dx);
	double ll = log(phi);
	return ll;
}

double ilogit(double x){
	double e;
	if (x < -20) x = -20;
	if (x > 20) x = 20;
	e = exp(x);
	return e / (1 + e);
}

double iloglog(double x){
	if (x < -10) x = -10;
	if (x > 10) x = 10;
	return exp(-exp(-x));
}

double icloglog(double x){
	return 1 - iloglog(-x);
}



double lambertw(const double z) {
  int i;
  const double eps = 4.0e-16;
  const double em1 = 0.3678794411714423215955237701614608; 
  double p, e, t, w;

  if (0.0 == z) return 0;
  if (z < -em1 + 1e-4) { // series near -em1 in sqrt(q)
    double q = z + em1;
    double r = sqrt(q);
    double q2 = q * q;
    double q3 = q2 * q;
    return 
     -1.0
     +2.331643981597124203363536062168 * r
     -1.812187885639363490240191647568 * q
     +1.936631114492359755363277457668 * r * q
     -2.353551201881614516821543561516 * q2
     +3.066858901050631912893148922704 * r * q2
     -4.175335600258177138854984177460 * q3
     +5.858023729874774148815053846119 * r * q3
     -8.401032217523977370984161688514 * q3 * q;  // error approx 1e-16
  }
  /* initial approx for iteration... */
  if (z < 1) { /* series near 0 */
    p = sqrt(2 * (2.7182818284590452353602874713526625 * z + 1));
    w = -1 + p * (1 + p * (-0.333333333333333333333 + p * 0.152777777777777777777777)); 
  } else {
    w = log(z); /* asymptotic */
  }
  if (z > 3) {
    w -= log(w); /* useful? */
  }
  for (i = 0; i < 10; i++) { /* Halley iteration */
    e = exp(w);
    t = w * e - z;
    p = w + 1;
    t /= e * p - 0.5 * (p + 1) * t / p; 
    w -= t;
    if (fabs(t) < eps * (1 + fabs(w))) return w; /* rel-abs error */
  }
  return 0;
}




//  log gamma function of real argument.
//      Algorithms and coefficient values from "Computation of Special
//      Functions", Zhang and Jin, John Wiley and Sons, 1996.
//
//  (C) 2003, C. Bond. All rights reserved.
//
//  Returns log(gamma) of real argument.
//  NOTE: Returns 1e308 if argument is 0 or negative.
//
double lgamm(double x){
    double x0,x2,xp,gl,gl0;
    int n,k;
    static double a[] = {
        8.333333333333333e-02,
       -2.777777777777778e-03,
        7.936507936507937e-04,
       -5.952380952380952e-04,
        8.417508417508418e-04,
       -1.917526917526918e-03,
        6.410256410256410e-03,
       -2.955065359477124e-02,
        1.796443723688307e-01,
       -1.39243221690590};
    
    x0 = x;
    if (x <= 0.0) return 1e308;
    else if ((x == 1.0) || (x == 2.0)) return 0.0;
    else if (x <= 7.0) {
        n = (int)(7-x);
        x0 = x+n;
    }
    x2 = 1.0/(x0*x0);
    xp = 2.0*3.14159265358979;
    gl0 = a[9];
    for (k=8;k>=0;k--) {
        gl0 = gl0*x2 + a[k];
    }
    gl = gl0/x0+0.5*log(xp)+(x0-0.5)*log(x0)-x0;
    if (x <= 7.0) {
        for (k=1;k<=n;k++) {
            gl -= log(x0-1.0);
            x0 -= 1.0;
        }
    }
    return gl;
}

double min(double x, double y){
	if (x < y) return x;
	return y;
}

double max(double x, double y){
	if (x > y) return x;
	return y;
}

// find x where A*exp(-ax) + B*exp(-bx) + C*exp(-cx) is maximum
double CalcTMax(double A, double a, double B, double b, double C, double c){
	double amax, x1, x0, d0, d1, xm, dm;
	// first derivative = -Aa*exp(-ax) - Bb*exp(-bx) -Cc*exp(-cx)
	// assume x >= 0 and largest of ax, bx, cx < 100
	#define DERIV(x) (-A*a*exp(-a*(x)) - B*b*exp(-b*(x)) -C*c*exp(-c*(x)))
	if (a <= 0 || b <= 0 || c <= 0) return 0;
	amax = a;
	if (amax < b) amax = b;
	if (amax < c) amax = c;
	x1 = 200 / amax;
	x0 = 0;
	d0 = DERIV(x0);
	d1 = DERIV(x1);

	if (d0 * d1 >= 0) return 0;
	xm = (x0 + x1)/2;
	dm = DERIV(xm);
	while(x1 - x0 > 1.0e-10){
		if (dm == 0) break;
		if (d1 * dm > 0){
			x1 = xm; d1 = dm;
		} else {
			x0 = xm; d0 = dm;
		}
		xm = (x0 + x1)/2;
		dm = DERIV(xm);
	}
	#undef  DERIV
	return xm;
}

// there's always somebody who asks why we don't have "ln", so here it is
double ln(double x){
	return log(x);
}
// version of log function that has a floor. used when log-transforming
double logfl(double x){
	if (x < _logFloor) x = _logFloor;
	return log(x);
}
void SetLogFloor(double* _lf){
	_logFloor = * _lf;
}
//double log10(double _x){	// conflicts with math.h
//	double _v = log(_x);
//	return _v * 0.43429448190325182765112891891661;
//}
double _exp_cached(double _x, double* _tempArg, double* _tempResult){
	if (_x == *_tempArg) return *_tempResult;
	return (*_tempResult = exp(*_tempArg = _x));
}
double _log_cached(double _x, double* _tempArg, double* _tempResult){
	if (_x == *_tempArg) return *_tempResult;
	return (*_tempResult = log(*_tempArg = _x));
}
double _ln_cached(double _x, double* _tempArg, double* _tempResult){
	return _log_cached(_x, _tempArg, _tempResult);
}
double _pow_cached(double _x, double _y, double* _tempArg1, double* _tempArg2, double* _tempResult){
	if (_x == *_tempArg1 && _y == *_tempArg2) return *_tempResult;
	if (_y == 0) return 1;
	if (_x == 0) return 0;
	return (*_tempResult = pow(*_tempArg1 = _x, *_tempArg2 = _y));
}

// routines for generating units strings
#define MAX_BUFFER 40
char _buffer[MAX_BUFFER][1024];
int _iWhichBuffer = 0;
char* _GetBuffer(){
	char* _p = _buffer[_iWhichBuffer];
	_iWhichBuffer++;
	if (_iWhichBuffer >= MAX_BUFFER) _iWhichBuffer = 0;
	_p[0] = '\0';
	return _p;
}
char* __NumberString(double _d){
	char* _buf = _GetBuffer();
	sprintf(_buf, "%g", _d);
	return _buf;
}
char* _units_Mul(const char* a, const char* b){
	char* _buf = _GetBuffer();
	if (*a && *b) sprintf(_buf, "%s*%s", a, b);
	return _buf;
}
char* _units_Div(const char* a, const char* b){
	char* _buf = _GetBuffer();
	if (*a && *b) sprintf(_buf, "%s/(%s)", a, b);
	return _buf;
}
char* _units_Exp(const char* a){
	char* _buf = _GetBuffer();
	if (*a) sprintf(_buf, "exp(%s)", a);
	return _buf;
}
char* _units_Log(const char* a){
	char* _buf = _GetBuffer();
	if (*a) sprintf(_buf, "log(%s)", a);
	return _buf;
}

//double abs(double x){	// conflicts with math.h
//	return (x < 0 ? -x : x);
//}

double vfwt(double f, double p){
	return pow(max(0, f), p/2);
}

double erfunc(double x){
	double c = .564189583547756e0;

	double a[] = {
	0.0,
	.771058495001320e-04,
	-.133733772997339e-02,
	.323076579225834e-01,
	.479137145607681e-01,
	.128379167095513e+00,
	};

	double b[] = {
	0.0,
	.301048631703895e-02,
	.538971687740286e-01,
	.375795757275549e+00,
	};

	double p[] = {
	0.0,
	-1.36864857382717e-07,
	5.64195517478974e-01,
	7.21175825088309e+00,
	4.31622272220567e+01,
	1.52989285046940e+02,
	3.39320816734344e+02,
	4.51918953711873e+02,
	3.00459261020162e+02,
	};

	double q[] = {
	0.0,
	1.00000000000000e+00,
	1.27827273196294e+01,
	7.70001529352295e+01,
	2.77585444743988e+02,
	6.38980264465631e+02,
	9.31354094850610e+02,
	7.90950925327898e+02,
	3.00459260956983e+02,
	};

	double r[] = {
	0.0,
	2.10144126479064e+00,
	2.62370141675169e+01,
	2.13688200555087e+01,
	4.65807828718470e+00,
	2.82094791773523e-01,
	};

	double s[] = {
	0.0,
	9.41537750555460e+01,
	1.87114811799590e+02,
	9.90191814623914e+01,
	1.80124575948747e+01,
	};

	double ax = fabs(x), rval;

	if (0){
	} else if (ax <= 0.5){
		double t = x*x;
		double top = ((((a[1]*t+a[2])*t+a[3])*t+a[4])*t+a[5]) + 1;
		double bot = ((b[1]*t+b[2])*t+b[3])*t + 1;
		rval = x* (top/bot);

	} else if (ax <= 4){
		double top = ((((((p[1]*ax+p[2])*ax+p[3])*ax+p[4])*ax+p[5])*ax+p[6])*ax+p[7])*ax + p[8];
		double bot = ((((((q[1]*ax+q[2])*ax+q[3])*ax+q[4])*ax+q[5])*ax+q[6])*ax+q[7])*ax + q[8];
		rval = 0.5 + (0.5-exp(-x*x)*top/bot);
		if (x < 0) rval = -rval;

	} else if (ax < 5.8){
		double x2 = x*x;
		double t = 1/x2;
		double top = (((r[1]*t+r[2])*t+r[3])*t+r[4])*t + r[5];
		double bot = (((s[1]*t+s[2])*t+s[3])*t+s[4])*t + 1;
		rval = (c-top/ (x2*bot))/ax;
		rval = 0.5 + (0.5-exp(-x2)*rval);
		if (x < 0) rval = -rval;

	} else {
		// we don't want to output pure +/- 1 because that could cause iprobit to generate 0
		rval = (1 - 2.2204460492503e-016); // this is erfunc(5.8 - 1e-10)
		if (x < 0) rval = -rval;
	}

	return rval;
}

double iprobit(double x){
	return (1 + erfunc(x/1.4142135623730950488016887242097)) / 2;
}

int unifToPoisson(double mean, double r){
	int x = 0;
	if (mean > 0 && mean < 100){
		double fx = exp(-mean);
		double series = fx;
		while (r >= series && x < 200){
			x++;
			fx = mean * fx / (double)x;
			series += fx;
		}
	}
	return x;
}

double vdis(int n, double a[], double b[]){
	double sum = 0, del;
	int i;
	for (i=0; i<n; i++){
		del = a[i]-b[i];
		sum += del*del;
	}
	return sqrt(sum);
}

static void _SqMatMulTo(int na, double* a, double* b, double* c){
#define A(i,j,n) a[(i)+(j)*n]
#define B(i,j,n) b[(i)+(j)*n]
#define C(i,j,n) c[(i)+(j)*n]
	if (0){
	} else if (na==1){
		C(0,0,1) = A(0,0,1) * B(0,0,1);
	} else if (na==2){
		C(0,0,2) = A(0,0,2) * B(0,0,2)
				+  A(0,1,2) * B(1,0,2)
				;
		C(0,1,2) = A(0,0,2) * B(0,1,2)
				+  A(0,1,2) * B(1,1,2)
				;

		C(1,0,2) = A(1,0,2) * B(0,0,2)
				+  A(1,1,2) * B(1,0,2)
				;
		C(1,1,2) = A(1,0,2) * B(0,1,2)
				+  A(1,1,2) * B(1,1,2)
				;

	} else if (na==3){
		C(0,0,3) = A(0,0,3) * B(0,0,3)
				+  A(0,1,3) * B(1,0,3)
				+  A(0,2,3) * B(2,0,3)
				;
		C(0,1,3) = A(0,0,3) * B(0,1,3)
				+  A(0,1,3) * B(1,1,3)
				+  A(0,2,3) * B(2,1,3)
				;
		C(0,2,3) = A(0,0,3) * B(0,2,3)
				+  A(0,1,3) * B(1,2,3)
				+  A(0,2,3) * B(2,2,3)
				;

		C(1,0,3) = A(1,0,3) * B(0,0,3)
				+  A(1,1,3) * B(1,0,3)
				+  A(1,2,3) * B(2,0,3)
				;
		C(1,1,3) = A(1,0,3) * B(0,1,3)
				+  A(1,1,3) * B(1,1,3)
				+  A(1,2,3) * B(2,1,3)
				;
		C(1,2,3) = A(1,0,3) * B(0,2,3)
				+  A(1,1,3) * B(1,2,3)
				+  A(1,2,3) * B(2,2,3)
				;

		C(2,0,3) = A(2,0,3) * B(0,0,3)
				+  A(2,1,3) * B(1,0,3)
				+  A(2,2,3) * B(2,0,3)
				;
		C(2,1,3) = A(2,0,3) * B(0,1,3)
				+  A(2,1,3) * B(1,1,3)
				+  A(2,2,3) * B(2,1,3)
				;
		C(2,2,3) = A(2,0,3) * B(0,2,3)
				+  A(2,1,3) * B(1,2,3)
				+  A(2,2,3) * B(2,2,3)
				;

	} else if (na==4){
		C(0,0,4) = A(0,0,4) * B(0,0,4)
				+  A(0,1,4) * B(1,0,4)
				+  A(0,2,4) * B(2,0,4)
				+  A(0,3,4) * B(3,0,4)
				;
		C(0,1,4) = A(0,0,4) * B(0,1,4)
				+  A(0,1,4) * B(1,1,4)
				+  A(0,2,4) * B(2,1,4)
				+  A(0,3,4) * B(3,1,4)
				;
		C(0,2,4) = A(0,0,4) * B(0,2,4)
				+  A(0,1,4) * B(1,2,4)
				+  A(0,2,4) * B(2,2,4)
				+  A(0,3,4) * B(3,2,4)
				;
		C(0,3,4) = A(0,0,4) * B(0,3,4)
				+  A(0,1,4) * B(1,3,4)
				+  A(0,2,4) * B(2,3,4)
				+  A(0,3,4) * B(3,3,4)
				;

		C(1,0,4) = A(1,0,4) * B(0,0,4)
				+  A(1,1,4) * B(1,0,4)
				+  A(1,2,4) * B(2,0,4)
				+  A(1,3,4) * B(3,0,4)
				;
		C(1,1,4) = A(1,0,4) * B(0,1,4)
				+  A(1,1,4) * B(1,1,4)
				+  A(1,2,4) * B(2,1,4)
				+  A(1,3,4) * B(3,1,4)
				;
		C(1,2,4) = A(1,0,4) * B(0,2,4)
				+  A(1,1,4) * B(1,2,4)
				+  A(1,2,4) * B(2,2,4)
				+  A(1,3,4) * B(3,2,4)
				;
		C(1,3,4) = A(1,0,4) * B(0,3,4)
				+  A(1,1,4) * B(1,3,4)
				+  A(1,2,4) * B(2,3,4)
				+  A(1,3,4) * B(3,3,4)
				;

		C(2,0,4) = A(2,0,4) * B(0,0,4)
				+  A(2,1,4) * B(1,0,4)
				+  A(2,2,4) * B(2,0,4)
				+  A(2,3,4) * B(3,0,4)
				;
		C(2,1,4) = A(2,0,4) * B(0,1,4)
				+  A(2,1,4) * B(1,1,4)
				+  A(2,2,4) * B(2,1,4)
				+  A(2,3,4) * B(3,1,4)
				;
		C(2,2,4) = A(2,0,4) * B(0,2,4)
				+  A(2,1,4) * B(1,2,4)
				+  A(2,2,4) * B(2,2,4)
				+  A(2,3,4) * B(3,2,4)
				;
		C(2,3,4) = A(2,0,4) * B(0,3,4)
				+  A(2,1,4) * B(1,3,4)
				+  A(2,2,4) * B(2,3,4)
				+  A(2,3,4) * B(3,3,4)
				;

		C(3,0,4) = A(3,0,4) * B(0,0,4)
				+  A(3,1,4) * B(1,0,4)
				+  A(3,2,4) * B(2,0,4)
				+  A(3,3,4) * B(3,0,4)
				;
		C(3,1,4) = A(3,0,4) * B(0,1,4)
				+  A(3,1,4) * B(1,1,4)
				+  A(3,2,4) * B(2,1,4)
				+  A(3,3,4) * B(3,1,4)
				;
		C(3,2,4) = A(3,0,4) * B(0,2,4)
				+  A(3,1,4) * B(1,2,4)
				+  A(3,2,4) * B(2,2,4)
				+  A(3,3,4) * B(3,2,4)
				;
		C(3,3,4) = A(3,0,4) * B(0,3,4)
				+  A(3,1,4) * B(1,3,4)
				+  A(3,2,4) * B(2,3,4)
				+  A(3,3,4) * B(3,3,4)
				;

	} else {
		int i, j, k;
		for (i=0; i<na; i++){
			for (j=0; j<na; j++){
				double sum = 0;
				for (k = 0; k < na; k++){
					sum += A(i,k,na) * B(k,j,na);
				}
				C(i,j,na) = sum;
			}
		}
	}
#undef  A
#undef  B
#undef  C
}

static void SqMatMulTransposeTo(int na, double* a, double* b, double* c){
#define A(i,j,n) a[(i)+(j)*n]
#define B(i,j,n) b[(j)+(i)*n]
#define C(i,j,n) c[(i)+(j)*n]
	if (0){
	} else if (na==1){
		C(0,0,1) = A(0,0,1) * B(0,0,1);
	} else if (na==2){
		C(0,0,2) = A(0,0,2) * B(0,0,2)
				+  A(0,1,2) * B(1,0,2)
				;
		C(0,1,2) = A(0,0,2) * B(0,1,2)
				+  A(0,1,2) * B(1,1,2)
				;

		C(1,0,2) = A(1,0,2) * B(0,0,2)
				+  A(1,1,2) * B(1,0,2)
				;
		C(1,1,2) = A(1,0,2) * B(0,1,2)
				+  A(1,1,2) * B(1,1,2)
				;

	} else if (na==3){
		C(0,0,3) = A(0,0,3) * B(0,0,3)
				+  A(0,1,3) * B(1,0,3)
				+  A(0,2,3) * B(2,0,3)
				;
		C(0,1,3) = A(0,0,3) * B(0,1,3)
				+  A(0,1,3) * B(1,1,3)
				+  A(0,2,3) * B(2,1,3)
				;
		C(0,2,3) = A(0,0,3) * B(0,2,3)
				+  A(0,1,3) * B(1,2,3)
				+  A(0,2,3) * B(2,2,3)
				;

		C(1,0,3) = A(1,0,3) * B(0,0,3)
				+  A(1,1,3) * B(1,0,3)
				+  A(1,2,3) * B(2,0,3)
				;
		C(1,1,3) = A(1,0,3) * B(0,1,3)
				+  A(1,1,3) * B(1,1,3)
				+  A(1,2,3) * B(2,1,3)
				;
		C(1,2,3) = A(1,0,3) * B(0,2,3)
				+  A(1,1,3) * B(1,2,3)
				+  A(1,2,3) * B(2,2,3)
				;

		C(2,0,3) = A(2,0,3) * B(0,0,3)
				+  A(2,1,3) * B(1,0,3)
				+  A(2,2,3) * B(2,0,3)
				;
		C(2,1,3) = A(2,0,3) * B(0,1,3)
				+  A(2,1,3) * B(1,1,3)
				+  A(2,2,3) * B(2,1,3)
				;
		C(2,2,3) = A(2,0,3) * B(0,2,3)
				+  A(2,1,3) * B(1,2,3)
				+  A(2,2,3) * B(2,2,3)
				;

	} else if (na==4){
		C(0,0,4) = A(0,0,4) * B(0,0,4)
				+  A(0,1,4) * B(1,0,4)
				+  A(0,2,4) * B(2,0,4)
				+  A(0,3,4) * B(3,0,4)
				;
		C(0,1,4) = A(0,0,4) * B(0,1,4)
				+  A(0,1,4) * B(1,1,4)
				+  A(0,2,4) * B(2,1,4)
				+  A(0,3,4) * B(3,1,4)
				;
		C(0,2,4) = A(0,0,4) * B(0,2,4)
				+  A(0,1,4) * B(1,2,4)
				+  A(0,2,4) * B(2,2,4)
				+  A(0,3,4) * B(3,2,4)
				;
		C(0,3,4) = A(0,0,4) * B(0,3,4)
				+  A(0,1,4) * B(1,3,4)
				+  A(0,2,4) * B(2,3,4)
				+  A(0,3,4) * B(3,3,4)
				;

		C(1,0,4) = A(1,0,4) * B(0,0,4)
				+  A(1,1,4) * B(1,0,4)
				+  A(1,2,4) * B(2,0,4)
				+  A(1,3,4) * B(3,0,4)
				;
		C(1,1,4) = A(1,0,4) * B(0,1,4)
				+  A(1,1,4) * B(1,1,4)
				+  A(1,2,4) * B(2,1,4)
				+  A(1,3,4) * B(3,1,4)
				;
		C(1,2,4) = A(1,0,4) * B(0,2,4)
				+  A(1,1,4) * B(1,2,4)
				+  A(1,2,4) * B(2,2,4)
				+  A(1,3,4) * B(3,2,4)
				;
		C(1,3,4) = A(1,0,4) * B(0,3,4)
				+  A(1,1,4) * B(1,3,4)
				+  A(1,2,4) * B(2,3,4)
				+  A(1,3,4) * B(3,3,4)
				;

		C(2,0,4) = A(2,0,4) * B(0,0,4)
				+  A(2,1,4) * B(1,0,4)
				+  A(2,2,4) * B(2,0,4)
				+  A(2,3,4) * B(3,0,4)
				;
		C(2,1,4) = A(2,0,4) * B(0,1,4)
				+  A(2,1,4) * B(1,1,4)
				+  A(2,2,4) * B(2,1,4)
				+  A(2,3,4) * B(3,1,4)
				;
		C(2,2,4) = A(2,0,4) * B(0,2,4)
				+  A(2,1,4) * B(1,2,4)
				+  A(2,2,4) * B(2,2,4)
				+  A(2,3,4) * B(3,2,4)
				;
		C(2,3,4) = A(2,0,4) * B(0,3,4)
				+  A(2,1,4) * B(1,3,4)
				+  A(2,2,4) * B(2,3,4)
				+  A(2,3,4) * B(3,3,4)
				;

		C(3,0,4) = A(3,0,4) * B(0,0,4)
				+  A(3,1,4) * B(1,0,4)
				+  A(3,2,4) * B(2,0,4)
				+  A(3,3,4) * B(3,0,4)
				;
		C(3,1,4) = A(3,0,4) * B(0,1,4)
				+  A(3,1,4) * B(1,1,4)
				+  A(3,2,4) * B(2,1,4)
				+  A(3,3,4) * B(3,1,4)
				;
		C(3,2,4) = A(3,0,4) * B(0,2,4)
				+  A(3,1,4) * B(1,2,4)
				+  A(3,2,4) * B(2,2,4)
				+  A(3,3,4) * B(3,2,4)
				;
		C(3,3,4) = A(3,0,4) * B(0,3,4)
				+  A(3,1,4) * B(1,3,4)
				+  A(3,2,4) * B(2,3,4)
				+  A(3,3,4) * B(3,3,4)
				;

	} else {
		int i, j, k;
		for (i=0; i<na; i++){
			for (j=0; j<na; j++){
				double sum = 0;
				for (k = 0; k < na; k++){
					sum += A(i,k,na) * B(k,j,na);
				}
				C(i,j,na) = sum;
			}
		}
	}
#undef  A
#undef  B
#undef  C
}

void _VecAddTo(int n, double* a, double* b){
	int i;
	for (i = 0; i < n; i++){*b += *a; b++; a++;}
}

void _MatMulTo(int na, int ma, double* a, int mb, double* b, double* c){
	int nb = ma;
	int nc = na;
	int mc = mb;
#define A(i,j) a[(i)+(j)*na]
#define B(i,j) b[(i)+(j)*nb]
#define C(i,j) c[(i)+(j)*nc]
	int i, j, k;

	if (na==ma && na==mb){
		_SqMatMulTo(na, a, b, c);
		return;
	}

	for (i=0; i<na; i++)
	for (j=0; j<mb; j++)
	{
		double sum = 0;

#if 1
		// hopefully faster code
		double* aa = & A(i, 0);
		double* bb = & B(0, j);
		k = ma;
		while(k >= 4){
			sum += *aa * *bb; aa += na; bb++;
			sum += *aa * *bb; aa += na; bb++;
			sum += *aa * *bb; aa += na; bb++;
			sum += *aa * *bb; aa += na; bb++;
			k -= 4;
		}
		if (k >= 2){
			sum += *aa * *bb; aa += na; bb++;
			sum += *aa * *bb; aa += na; bb++;
			k -= 2;
		}
		if (k >= 1){
			sum += *aa * *bb; aa += na; bb++;
			k -= 1;
		}
#else
		for (k=0; k<ma; k++) sum += A(i,k) * B(k,j);
#endif
		C(i,j) = sum;
	}
#undef  A
#undef  B
#undef  C
}

static void _MatZero(int na, int ma, double* a){
	int len = na * ma;
	if (0){
	} else if (len==1){
		a[0] = 0;
	} else if (len==4){
		a[0] = 0;
		a[1] = 0;
		a[2] = 0;
		a[3] = 0;
	} else if (len==9){
		a[0] = 0;
		a[1] = 0;
		a[2] = 0;
		a[3] = 0;
		a[4] = 0;
		a[5] = 0;
		a[6] = 0;
		a[7] = 0;
		a[8] = 0;
	} else if (len==16){
		a[0] = 0;
		a[1] = 0;
		a[2] = 0;
		a[3] = 0;
		a[4] = 0;
		a[5] = 0;
		a[6] = 0;
		a[7] = 0;
		a[8] = 0;
		a[9] = 0;
		a[10] = 0;
		a[11] = 0;
		a[12] = 0;
		a[13] = 0;
		a[14] = 0;
		a[15] = 0;
	} else {
		double* pa = a + len;
		while (pa > a) *--pa = 0;
	}
}

static void _MatIdentity(int na, double* a){
	_MatZero(na, na, a);
	if (0){
	} else if (na==1){
		a[0] = 1;
	} else if (na==2){
		a[0*2+0] = 1;
		a[1*2+1] = 1;
	} else if (na==3){
		a[0*3+0] = 1;
		a[1*3+1] = 1;
		a[2*3+2] = 1;
	} else if (na==4){
		a[0*4+0] = 1;
		a[1*4+1] = 1;
		a[2*4+2] = 1;
		a[3*4+3] = 1;
	} else {
		int i;
		for (i = 0; i < na; i++) a[i*na+i] = 1;
	}
}

static int MatEqual(int na, int ma, double* a, double* b){
	int len = na * ma;
	if (0){
	} else if (len==1){
		if (b[0] != a[0]) return 0;
	} else if (len==4){
		if (b[0] != a[0]) return 0;
		if (b[1] != a[1]) return 0;
		if (b[2] != a[2]) return 0;
		if (b[3] != a[3]) return 0;
	} else if (len==9){
		if (b[0] != a[0]) return 0;
		if (b[1] != a[1]) return 0;
		if (b[2] != a[2]) return 0;
		if (b[3] != a[3]) return 0;
		if (b[4] != a[4]) return 0;
		if (b[5] != a[5]) return 0;
		if (b[6] != a[6]) return 0;
		if (b[7] != a[7]) return 0;
		if (b[8] != a[8]) return 0;
	} else if (len==16){
		if (b[0] != a[0]) return 0;
		if (b[1] != a[1]) return 0;
		if (b[2] != a[2]) return 0;
		if (b[3] != a[3]) return 0;
		if (b[4] != a[4]) return 0;
		if (b[5] != a[5]) return 0;
		if (b[6] != a[6]) return 0;
		if (b[7] != a[7]) return 0;
		if (b[8] != a[8]) return 0;
		if (b[9] != a[9]) return 0;
		if (b[10] != a[10]) return 0;
		if (b[11] != a[11]) return 0;
		if (b[12] != a[12]) return 0;
		if (b[13] != a[13]) return 0;
		if (b[14] != a[14]) return 0;
		if (b[15] != a[15]) return 0;
	} else {
		double* pa = a + len;
		double* pb = b + len;
		while (pa > a){
			if (*--pb != *--pa) return 0;
		}
	}
	return 1;
}

static void _MatCopyTo(int na, int ma, double* a, double* b){
	int len = na * ma;
	if (0){
	} else if (len==1){
		b[0] = a[0];
	} else if (len==4){
		b[0] = a[0];
		b[1] = a[1];
		b[2] = a[2];
		b[3] = a[3];
	} else if (len==9){
		b[0] = a[0];
		b[1] = a[1];
		b[2] = a[2];
		b[3] = a[3];
		b[4] = a[4];
		b[5] = a[5];
		b[6] = a[6];
		b[7] = a[7];
		b[8] = a[8];
	} else if (len==16){
		b[0] = a[0];
		b[1] = a[1];
		b[2] = a[2];
		b[3] = a[3];
		b[4] = a[4];
		b[5] = a[5];
		b[6] = a[6];
		b[7] = a[7];
		b[8] = a[8];
		b[9] = a[9];
		b[10] = a[10];
		b[11] = a[11];
		b[12] = a[12];
		b[13] = a[13];
		b[14] = a[14];
		b[15] = a[15];
	} else {
		double* pa = a + len;
		double* pb = b + len;
		while (pa > a) *--pb = *--pa;
	}
}

static void _MatScaleTo(int na, int ma, double* a, double* b, double scale){
	int len = na * ma;
	if (scale == 1){
		_MatCopyTo(na, ma, a, b);
		return;
	}
	if (0){
	} else if (len==1){
		b[0] = a[0] * scale;
	} else if (len==4){
		b[0] = a[0] * scale;
		b[1] = a[1] * scale;
		b[2] = a[2] * scale;
		b[3] = a[3] * scale;
	} else if (len==9){
		b[0] = a[0] * scale;
		b[1] = a[1] * scale;
		b[2] = a[2] * scale;
		b[3] = a[3] * scale;
		b[4] = a[4] * scale;
		b[5] = a[5] * scale;
		b[6] = a[6] * scale;
		b[7] = a[7] * scale;
		b[8] = a[8] * scale;
	} else if (len==16){
		b[0] = a[0] * scale;
		b[1] = a[1] * scale;
		b[2] = a[2] * scale;
		b[3] = a[3] * scale;
		b[4] = a[4] * scale;
		b[5] = a[5] * scale;
		b[6] = a[6] * scale;
		b[7] = a[7] * scale;
		b[8] = a[8] * scale;
		b[9] = a[9] * scale;
		b[10] = a[10] * scale;
		b[11] = a[11] * scale;
		b[12] = a[12] * scale;
		b[13] = a[13] * scale;
		b[14] = a[14] * scale;
		b[15] = a[15] * scale;
	} else {
		double* pa = a + len;
		double* pb = b + len;
		while (pa > a) *--pb = *--pa * scale;
	}
}

static void _MatAddTo(int na, int ma, double* a, double* b){
	int len = na * ma;
	if (0){
	} else if (len==1){
		b[0] += a[0];
	} else if (len==4){
		b[0] += a[0];
		b[1] += a[1];
		b[2] += a[2];
		b[3] += a[3];
	} else if (len==9){
		b[0] += a[0];
		b[1] += a[1];
		b[2] += a[2];
		b[3] += a[3];
		b[4] += a[4];
		b[5] += a[5];
		b[6] += a[6];
		b[7] += a[7];
		b[8] += a[8];
	} else if (len==16){
		b[0] += a[0];
		b[1] += a[1];
		b[2] += a[2];
		b[3] += a[3];
		b[4] += a[4];
		b[5] += a[5];
		b[6] += a[6];
		b[7] += a[7];
		b[8] += a[8];
		b[9] += a[9];
		b[10] += a[10];
		b[11] += a[11];
		b[12] += a[12];
		b[13] += a[13];
		b[14] += a[14];
		b[15] += a[15];
	} else {
		double* pa = a + len;
		double* pb = b + len;
		while (pa > a) *--pb += *--pa;
	}
}

static void _MatSubFrom(int na, int ma, double* a, double* b){
	int len = na * ma;
	if (0){
	} else if (len==1){
		b[0] -= a[0];
	} else if (len==4){
		b[0] -= a[0];
		b[1] -= a[1];
		b[2] -= a[2];
		b[3] -= a[3];
	} else if (len==9){
		b[0] -= a[0];
		b[1] -= a[1];
		b[2] -= a[2];
		b[3] -= a[3];
		b[4] -= a[4];
		b[5] -= a[5];
		b[6] -= a[6];
		b[7] -= a[7];
		b[8] -= a[8];
	} else if (len==16){
		b[0] -= a[0];
		b[1] -= a[1];
		b[2] -= a[2];
		b[3] -= a[3];
		b[4] -= a[4];
		b[5] -= a[5];
		b[6] -= a[6];
		b[7] -= a[7];
		b[8] -= a[8];
		b[9] -= a[9];
		b[10] -= a[10];
		b[11] -= a[11];
		b[12] -= a[12];
		b[13] -= a[13];
		b[14] -= a[14];
		b[15] -= a[15];
	} else {
		double* pa = a + len;
		double* pb = b + len;
		while (pa > a) *--pb -= *--pa;
	}
}

static int _MatFinite(int na, int ma, double* a){
	int len = na * ma;
	if (0){
	} else if (len==1){
		if (!_finite(a[0])) return 0;
	} else if (len==4){
		if (!_finite(a[0])) return 0;
		if (!_finite(a[1])) return 0;
		if (!_finite(a[2])) return 0;
		if (!_finite(a[3])) return 0;
	} else if (len==9){
		if (!_finite(a[0])) return 0;
		if (!_finite(a[1])) return 0;
		if (!_finite(a[2])) return 0;
		if (!_finite(a[3])) return 0;
		if (!_finite(a[4])) return 0;
		if (!_finite(a[5])) return 0;
		if (!_finite(a[6])) return 0;
		if (!_finite(a[7])) return 0;
		if (!_finite(a[8])) return 0;
	} else if (len==16){
		if (!_finite(a[0])) return 0;
		if (!_finite(a[1])) return 0;
		if (!_finite(a[2])) return 0;
		if (!_finite(a[3])) return 0;
		if (!_finite(a[4])) return 0;
		if (!_finite(a[5])) return 0;
		if (!_finite(a[6])) return 0;
		if (!_finite(a[7])) return 0;
		if (!_finite(a[8])) return 0;
		if (!_finite(a[9])) return 0;
		if (!_finite(a[10])) return 0;
		if (!_finite(a[11])) return 0;
		if (!_finite(a[12])) return 0;
		if (!_finite(a[13])) return 0;
		if (!_finite(a[14])) return 0;
		if (!_finite(a[15])) return 0;
	} else {
		double* pa = a + len;
		while (pa > a){
			if (!_finite(*--pa)) return 0;
		}
	}
	return 1;
}

static double MatLargest(int na, int ma, double* a){
	double result = 0, temp;
	int len = na * ma;
	if (0){
	} else if (len==1){
		temp = a[0]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
	} else if (len==4){
		temp = a[0]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[1]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[2]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[3]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
	} else if (len==9){
		temp = a[0]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[1]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[2]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[3]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[4]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[5]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[6]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[7]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[8]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
	} else if (len==16){
		temp = a[0]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[1]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[2]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[3]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[4]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[5]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[6]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[7]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[8]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[9]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[10]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[11]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[12]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[13]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[14]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		temp = a[15]; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
	} else {
		double* pa = a + len;
		while (pa > a){
			temp = *--pa; if (temp){if (temp < 0) temp = -temp; if (result < temp) result = temp;}
		}
	}
	return result;
}

static int MatInTolerance(int na, int ma, double* a, double postol){
	double negtol = -postol;
	int len = na * ma;
	double temp;
	if (0){
	} else if (len==1){
		temp = a[0]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
	} else if (len==4){
		temp = a[0]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[1]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[2]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[3]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
	} else if (len==9){
		temp = a[0]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[1]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[2]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[3]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[4]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[5]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[6]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[7]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[8]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
	} else if (len==16){
		temp = a[0]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[1]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[2]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[3]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[4]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[5]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[6]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[7]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[8]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[9]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[10]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[11]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[12]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[13]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[14]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		temp = a[15]; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
	} else {
		double temp;
		double* pa = a + len;
		while (pa > a){
			temp = *--pa; if (temp){if (temp<negtol) return 0; if (temp>postol) return 0;}
		}
	}
	return 1;
}

////////////////// Interpolated table stuff

void _tableGetXPos(int n, double x, int* ix0, int* ix1, double* xFrac, ...){
	int i;
	double* a = 0L;
	va_list argp;
	va_start(argp, xFrac);
	a = (double*)argp;
	if (x < a[0]){*ix0 = *ix1 = 0; *xFrac = 0; return;}
	for (i = 0; i < n-1; i++){
		if (x >= a[i] && x <= a[i+1]){
			*ix0 = i;
			*ix1 = i+1;
			if (a[i]==a[i+1]){
				*xFrac = 0.5;
			} else {
				*xFrac = (x - a[i]) / (a[i+1] - a[i]);
			}
			return;
		}
	}
	{*ix0 = *ix1 = n-1; *xFrac = 1; return;}
	va_end(argp);
}

void _tableSetYVal(int n, double* y, int ix0, int ix1, double xFrac, ...){
	double* a = 0L;
	va_list argp;
	va_start(argp, xFrac);
	a = (double*)argp;
	*y = a[ix0] * (1 - xFrac) + a[ix1] * xFrac;
	va_end(argp);
}

////////////////// Delay stuff

void _setOnTrajectory(int b){
	_bOnTrajectory = !!b;
}

void _initDelayTable(_delayTable_t* pt){
	int i;
	double* pmem = NULL;
	pt->delt = 1;
	pt->c = 1;
	pt->gammc = 1;
	pt->lgammc = 0;
	pt->iDoseMode = 0;
	if (pt->t == NULL || pt->y == NULL){
		pt->mxt = GROW_DELAYTABLE;
		pmem = (double*)calloc(pt->mxt, sizeof(double));
		if (pmem == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
		pt->t = pmem;
		pmem = (double*)calloc(pt->mxt, sizeof(double));
		if (pmem == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
		pt->y = pmem;
	}
	pt->nt = 0;

	if (pt->tDose == NULL || pt->yDose == NULL || pt->rDose == NULL){
		pt->mxtDose = GROW_DELAYTABLE;
		pmem = (double*)calloc(pt->mxtDose, sizeof(double));
		if (pmem == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
		pt->tDose = pmem;
		pmem = (double*)calloc(pt->mxtDose, sizeof(double));
		if (pmem == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
		pt->yDose = pmem;
		pmem = (double*)calloc(pt->mxtDose, sizeof(double));
		if (pmem == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
		pt->rDose = pmem;
	}
	pt->ntDose = 0;

	pt->_yHist = 0;

	//pt->cq = 0;
	//pt->nq = 0;
	//memset(pt->q, 0, sizeof(pt->q));
	//memset(pt->p, 0, sizeof(pt->p));

	////pt->latestt = 0;
	////pt->deltq = 0;
	////memset(pt->transt, 0, sizeof(pt->transt));
}

// Note to maintainers: This routine and the next one have identical C code,
// so the second routine is just a copy of the first, except for the macros.
// This is to try to prevent coding errors.
void _delayInsertT(double t, double y, _delayTable_t* pt){
#define MXT pt->mxt
#define NT pt->nt
#define T pt->t
#define Y pt->y
	int i;
	// if NT is equal to MXT, the table is maxed out, so deal with it
	while(NT >= MXT){
		// if growing it would make it too big
		if (MXT + GROW_DELAYTABLE > MAX_DELAYTABLE){
			// just drop off the oldest point
			NT--;
		// otherwise we can grow it, so
		} else {
			// grow T and Y to the new size
			double *newt = NULL, *newy = NULL;
			int newSize = MXT + GROW_DELAYTABLE;
			newt = (double*)calloc(newSize, sizeof(double));
			if (newt == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
			newy = (double*)calloc(newSize, sizeof(double));
			if (newy == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
			memcpy(newt, T, NT * sizeof(double));
			memcpy(newy, Y, NT * sizeof(double));
			free(T);
			free(Y);
			T = newt;
			Y = newy;
			MXT = newSize;
		}
	}
	// shift all the data so the 0 point is available
	memmove(&T[1], &T[0], NT * sizeof(double));
	memmove(&Y[1], &Y[0], NT * sizeof(double));
#ifdef R
	memmove(&R[1], &R[0], NT * sizeof(double));
#endif
	NT++;
	// put the new point in at point 0
	T[0] = t;
	Y[0] = y;
#ifdef R
	R[0] = r;
#endif
	// most recent T should be later than in table.
	// if not, ripple it into position.
	// assume table is already in descending order
	for (i = 0; i < NT-1 && T[i] < T[i+1]; i++){
		double temp;
		temp = T[i]; T[i] = T[i+1]; T[i+1] = temp;
		temp = Y[i]; Y[i] = Y[i+1]; Y[i+1] = temp;
#ifdef R
		temp = R[i]; R[i] = R[i+1]; R[i+1] = temp;
#endif
	}

#ifndef R
	// remove duplicates
	for (i = 0; i < NT-1; i++){
		if (T[i]==T[i+1] && Y[i]==Y[i+1]){
			memmove(&T[i], &T[i+1], (NT-i) * sizeof(double));
			memmove(&Y[i], &Y[i+1], (NT-i) * sizeof(double));
#ifdef R
			memmove(&R[i], &R[i+1], (NT-i) * sizeof(double));
#endif
			NT--;
			break;
		}
	}
#endif

#undef MXT
#undef NT
#undef T
#undef Y
}

void _delayInsertTDose(double t, double y, double r, _delayTable_t* pt){
#define MXT pt->mxtDose
#define NT pt->ntDose
#define T pt->tDose
#define Y pt->yDose
#define R pt->rDose
	int i;
	// if NT is equal to MXT, the table is maxed out, so deal with it
	while(NT >= MXT){
		// if growing it would make it too big
		if (MXT + GROW_DELAYTABLE > MAX_DELAYTABLE){
			// just drop off the oldest point
			NT--;
		// otherwise we can grow it, so
		} else {
			// grow T and Y to the new size
			double *newt = NULL, *newy = NULL;
			int newSize = MXT + GROW_DELAYTABLE;
			newt = (double*)calloc(newSize, sizeof(double));
			if (newt == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
			newy = (double*)calloc(newSize, sizeof(double));
			if (newy == NULL){SetErrorMsg("Error: Memory exhaustion in delay table"); exit(1);}
			memcpy(newt, T, NT * sizeof(double));
			memcpy(newy, Y, NT * sizeof(double));
			free(T);
			free(Y);
			T = newt;
			Y = newy;
			MXT = newSize;
		}
	}
	// shift all the data so the 0 point is available
	memmove(&T[1], &T[0], NT * sizeof(double));
	memmove(&Y[1], &Y[0], NT * sizeof(double));
#ifdef R
	memmove(&R[1], &R[0], NT * sizeof(double));
#endif
	NT++;
	// put the new point in at point 0
	T[0] = t;
	Y[0] = y;
#ifdef R
	R[0] = r;
#endif
	// most recent T should be later than in table.
	// if not, ripple it into position.
	// assume table is already in descending order
	for (i = 0; i < NT-1 && T[i] < T[i+1]; i++){
		double temp;
		temp = T[i]; T[i] = T[i+1]; T[i+1] = temp;
		temp = Y[i]; Y[i] = Y[i+1]; Y[i+1] = temp;
#ifdef R
		temp = R[i]; R[i] = R[i+1]; R[i+1] = temp;
#endif
	}
#undef MXT
#undef NT
#undef T
#undef Y
#undef R
}

void _maybeSetGammaTable(double c, int bCdf, _delayTable_t *pt){
	// we want the mean to be 1, so we make the scale factor one over the shape
	double b = 1/c;
	double lb, lgammc, gammc;
	double q, minq, maxq, delq;
	int maxnq = COUNTOF(pt->q);
	double sum = 0;
	int i;
	// if c hasn't changed, don't change the q table
	if (pt->cq != c){
		pt->cq = c;
		lb = log(b);
		lgammc = lgamm(c);
		gammc = exp(lgammc);
		// the range of q values depends on c
		if (0){
		} else if (c <    0.01){
			minq = 0;
			maxq = 265;
		} else if (c <    0.1 ){
			minq = 0;
			maxq = 151;
		} else if (c <    1   ){
			minq = 0;
			maxq = 34;
		} else if (c <   10   ){
			minq = 0;
			maxq = 7;
		} else if (c <  100   ){
			minq = 0.29;
			maxq = 2.3;
		} else if (c < 1000   ){
			minq = 0.7;
			maxq = 1.4;
		} else {
			minq = 0.9;
			maxq = 1.1;
		}
		// put the q and p values into the table
		delq = (maxq - minq)/(maxnq-1);
		for (q = minq, i = 0; i < maxnq-2; q += delq, i++){
			double p = 0;
			if (bCdf){
				p = gammCDF(q, c, b, lb, lgammc, gammc);
			} else {
				p = gammPDF(q, c, b, lb, lgammc);
				sum += p * delq;
			}
			pt->q[i] = q;
			pt->p[i] = p;
		}
		if (bCdf){
			// last entry in table infinite q, p=1
			pt->q[i] = q;     pt->p[i] = 1; i++;
			pt->q[i] = 1e100; pt->p[i] = 1; i++;
		} else {
			// last entry in table infinite q, p=0
			pt->q[i] = q;     pt->p[i] = 0; i++;
			pt->q[i] = 1e100; pt->p[i] = 0; i++;
		}
		ASSERT(i == maxnq);
		pt->nq = maxnq;
	}
}

double _getGammaPValue(double q, _delayTable_t * pt){
	// find the member of pt->q that is >= q
	int i = 0;
	int nq = pt->nq;
	double rvalue = 0;
	double q0, q1, p0, p1, dp, dq;
	if (q <= pt->q[0]) return pt->p[0];
	if (q >= pt->q[nq-1]) return pt->p[nq-1];
#if (CDF_TABLE_SIZE > 512)
	if (q >= pt->q[i+512]) i += 512;
#endif
#if (CDF_TABLE_SIZE > 256)
	if (q >= pt->q[i+256]) i += 256;
#endif
#if (CDF_TABLE_SIZE > 128)
	if (q >= pt->q[i+128]) i += 128;
#endif
	if (q >= pt->q[i+ 64]) i +=  64;
	if (q >= pt->q[i+ 32]) i +=  32;
	if (q >= pt->q[i+ 16]) i +=  16;
	if (q >= pt->q[i+  8]) i +=   8;
	if (q >= pt->q[i+  4]) i +=   4;
	if (q >= pt->q[i+  2]) i +=   2;
	if (q >= pt->q[i+  1]) i +=   1;
	// now i is in the range 0 .. nq-1
	// and it points to the last value <= q
	// return the limit value or the interpolation
	if (0){
	//} else if (i <= 0){
	//	return pt->p[0];
	//} if (i >= nq-1){
	//	return pt->p[nq-1];
	} else {
		q0 = pt->q[i]; q1 = pt->q[i+1]; dq = q1-q0;
		p0 = pt->p[i]; p1 = pt->p[i+1]; dp = p1-p0;
		rvalue = p0 + dp*(q-q0)/dq;
	}
	return rvalue;
}

void _insertDose(double tnow, double yy, double rate, _delayTable_t* pt){
	int i;
	// if this is a repeat, add yy
	if (pt->ntDose > 0 && pt->tDose[0]==tnow){
		pt->yDose[0] += yy;
	} else if (pt->ntDose == 0 || tnow > pt->tDose[0]){
		_delayInsertTDose(tnow, yy, rate, pt);
	}
}

void _delayAddIncrements(double* py, double ta, double tb, double delt, double shape, double threshold, _delayTable_t* pt){
	int i;
	double maxtta, g0, g1;
	double c = shape, lgammc = lgamm(c);
	double gammc = exp(lgammc);
	double b = delt / c, lb = log(b);
	double dg;
	// handle discrete delay
	if (shape >= threshold){
		for (i = 0; i < pt->ntDose; i++){
			double t = pt->tDose[i] + delt;
			double y = pt->yDose[i];
			if (t >= ta && t < tb){
				*py += y;
			}
		}
	// handle distributed delay
	} else {
		for (i = 0; i < pt->ntDose; i++){
			double t = pt->tDose[i];
			double y = pt->yDose[i];
			maxtta = t; if (maxtta < ta) maxtta = ta;
			if (maxtta >= tb) continue;
			g0 = gammCDF(maxtta-t, c, b, lb, lgammc, gammc);
			g1 = gammCDF(    tb-t, c, b, lb, lgammc, gammc);
			dg = g1 - g0;
			*py += dg * y;
		}
	}
}

int nDelayFuncCalls = 0;

double ConvTrap(
	double dx
	, double a, double b
	, double c, double d
	)
{
	double dxx = dx*dx, dxxx = dxx*dx;
	double vol = a*c*dxxx/3 + (a*d+b*c)*dxx/2 + b*d*dx;
	return vol;
}

void testTrap(){
	double dx = 1;
	double a = 1, b = 1;
	double c = 2, d = 2;
	double vol = ConvTrap(dx, a, b, c, d); 
	// result should be ac/3 + (ad+bc)/2 + bd
	double tv = a*c*dx*dx*dx/3 + (a*d+b*c)*dx*dx/2 + b*d*dx;
	ASSERT(fabs(tv-vol) < 1e-12);
}

void MyAssert(BOOL b){
	if (!b){
		int breakhere = 1;
	}
}

double _delayFunc(double tnow, double yy, double delt, double shape, double threshold, _delayTable_t* pt){
	double yres = 0;
	// tget is relative to most recent time
	// if most recent time is 300, and delt is 100, tget is (300-100) = 200
	double tget = (delt > 0 ? tnow - delt : tnow);
	int i, j, nt;
	int bStoreThePoint = 0;
	nDelayFuncCalls++;
	if (_finite(yy)){
		// if discrete delay
		if (shape >= threshold){
			// only store if on the trajectory (intermediate points can be noisy)
			bStoreThePoint = _bOnTrajectory;
		// if distributed delay and yy == 0, no point storing it
		} else if (yy == 0){
		// if distributed delay and yy != 0
		} else {
			// store the point. Even if it's noisy, it's better than nothing.
			bStoreThePoint = 1;
			//bStoreThePoint = _bOnTrajectory;
		}
	}
	// if storing, record the point
	if (bStoreThePoint){
		if (0){
		//// MD: allow more than one value at the same time
		////// if this is a repeat point, keep the most recent value
		////} else if (pt->nt > 0 && tnow == pt->t[0]){
		////	pt->y[0] = yy;
		//// don't insert the point if it is the same as the previous point
		//} else if (pt->nt > 0 && tnow == pt->t[0] && yy == pt->y[0]){
		//// if it is the same value as the previous two points, just replace the previous point
		//} else if (pt->nt > 1 && pt->y[0]==yy && pt->y[1]==yy){
		//	pt->t[0] = tnow;
		//// else insert the point (expect it to be same time or later than last point)
		////} else if (pt->nt == 0 || tnow >= pt->t[0]){
		} else {
			_delayInsertT(tnow, yy, pt);
		}
	}
	// whether storing or not, retrieve the value
	// handle the simple discrete delay case
	// where either there is no shape parameter or there is but delt is too small
	if (shape >= threshold || delt < 1e-12){
		nt = pt->nt;
		if (nt == 0){
			yres = pt->_yHist;
		} else if (tget < pt->t[nt-1]){
			yres = pt->_yHist;
		} else if (nt == 1 || tget >= pt->t[0]){
			//yres = pt->y[0];
			yres = yy;
		} else {
			double t0, t1, y0, y1;
			t1 = tnow; y1 = yy;
			for (i = 0; i < pt->nt; i++){
				t0 = pt->t[i], y0 = pt->y[i];
				if (tget >= t0){
					double dt = t1-t0, dy = y1-y0;
					if (dt > 0){
						double r = (tget - t0)/dt;
						yres = y0 + r * dy;
						break;
					} else if (dt == 0){
						yres = y1;
						break;
					}
				}
				t1 = t0; y1 = y0;
			}
		}
	// handle the gamma-distributed case
	} else {
		// the density function is G(x; b, c) = (x/b)^(c-1)*exp(-x/b)/(b*gamma(c))
		// where c>0 is shape, b>0 = delt/c, and x goes from 0 to inf, mean = delt
		// convolution is integ(s = -inf..t) F(s)*G(t-s)*ds  where F(s) is the history of the signal
		// note: the further back in time the table point is, the further out on the right tail of the pdf
		double c = shape, lgammc = lgamm(c);
		double gammc = exp(lgammc);
		double b = delt / c, lb = log(b);
		nt = pt->nt;
		if (nt == 0){
			yres = pt->_yHist;
		//} else if (tget < pt->t[nt-1]){
		//	yres = 0;
		//} else if (tget > pt->t[0]){
		//	yres = pt->y[0];
		} else if (nt == 1){
			//yres = pt->y[0];
			yres = pt->_yHist;
		//// convolution using CDF table
		//} else if (c < 1000){
		//	_maybeSetGammaTable(shape, TRUE, pt);
		//	// foreach history value
		//	double p0 = _getGammaPValue(pt->transt[0], pt);
		//	double p1;
		//	double t0 = pt->t[0], t1;
		//	double y0 = pt->y[0], y1;
		//	for (i = 1; i < pt->nt; i++){
		//		double ym, del;
		//		t1 = pt->t[i];
		//		y1 = pt->y[i];
		//		p1 = _getGammaPValue(pt->transt[i], pt);
		//		ASSERT(_finite(p1));
		//		ym = (y0+y1)/2;
		//		del = ym * (p1-p0);
		//		yres += del;
		//		ASSERT(_finite(yres));
		//		p0 = p1;
		//		t0 = t1;
		//		y0 = y1;
		//	}
		//	// add the prior piece, if any
		//	p0 = p1;
		//	p1 = 1;
		//	yres += pt->_yHist * (p1-p0);
		// convolution using CDF
		} else if (FALSE && c < 1000){
			_maybeSetGammaTable(shape, TRUE, pt);
			// iterate from i=0 (latest time) to i=nt-1 (earliest time)
			// x0 is later time, x1 is earlier time, so x0-x1 is positive
			// xlatest is time corresponding to 0
			// if xlatest is 293, and x is 250, then evaluate the CDF at 293-250 = 43
			//double lgammc = lgamm(c);
			double t1, y1;
			double t0 = tnow, y0 = yy;
			double g0 = 0;
			double g1;
			for (i = 0; i < pt->nt && (g0 < 1); i++){
				double dt, ym, del;
				t1 = pt->t[i]; y1 = pt->y[i];
				g1 = _getGammaPValue((tnow-t1)/delt, pt);
				//g1 = gammCDF((tnow-t1), c, b, lb, lgammc, gammc);
				//ASSERT(_finite(g1));
				dt = t0-t1; ym = (y0+y1)/2;
				del = ym * (g1 - g0);
				yres += del;
				//ASSERT(_finite(yres));
				t0 = t1; y0 = y1; g0 = g1;
			}
			// add the prior piece, if any
			g0 = g1;
			g1 = 1;
			yres += pt->_yHist * (g1 - g0);
		} else if (c < 1){
			// iterate from i=0 (latest time) to i=nt-1 (earliest time)
			// x0 is later time, x1 is earlier time, so x0-x1 is positive
			// xlatest is time corresponding to 0
			// if xlatest is 293, and x is 250, then evaluate the CDF at 293-250 = 43
			//double lgammc = lgamm(c);
			double t1, y1;
			double t0 = tnow, y0 = yy;
			double g0 = 0;
			double g1;
			ASSERT(_finite(g0));
			for (i = 0; i < pt->nt && (g0 < 1); i++){
				double dt, ym, del;
				t1 = pt->t[i]; y1 = pt->y[i];
				g1 = gammCDF((tnow-t1), c, b, lb, lgammc, gammc);
				ASSERT(_finite(g1));
				dt = t0-t1; ym = (y0+y1)/2;
				del = ym * (g1 - g0);
				yres += del;
				ASSERT(_finite(yres));
				t0 = t1; y0 = y1; g0 = g1;
			}
			// add the prior piece, if any
			g0 = g1;
			g1 = 1;
			yres += pt->_yHist * (g1 - g0);
		// convolution using PDF
		} else {
			double t1, y1;
			double t0 = tnow, y0 = yy;
			double g1, g0 = 0;
			//double oldg1, oldg0 = 0;
			double dt, trap, trap1;
			//_maybeSetGammaTable(shape, FALSE, pt); // FALSE to tabulate PDF, not CDF
			if (c < 1){
				g0 = -1;
			} else {
				g0 = gammPDF((tnow-t0), c, b, lb, lgammc);
				//g0 = _getGammaPValue((tnow-t0)/delt, pt)/delt;
				//MyAssert(fabs(oldg0-g0) < 1e-4);
			}
			for (i = 0; i < pt->nt; i++){
				t1 = pt->t[i]; y1 = pt->y[i];
				if (t1 == tnow) continue;
				if (t1 > tnow) g1 = 0;
				else {
					g1 = gammPDF((tnow-t1), c, b, lb, lgammc);
					//g1 = _getGammaPValue((tnow-t1)/delt, pt)/delt;
					//MyAssert(fabs(oldg1-g1) < 1e-4);
				}
				if ((tnow-t1)/delt > 1 && (g1 < 1e-10)) break;
				if (g0 >= 0){
					dt = t0-t1;
#if 0
					// trapezoidal convolution from 1 to 0
					trap = ConvTrap(dt
						, (y0-y1)/dt, y1
						, (g0-g1)/dt, g1
						);
#else
					trap = dt*(y1*g1 + y0*g0)/2;
#endif
					yres += trap;
				}
				t0 = t1; y0 = y1; g0 = g1;
			}
			if (i < pt->nt){
				int breakhere = 1;
			}
			// do the prior piece, using CDF
			if (pt->_yHist != 0){
				g0 = gammCDF((tnow-t0), c, b, lb, lgammc, exp(lgammc));
				g1 = 1;
				yres += pt->_yHist * (g1 - g0);
			}
		}
		// commented out because doing this in AddIncrements
		if (pt->iDoseMode == DBM_DOSE_TABLE){
			// now add in dose function pieces
			for (i = 0; i < pt->ntDose; i++){
				double t, y, r, g0, g1, dg = 0, dy = 0;
				double pg0, pdy;
#if 0
				extern double euler_dt;
				extern double RKStepSize;
				//double dSimInfusionLength = euler_dt;
				double dSimInfusionLength = RKStepSize;
#endif
				t = pt->tDose[i];
				y = pt->yDose[i];
				r = pt->rDose[i];
#if 1
				if (r == _NA){
					pg0 = gammPDF((tnow - t), c, b, lb, lgammc);
					pdy = pg0 * y;
					yres += pdy;
				} else {
					double d = y / r; // convert rate to duration
					double t0 = t;
					double t1 = t0 + d;
					if (tnow >= t0){
						if (tnow <= t1){
							dg = gammCDF((tnow - t0), c, b, lb, lgammc, gammc);
						} else {
							g0 = gammCDF((tnow - t0), c, b, lb, lgammc, gammc);
							g1 = gammCDF((tnow - t1), c, b, lb, lgammc, gammc);
							dg = g0 - g1;
						}
					}
					dy = dg * r;
					yres += dy;
				}
#else
				// simulate a .01 infusion
				g0 = gammCDF((tnow - t), c, b, lb, lgammc, gammc);
				g1 = gammCDF((tnow + dSimInfusionLength - t), c, b, lb, lgammc, gammc);
				dg = g1 - g0;
				if (dSimInfusionLength > 0){
					dy = dg * y / dSimInfusionLength;
				}
				yres += dy;
#endif
			}
		}
	}
	return yres;
}

////////////////// Peak stuff

void _initPeakTable(_peakTable_t* pt){
	int i;
	for (i = 0; i < 4; i++){
		pt->v[i] = -1;
		pt->v[i+4] = 0;
	}
	pt->v[8] = pt->v[9] = 0;
}

BOOL _cubicPeak(double x[4], double y[4], double *Xmax, double*Ymax, BOOL bMin){
	int i, j, k;
	double yd[4];
	double v[4], w[4], z[4];
	double A, B, C, rr, r, x1, x2, ydotdot1, ydotdot2, ym, xm, xm2, xm3;
#define X(i)(x[(i)-1])
#define Y(i)(y[(i)-1])
#define YD(i)(yd[(i)-1])
#define V(i)(v[(i)-1])
#define W(i)(w[(i)-1])
#define Z(i)(z[(i)-1])
/**** This is based on the Lagrange interpolation formula:
https://en.wikipedia.org/wiki/Lagrange_polynomial
P(x) = y1 * (x-x2)(x-x3)(x-x4)/((x1-x2)(x1-x3)(x1-x4)) + y2 * ...
d(i) is the denominator for the y(i) term, and yd(i) = y(i)/d(i)

Expanding terms, the polynomial is restated as the cubic:
P(x) = Sum(i=1,4) yd(i) * (x^3 - v(i)*x^2 + w(i)*x^1 - z(i)*x^0)

The first derivative is quadratic, and A, B, and C are its coefficients.
Solve this to get the two x points where the derivative is zero.

Then evaluate the second derivative to see which one is the peak and which one is the trough.
Determine the y value at the the peak or trough, as desired.
****/
	// if it doesn't look like a peak, don't even bother
	if (bMin){
		if (Y(2) >= Y(1)) return FALSE;
		if (Y(3) >= Y(4)) return FALSE;
	} else {
		if (Y(2) <= Y(1)) return FALSE;
		if (Y(3) <= Y(4)) return FALSE;
	}
	// get YD(i)
	for (i = 1; i <= 4; i++){
		double d = 1;
		for (j = 1; j <= 4; j++){
			if (j != i) d *= (X(i) - X(j));
		}
		YD(i) = Y(i)/d;
	}
	// get V, W, Z
	for (i = 1; i <= 4; i++){
		V(i) = 0;
		for (j = 1; j <= 4; j++){
			if (j != i) V(i) += X(j);
		}
		W(i) = 0;
		for (j = 1; j <= 4; j++){
			for (k = j+1; k <= 4; k++){
				if (j != i && k != i) W(i) += X(j) * X(k);
			}
		}
		Z(i) = 1;
		for (j = 1; j <= 4; j++){
			if (j != i) Z(i) *= X(j);
		}
	}
	// get coefficients of the derivative, which is quadratic
	A = B = C = 0;
	for (i = 1; i <= 4; i++){
		A += YD(i) * 3;
		B += YD(i) * (-2 * V(i));
		C += YD(i) * W(i);
	}
	// get the X coords of the min and max
	rr = B*B - 4*A*C;
	if (rr < 0) return FALSE;
	r = sqrt(rr);
	x1 = (-B-r)/(2*A);
	x2 = (-B+r)/(2*A);
	// get second derivative at each x value (should be opposites)
	ydotdot1 = ydotdot2 = 0;
	for (i = 1; i <= 4; i++){
		ydotdot1 += YD(i) * (6*x1 - 2*V(i));
		ydotdot2 += YD(i) * (6*x2 - 2*V(i));
	}
	// for peak, choose the root with negative curvature, else positive
	xm = 0;
	if (bMin){
		if (ydotdot1 > 0){
			xm = x1;
		} else if (ydotdot2 > 0){
			xm = x2;
		} else return FALSE;
	} else {
		if (ydotdot1 < 0){
			xm = x1;
		} else if (ydotdot2 < 0){
			xm = x2;
		} else return FALSE;
	}
	// if xm is not in the original x range
	// reject it (though it should have been caught above)
	if (xm >= X(1) && xm <= X(4));
	else return FALSE;
	ym = 0;
	xm2 = xm*xm;
	xm3 = xm2*xm;
	for (i = 1; i <= 4; i++){
		ym += YD(i) * (xm3 - xm2*V(i) + xm*W(i) - Z(i));
	}
	*Xmax = xm;
	*Ymax = ym;
#undef X
#undef Y
#undef YD
#undef V
#undef W
#undef Z
	return TRUE;
}

double _peakFunc(double _tm, double _v, double _bMin, double *_cmax, _peakTable_t *_pt){
	double _tmax = 0;
#define TABL(i)((*_pt).v[(i)-1])
	double _xm = 0, _ym = 0;
	// ignore this if not on the trajectory
	//if (!OnTrajectory()) return;
	// if this is a repeat point, ignore it
	if (_tm > TABL(4)*1.0000001){
		// shift in the new point at the upper end of the table
		TABL(1) = TABL(2); TABL(2) = TABL(3); TABL(3) = TABL(4);
		TABL(4) = _tm;
		TABL(5) = TABL(6); TABL(6) = TABL(7); TABL(7) = TABL(8);
		TABL(8) = _v;
		// if we have 4 points
		if (TABL(1) >= 0){
			// try to get a peak
			if (_cubicPeak(&TABL(1), &TABL(5), &_xm, &_ym, _bMin != 0)){
				// if the peak is higher than what we have seen so far, save it
				if (_bMin){
					if (TABL(10) > _ym){TABL(9) = _xm; TABL(10) = _ym;}
				} else {
					if (TABL(10) < _ym){TABL(9) = _xm; TABL(10) = _ym;}
				}
			}
		}
	}
	// always show the highest peak seen so far
	_tmax = TABL(9);
	*_cmax = TABL(10);
#undef TABL
	return _tmax;
}

////////////////// Closed Form stuff

void CF2Clear(_cf2machine* _p){
	memset(_p, 0, sizeof(*_p));
}
void CF2InitMacro1(_cf2machine* _p, double _al0){
	CF2Reset(_p);
	_p->na  = 1;
	_p->a11[0] =  1;
	_p->  x[0] = - _al0;
}
void CF2InitMacro2(_cf2machine* _p, double _al0, double _a1, double _al1){
	CF2Reset(_p);
	_p->na = 2;
	_p->x[0] = - _al0;
	_p->x[1] = - _al1;
	_p->a11[0] = 1 - _a1;
	_p->a11[1] = _a1;
}
void CF2InitMacro3(_cf2machine* _p, double _al0, double _a1, double _al1, double _a2, double _al2){
	CF2Reset(_p);
	_p->na = 3;
	_p->x[0] = - _al0;
	_p->x[1] = - _al1;
	_p->x[2] = - _al2;
	_p->a11[0] = 1 - _a1 - _a2;
	_p->a11[1] = _a1;
	_p->a11[2] = _a2;
}

void CF2InitMacro1New(_cf2machine* _p, double _a0, double _al0){
	CF2Reset(_p);
	_p->na  = 1;
	_p->a11[0] =  _a0;
	_p->  x[0] = - _al0;
}
void CF2InitMacro2New(_cf2machine* _p, double _a0, double _al0, double _a1, double _al1){
	CF2Reset(_p);
	_p->na = 2;
	_p->x[0] = - _al0;
	_p->x[1] = - _al1;
	_p->a11[0] = _a0;
	_p->a11[1] = _a1;
}
void CF2InitMacro3New(_cf2machine* _p, double _a0, double _al0, double _a1, double _al1, double _a2, double _al2){
	CF2Reset(_p);
	_p->na = 3;
	_p->x[0] = - _al0;
	_p->x[1] = - _al1;
	_p->x[2] = - _al2;
	_p->a11[0] = _a0;
	_p->a11[1] = _a1;
	_p->a11[2] = _a2;
}
void CF2AppendKaNew(_cf2machine* _p, double Ka){
	int i, n = _p->na;
	double sum = 0;
	for (i = 0; i < n; i++){
		sum += _p->a11[i];
	}
	// add absorption rate term
	_p->a11[n] = -sum;
	_p->x[n] = - Ka;
	_p->na = n+1;
}

//int iWhichInitMicroCall = 0;
void CF2InitMicro1(_cf2machine* _p, double _k10){
	//iWhichInitMicroCall++;
	//if (_k10 <= 0)
	//	err = 1;
	//if (err) return;
	CF2Reset(_p);
	_p->na  = 1;
	_p->a11[0] =  1;
	_p->  x[0] = - _k10;
}
void CF2InitMicro2(_cf2machine* _p, double _k10, double _k12, double _k21){
    double ktot, temp, root;
	//if (_k10 <= 0 || _k12 <= 0 || _k21 <= 0)
	//	err = 1;
	ktot = _k21 + _k12 + _k10;
	temp = ktot*ktot - 4 * _k10 * _k21;
	//if (temp < 0)
	//	err = 2;
	//if (err) return;
	ASSERT(temp >= 0);
	root = sqrt(temp);
	CF2Reset(_p);
	_p->na = 2;
	_p->x[0] = (-ktot - root)/2;
	_p->x[1] = (-ktot + root)/2;
	ASSERT(_p->x[0] < 0);
	ASSERT(_p->x[1] < 0);
	ASSERT(_p->x[1] != _p->x[0]);
	_p->a11[0]  = (    _k21 + _p->x[0])/(_p->x[0] - _p->x[1]);	// cpt 1 from y1(0)
	_p->a11[1]  = (    _k21 + _p->x[1])/(_p->x[1] - _p->x[0]);

	_p->a12[0]  = (    _k21       )/(_p->x[0] - _p->x[1]);	// cpt 1 from y2(0)
	_p->a12[1]  = (    _k21       )/(_p->x[1] - _p->x[0]);

	_p->a21[0]  = (    _k12       )/(_p->x[0] - _p->x[1]);	// cpt 2 from y1(0)
	_p->a21[1]  = (    _k12       )/(_p->x[1] - _p->x[0]);

	_p->a22[0]  = (_k10 + _k12 + _p->x[0])/(_p->x[0] - _p->x[1]);	// cpt 2 from y2(0)
	_p->a22[1]  = (_k10 + _k12 + _p->x[1])/(_p->x[1] - _p->x[0]);
}
void CF2InitMicro3(_cf2machine* _p, double _k10, double _k12, double _k21, double _k13, double _k31){
	const double pi = 2 * asin(1.0);
	double u0, u1, u2, p, q, r1, r2, r3, r4;
	//if (_k10 <= 0 || _k12 <= 0 || _k21 <= 0 || _k13 <= 0 || _k31 <= 0)
	//	err = 1;
	CF2Reset(_p);
	_p->na = 3;
	u0 = _k10 * _k21 * _k31;
	u1 = _k10 * _k31 + _k21 * _k31 + _k21 * _k13 + _k10 * _k21 + _k31 * _k12;
	u2 = _k10 + _k12 + _k13 + _k21 + _k31;
	p = u1 - (u2*u2/3);
	q = 2 * u2 * u2 * u2 / 27 - u1 * u2 / 3 + u0;
	//if (p >= 0)
	//	err = 2;
	//if (err) return;
	ASSERT(p < 0);
	r1 = sqrt(-p*p*p/27);
	r2 = -q/2/r1;
	ASSERT(r2 >= -1 && r2 <= 1);
	r3 = acos(r2)/3;
	r4 = 2 * exp(log(r1)/3);
	_p->x[0] = (cos(r3+0*pi/3)*r4 - u2/3);
	ASSERT(_p->x[0] < 0);
	_p->x[1] = (cos(r3+2*pi/3)*r4 - u2/3);
	ASSERT(_p->x[1] < 0);
	_p->x[2] = (cos(r3+4*pi/3)*r4 - u2/3);
	ASSERT(_p->x[2] < 0);
	ASSERT(_p->x[0] != _p->x[1]);
	ASSERT(_p->x[1] != _p->x[2]);
	ASSERT(_p->x[0] != _p->x[2]);

	//---------------------------------------------------------------------------------
	_p->a11[0]  = (    _k21 + _p->x[0])*(_k31 + _p->x[0]) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);	// cpt 1 from y1(0)
	_p->a11[1]  = (    _k21 + _p->x[1])*(_k31 + _p->x[1]) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a11[2]  = (    _k21 + _p->x[2])*(_k31 + _p->x[2]) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);

	_p->a12[0]  = (    _k21     )*(_k31 + _p->x[0]) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);	// cpt 1 from y2(0)
	_p->a12[1]  = (    _k21     )*(_k31 + _p->x[1]) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a12[2]  = (    _k21     )*(_k31 + _p->x[2]) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);

	_p->a13[0]  = (    _k31     )*(_k21 + _p->x[0]) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);	// cpt 1 from y3(0)
	_p->a13[1]  = (    _k31     )*(_k21 + _p->x[1]) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a13[2]  = (    _k31     )*(_k21 + _p->x[2]) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);

	//---------------------------------------------------------------------------------
	_p->a21[0]  = (    _k12     )*(_k31 + _p->x[0]) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);					// cpt 2 from y1(0)
	_p->a21[1]  = (    _k12     )*(_k31 + _p->x[1]) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a21[2]  = (    _k12     )*(_k31 + _p->x[2]) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);

	_p->a22[0]  = ((    _k31 + _p->x[0])*(_k12 + _k10 + _p->x[0]) + _k13*_p->x[0]) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);	// cpt 2 from y2(0)
	_p->a22[1]  = ((    _k31 + _p->x[1])*(_k12 + _k10 + _p->x[1]) + _k13*_p->x[1]) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a22[2]  = ((    _k31 + _p->x[2])*(_k12 + _k10 + _p->x[2]) + _k13*_p->x[2]) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);

	_p->a23[0]  = (_k12*_k31) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);									// cpt 2 from y3(0)
	_p->a23[1]  = (_k12*_k31) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a23[2]  = (_k12*_k31) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);

	//---------------------------------------------------------------------------------
	_p->a31[0]  = (    _k13     )*(_k21 + _p->x[0]) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);					// cpt 3 from y1(0)
	_p->a31[1]  = (    _k13     )*(_k21 + _p->x[1]) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a31[2]  = (    _k13     )*(_k21 + _p->x[2]) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);

	_p->a32[0]  = (_k13*_k21) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);									// cpt 3 from y3(0)
	_p->a32[1]  = (_k13*_k21) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a32[2]  = (_k13*_k21) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);

	_p->a33[0]  = ((    _k21 + _p->x[0])*(_k13 + _k10 + _p->x[0]) + _k12*_p->x[0]) / (_p->x[0] - _p->x[1]) / (_p->x[0] - _p->x[2]);	// cpt 3 from y2(0)
	_p->a33[1]  = ((    _k21 + _p->x[1])*(_k13 + _k10 + _p->x[1]) + _k12*_p->x[1]) / (_p->x[1] - _p->x[0]) / (_p->x[1] - _p->x[2]);
	_p->a33[2]  = ((    _k21 + _p->x[2])*(_k13 + _k10 + _p->x[2]) + _k12*_p->x[2]) / (_p->x[2] - _p->x[0]) / (_p->x[2] - _p->x[1]);
}
void CF2Convolve(
	  int _na, double _a[], double _al[]
	, int _nb, double _b[], double _bl[]
	, double _u[], double _v[]
	)
{
	int i, j;
	double sum;
	for (i=0; i<_na; i++){
		ASSERT(_al[i] < 0);
		_v[i] = _al[i];
	}

	for (j=0; j<_nb; j++){
		ASSERT(_bl[j] < 0);
		_v[_na+j] = _bl[j];
	}

	for (i=0; i<_na; i++){
		sum = 0;
		for (j=0; j<_nb; j++){
			// this convolution has a singularity if any alphas match,
			ASSERT(_bl[j] != _al[i]);
			//sum += _b[j]/(_bl[j] - _al[i]);
			sum += _b[j]/(_al[i] - _bl[j]);
		}
		_u[i] = sum * _a[i];
	}

	for (j=0; j<_nb; j++){
		sum = 0;
		for (i=0; i<_na; i++){
			ASSERT(_bl[j] != _al[i]);
			//sum += _a[i]/(a_l[i]-_bl[j]);
			sum += _a[i]/(_bl[j]-_al[i]);
		}
		_u[_na+j] = sum * _b[j];
	}
}
void CF2AdjustRoundoff(int _na, double _a[]){
	// example: _na = 3, _a = -1.6534994858145, 1.5464462632091, 0.10705322260540
	int _i;
	double _sum = 0, _dif = 0;
	if (_na < 2) return;
	// only consider case where first is negative and rest are positive
	if (_a[0] >= 0) return;
	for (_i = 1; _i < _na; _i++) if (_a[_i] <= 0) return;
	for (_i = 1; _i < _na; _i++) _sum += _a[_i]; // sum = 1.6534994858145
	_dif = _sum + _a[0]; // dif = -2.0816681711722e-016
	if (_dif < 0) _dif = - _dif; // dif = 2.0816681711722e-016
	if (_dif > _sum * 1e-14) return; // compare against 1.6534994858145e-014
	_a[0] = - _sum; // if dif < roundoff error, make it zero
}
void CF2Add1stOrd(_cf2machine* _p, double _scale, double _ka){
	BOOL bChanged;
	int i, j;
	double a[5],  al[5];
	double b[2],  bl[2];
	double u[10], v[10];
	int nb = 1, nu = _p->na+1;
	//if (ka <= 0)
	//	err = 1;
	//if (err) return;
	for (i = 0; i < _p->na; i++) al[i] = _p->x[i];
	// if ka is the same as any a[i], adjust it slightly
	bChanged = TRUE;
	for (j = 0; j < 10 && bChanged; j++){
		bChanged = FALSE;
		for (i=0; i<_p->na; i++){
			if (fabs(_ka + _p->x[i]) < 1e-7){
				_ka += 1e-6;
				bChanged = TRUE;
			}
		}
	}
	bl[0] = - _ka;
	b[0]  = - bl[0];

	// get the exponents here
	v[0] = bl[0];
	for (i = 0; i < _p->na; i++) v[i+1] = _p->x[i];

	// residues for compartment A as a function of A
	_p->aaa[0] = 1;

	// residues for compartment 1 as a function of A
	for (i = 0; i < _p->na; i++) a[i] =        _p->a11[i];
	CF2Convolve(nb, b, bl, _p->na, a, al, u, v);
	// make sure amount in plasma cpt when empty is exactly zero
	CF2AdjustRoundoff(nu, u);
	for (i = 0; i < nu; i++)               _p->a1a[i] = u[i];

	// residues for compartment 2 as a function of A
	for (i = 0; i < _p->na; i++) a[i] =        _p->a21[i];
	CF2Convolve(nb, b, bl, _p->na, a, al, u, v);
	for (i = 0; i < nu; i++)               _p->a2a[i] = u[i];

	// residues for compartment 3 as a function of A
	for (i = 0; i < _p->na; i++) a[i] =        _p->a31[i];
	CF2Convolve(nb, b, bl, _p->na, a, al, u, v);
	for (i = 0; i < nu; i++)               _p->a3a[i] = u[i];

	#define SHIFT_A(a) for (i = nu; --i >= 1;) _p->a[i] = _p->a[i-1];	_p->a[0] = 0
	// residues for compartment 1 as a function of 1
	SHIFT_A(a11);
	SHIFT_A(a21);
	SHIFT_A(a31);

	SHIFT_A(a12);
	SHIFT_A(a22);
	SHIFT_A(a32);

	SHIFT_A(a13);
	SHIFT_A(a23);
	SHIFT_A(a33);
	#undef  SHIFT_A

	for (i = 0; i < nu; i++) _p->x[i] = v[i];
	_p->na = nu;
}

void CF2Reset(_cf2machine* _p){CF2Clear(_p);}
void CF2SetTimeOfState(_cf2machine* _p, double _t){_p->dTimeOfState = _t;}
void CF2SetTimeOfGetVal(_cf2machine* _p, double _t){_p->dTimeOfGetVal = _t;}
void CF2Advance(_cf2machine* _p, double _dt){
	int i;
	double ta;

	double trateaa;
	double trate1a;
	double trate2a;
	double trate3a;

	double trate11;
	double trate21;
	double trate31;
	for (i=0; i<_p->na; i++){
		ASSERT(_p->x[i] < 0);
		ta = exp( _p->x[i] * _dt);

		trateaa = - _p->dRateA * _p->aaa[i] * (1-ta) / _p->x[i];
		trate1a = - _p->dRateA * _p->a1a[i] * (1-ta) / _p->x[i];
		trate2a = - _p->dRateA * _p->a2a[i] * (1-ta) / _p->x[i];
		trate3a = - _p->dRateA * _p->a3a[i] * (1-ta) / _p->x[i];

		trate11 = - _p->dRate1 * _p->a11[i] * (1-ta) / _p->x[i];
		trate21 = - _p->dRate1 * _p->a21[i] * (1-ta) / _p->x[i];
		trate31 = - _p->dRate1 * _p->a31[i] * (1-ta) / _p->x[i];

		_p->staa[i] = _p->staa[i] * ta + trateaa;

		_p->st1a[i] = _p->st1a[i] * ta + trate1a;
		_p->st11[i] = _p->st11[i] * ta + trate11;
		_p->st12[i] = _p->st12[i] * ta;
		_p->st13[i] = _p->st13[i] * ta;

		_p->st2a[i] = _p->st2a[i] * ta + trate2a;
		_p->st21[i] = _p->st21[i] * ta + trate21;
		_p->st22[i] = _p->st22[i] * ta;
		_p->st23[i] = _p->st23[i] * ta;

		_p->st3a[i] = _p->st3a[i] * ta + trate3a;
		_p->st31[i] = _p->st31[i] * ta + trate31;
		_p->st32[i] = _p->st32[i] * ta;
		_p->st33[i] = _p->st33[i] * ta;
	}

	_p->dTimeOfState += _dt;
	_p->dTimeOfGetVal = _p->dTimeOfState;
}

double CF2GetValA(_cf2machine* _p){
	double val = 0;
	double _staa0 = _p->staa[0];
	double _dt = (_p->dTimeOfGetVal - _p->dTimeOfState);
	if (_dt != 0){
		double ta = exp( _p->x[0] * _dt);
		double trateaa = - _p->dRateA * _p->aaa[0] * (1-ta) / _p->x[0];
		_staa0 = _staa0 * ta + trateaa;
	}
	val += _staa0;
	return val;
}
double CF2GetVal1(_cf2machine* _p){
	int i;
	double val = 0;
	double _st1a[5], _st11[5], _st12[5], _st13[5];
	double _dt = (_p->dTimeOfGetVal - _p->dTimeOfState);
	for (i=0; i<_p->na; i++){
		_st1a[i] = _p->st1a[i];
		_st11[i] = _p->st11[i];
		_st12[i] = _p->st12[i];
		_st13[i] = _p->st13[i];
	}
	if (_dt != 0){
		for (i = 0; i < _p->na; i++){
			double ta = exp( _p->x[i] * _dt);
			double trate1a = - _p->dRateA * _p->a1a[i] * (1-ta) / _p->x[i];
			double trate11 = - _p->dRate1 * _p->a11[i] * (1-ta) / _p->x[i];
			_st1a[i] = _p->st1a[i] * ta + trate1a;
			_st11[i] = _p->st11[i] * ta + trate11;
			_st12[i] = _p->st12[i] * ta;
			_st13[i] = _p->st13[i] * ta;
		}
	}
	for (i=0; i<_p->na; i++){
		val += _st1a[i];
		val += _st11[i];
		val += _st12[i];
		val += _st13[i];
	}
	return val;
}
double CF2GetVal2(_cf2machine* _p){
	int i;
	double val = 0;
	double _st2a[5], _st21[5], _st22[5], _st23[5];
	double _dt = (_p->dTimeOfGetVal - _p->dTimeOfState);
	for (i=0; i<_p->na; i++){
		_st2a[i] = _p->st2a[i];
		_st21[i] = _p->st21[i];
		_st22[i] = _p->st22[i];
		_st23[i] = _p->st23[i];
	}
	if (_dt != 0){
		for (i = 0; i < _p->na; i++){
			double ta = exp( _p->x[i] * _dt);
			double trate2a = - _p->dRateA * _p->a2a[i] * (1-ta) / _p->x[i];
			double trate21 = - _p->dRate1 * _p->a21[i] * (1-ta) / _p->x[i];
			_st2a[i] = _p->st2a[i] * ta + trate2a;
			_st21[i] = _p->st21[i] * ta + trate21;
			_st22[i] = _p->st22[i] * ta;
			_st23[i] = _p->st23[i] * ta;
		}
	}
	for (i=0; i<_p->na; i++){
		val += _st2a[i];
		val += _st21[i];
		val += _st22[i];
		val += _st23[i];
	}
	return val;
}
double CF2GetVal3(_cf2machine* _p){
	int i;
	double val = 0;
	double _st3a[5], _st31[5], _st32[5], _st33[5];
	double _dt = (_p->dTimeOfGetVal - _p->dTimeOfState);
	for (i=0; i<_p->na; i++){
		_st3a[i] = _p->st3a[i];
		_st31[i] = _p->st31[i];
		_st32[i] = _p->st32[i];
		_st33[i] = _p->st33[i];
	}
	if (_dt != 0){
		for (i = 0; i < _p->na; i++){
			double ta = exp( _p->x[i] * _dt);
			double trate3a = - _p->dRateA * _p->a3a[i] * (1-ta) / _p->x[i];
			double trate31 = - _p->dRate1 * _p->a31[i] * (1-ta) / _p->x[i];
			_st3a[i] = _p->st3a[i] * ta + trate3a;
			_st31[i] = _p->st31[i] * ta + trate31;
			_st32[i] = _p->st32[i] * ta;
			_st33[i] = _p->st33[i] * ta;
		}
	}
	for (i=0; i<_p->na; i++){
		val += _st3a[i];
		val += _st31[i];
		val += _st32[i];
		val += _st33[i];
	}
	return val;
}
double CF2GetRateA(_cf2machine* _p){return _p->dRateA;}
double CF2GetRate1(_cf2machine* _p){return _p->dRate1;}

void CF2SetValA(_cf2machine* _p, double _dv){
	int i;
	for (i=0; i<_p->na; i++) _p->staa[i] = _dv * _p->aaa[i]; // set state for how A depends on A
	for (i=0; i<_p->na; i++) _p->st1a[i] = _dv * _p->a1a[i]; // set state for how 1 depends on A
	for (i=0; i<_p->na; i++) _p->st2a[i] = _dv * _p->a2a[i]; // set state for how 2 depends on A
	for (i=0; i<_p->na; i++) _p->st3a[i] = _dv * _p->a3a[i]; // set state for how 3 depends on A
}
void CF2SetVal1(_cf2machine* _p, double _dv){
	int i;
	for (i=0; i<_p->na; i++) _p->st11[i] = _dv * _p->a11[i]; // set state for how 1 depends on 1
	for (i=0; i<_p->na; i++) _p->st21[i] = _dv * _p->a21[i]; // set state for how 2 depends on 1
	for (i=0; i<_p->na; i++) _p->st31[i] = _dv * _p->a31[i]; // set state for how 3 depends on 1
}
void CF2SetVal2(_cf2machine* _p, double _dv){
	int i;
	for (i=0; i<_p->na; i++) _p->st12[i] = _dv * _p->a12[i]; // set state for how 1 depends on 2
	for (i=0; i<_p->na; i++) _p->st22[i] = _dv * _p->a22[i]; // set state for how 2 depends on 2
	for (i=0; i<_p->na; i++) _p->st32[i] = _dv * _p->a32[i]; // set state for how 3 depends on 2
}
void CF2SetVal3(_cf2machine* _p, double _dv){
	int i;
	for (i=0; i<_p->na; i++) _p->st13[i] = _dv * _p->a13[i]; // set state for how 1 depends on 3
	for (i=0; i<_p->na; i++) _p->st23[i] = _dv * _p->a23[i]; // set state for how 2 depends on 3
	for (i=0; i<_p->na; i++) _p->st33[i] = _dv * _p->a33[i]; // set state for how 3 depends on 3
}
void CF2SetRateA(_cf2machine* _p, double _dr){_p->dRateA = _dr;}
void CF2SetRate1(_cf2machine* _p, double _dr){_p->dRate1 = _dr;}

void CF2AddValA(_cf2machine* _p, double _dv){
	int i;
	for (i=0; i<_p->na; i++) _p->staa[i] += _dv * _p->aaa[i]; // set state for how A depends on A
	for (i=0; i<_p->na; i++) _p->st1a[i] += _dv * _p->a1a[i]; // set state for how 1 depends on A
	for (i=0; i<_p->na; i++) _p->st2a[i] += _dv * _p->a2a[i]; // set state for how 2 depends on A
	for (i=0; i<_p->na; i++) _p->st3a[i] += _dv * _p->a3a[i]; // set state for how 3 depends on A
}
void CF2AddVal1(_cf2machine* _p, double _dv){
	int i;
	for (i=0; i<_p->na; i++) _p->st11[i] += _dv * _p->a11[i]; // set state for how 1 depends on 1
	for (i=0; i<_p->na; i++) _p->st21[i] += _dv * _p->a21[i]; // set state for how 2 depends on 1
	for (i=0; i<_p->na; i++) _p->st31[i] += _dv * _p->a31[i]; // set state for how 3 depends on 1
}
void CF2AddVal2(_cf2machine* _p, double _dv){
	int i;
	for (i=0; i<_p->na; i++) _p->st12[i] += _dv * _p->a12[i]; // set state for how 1 depends on 2
	for (i=0; i<_p->na; i++) _p->st22[i] += _dv * _p->a22[i]; // set state for how 2 depends on 2
	for (i=0; i<_p->na; i++) _p->st32[i] += _dv * _p->a32[i]; // set state for how 3 depends on 2
}
void CF2AddVal3(_cf2machine* _p, double _dv){
	int i;
	for (i=0; i<_p->na; i++) _p->st13[i] += _dv * _p->a13[i]; // set state for how 1 depends on 3
	for (i=0; i<_p->na; i++) _p->st23[i] += _dv * _p->a23[i]; // set state for how 2 depends on 3
	for (i=0; i<_p->na; i++) _p->st33[i] += _dv * _p->a33[i]; // set state for how 3 depends on 3
}
void CF2AddRateA(_cf2machine* _p, double _dr){_p->dRateA += _dr;}
void CF2AddRate1(_cf2machine* _p, double _dr){_p->dRate1 += _dr;}

/****
TODO: When we get around to it:
How to convert from matrix exponent form to closed form.
Here's the general form of the Jacobian (not counting the infusion column)
General 3-compartment, 1st-order, with effect cpt:

			A1					A2		A3		Aa		Ce
		-------------------------------------------------------
A1'		|	-K10 -K13 -K12	  |	+K21  |	+K31  |	F*Ka  |		  |
		-------------------------------------------------------
A2'		|	+K12			  |	-K21  |		  |		  |		  |
		-------------------------------------------------------
A3'		|	+K13			  |		  |	-K31  |		  |		  |
		-------------------------------------------------------
Aa'		|	-Ka				  |		  |		  |		  |		  |
		-------------------------------------------------------
Ce'		|	+KCe/V1			  |		  |		  |		  |	-KCe  |
		-------------------------------------------------------

1) identify peripheral compartments, in this case columns 2 & 3. (get K21 & K31) Suggesting 1 is central.
2) identify absorption compartment, int this case column 4 and row 4, suggesting central compartment, rowcol 1.
   (get Ka and F)
3) identify effect compartment, in this case rowcol 4, suggesting 1 as central.
   (get KCe and V1)  Confirm V1 is correct.
4) even without absorption and central cpt suggestions, the remaining rowcol is 1. (Get K10)
5) confirm all Ks are >0, F>0, V1>0, V1 = computed volume of central cpt.
6) make sure none of K10, K12, K21, K13, K31 are equal.

Simple 1-compartment, 1st-order:

			A1		Aa
		-------------------
A1'		|	-K10  |	F*Ka  |
		-------------------
Aa'		|	-Ka	  |		  |
		-------------------

In this case, rules 1 & 3 fail.

Simple 1-compartment, with effect cpt:

			A1			Ce
		-----------------------
A1'		|	-K10	  |		  |
		-----------------------
Ce'		|	+KCe/V1	  |	-KCe  |
		-----------------------

In this case, rules 1 & 2 fail.

Only do this at start of subject, and it is not valid if jacobian changes during subject.
If there is a Ce, then no other compartment may be referred to.
If there is not a Ce, then only A1 may be referred to.
Otherwise cannot use closed form.
If there is an absorption compartment, may be necessary to have 2 superimposed closed form machines, one for
vascular doses, and one for non-vascular.

****/

void MatrixExponentTaylorSeries(double inmat[], double result[], double tolerance){
	#define N (NINTEG_ALL + 1)
	double k;
	double term[N*N];
	double temp[N*N];
	// clear the result matrix
	_MatZero(N, N, result);
	// first term is identity matrix
	_MatIdentity(N, term);
	// for k = 1, ...
	for (k=1; k<100; k++){
		// add term to the result
		_MatAddTo(N, N, term, result);
		// see if we need more terms
		if (MatInTolerance(N, N, term, tolerance)) break;
		// multiply the term by the matrix, and divide by k
		_SqMatMulTo(N, inmat, term, temp);
		_MatScaleTo(N, N, temp, term, 1.0/k);
	}
	#undef N
}

// see if this speeds things up
int _bOldMatInitialized;
#define N (NINTEG_ALL + 1)
double _oldmat[N*N];
double _oldexpmat[N*N];
#undef N

// set result = exp(jacobian * dt)
void MatrixExponent(double dt, double* result, int* pError){
	#define N (NINTEG_ALL + 1)
	int nHalve = 0;
	double tolerance = 1e-10;
	int i;
//	double k;
	double inmat[N*N];
//	double term[N*N];
	double temp[N*N];
	double temp1[N*N];
	double dLargestElement = 0;
	double dScaleDown = 1;
//	if (!_finite(_oldexpmat[0])){
//		CallDebug(500, _oldexpmat[0]);
//	}
	*pError = 0;
	// scale the jacobian by delta-time
//	if (!_finite(_jacobian[0])) CallDebug(300, _jacobian[0]);
//	if (!_finite(dt)) CallDebug(301, dt);
	_MatScaleTo(N, N, _jacobian, inmat, dt);
//	if (!_finite(inmat[0])) CallDebug(302, inmat[0]);
	// if we have a prior matrix and result
	if (_bOldMatInitialized){ // normal code
//	if (_bOldMatInitialized && _finite(_oldexpmat[0])){ // workaround nan bug
		// if current matrix is equal to the old matrix, use the old result
		if (MatEqual(N, N, inmat, _oldmat)){
			_MatCopyTo(N, N, _oldexpmat, result);
			return;
		}
		// form the matrix temp = inmat - _oldmat (the difference matrix)
//		if (!_finite(_oldmat[0])) CallDebug(303, _oldmat[0]);
		_MatCopyTo(N, N, inmat, temp);
		_MatSubFrom(N, N, _oldmat, temp);
		// see how big it is
		dLargestElement = MatLargest(N, N, temp);
		// if the difference is not too great
		if (dLargestElement < 0.1){
			// form the matrix temp1 = exp(temp)
			MatrixExponentTaylorSeries(temp, temp1, tolerance);
			// form result = exp(oldmat) * exp(inmat - oldmat)
			_SqMatMulTo(N, _oldexpmat, temp1, result);
			// check for errors
			if (!_MatFinite(N, N, temp)){
//				CallDebug(4981, temp[0]);
//				CallDebug(4982, temp1[0]);
//				CallDebug(4983, inmat[0]);
//				CallDebug(4984, _oldmat[0]);
//				CallDebug(4985, dLargestElement);
				*pError = 1;
				return;
			}
			// record the result
			_MatCopyTo(N, N, inmat, _oldmat);
			_MatCopyTo(N, N, result, _oldexpmat); _bOldMatInitialized = 1;
			return;
		}
	}

	// if not, record the mat and expmat
	_MatCopyTo(N, N, inmat, _oldmat);
	_MatZero(N, N, _oldexpmat);

	// halve the matrix until its largest element is <= 1
	dLargestElement = MatLargest(N, N, inmat);
	if (!_finite(dLargestElement)){
//		CallDebug(499, dLargestElement);
		*pError = 1;
		return;
	}
	while(dLargestElement > 1){
		dLargestElement *= 0.5;
		dScaleDown *= 0.5;
		nHalve++;
	}
	if (nHalve > 0){
		_MatScaleTo(N, N, inmat, inmat, dScaleDown);
	}
	// run the basic taylor series
	MatrixExponentTaylorSeries(inmat, result, tolerance);
	// square the matrix the number of times that it was halved
	for (i=0; i<nHalve; i++){
		_SqMatMulTo(N, result, result, temp);
		_MatCopyTo(N, N, temp, result);
	}
	// record the result
	_MatCopyTo(N, N, result, _oldexpmat); _bOldMatInitialized = 1;
//	if (!_finite(_oldexpmat[0])){
//		CallDebug(501, _oldexpmat[0]);
//	}
	// check for errors
	if (!_MatFinite(N, N, temp)){
//		CallDebug(502, _oldexpmat[0]);
		*pError = 1;
		return;
	}
	#undef  N
}

void AdvanceByMatrixExponent(double dt){
	int Error = 0;
	// size of matrix is number of integrators plus 1
	#define N (NINTEG_ALL + 1)
	double exmat[N * N];
	double tempstate[N];
//	if (!_MatFinite(N, N, _jacobian)){
//		SetErrorMsg("Error: non-finite terms in differential equations");
//		return;
//	}
	MatrixExponent(dt, exmat, &Error);
	if (Error){
		SetErrorMsg("Error: non-finite terms in differential equations");
		return;
	}
	// make sure extra element of state is 1, so that infusions get done
	zzY[NINTEG_ALL] = 1;
	// state = exmat * state
	_MatMulTo(N, N, exmat, 1, zzY, tempstate);
	memcpy(zzY, tempstate, sizeof(double)*N);
	#undef  N
}

int* _nPrediction;
double* _daXPrediction;
double* _taLastDose;
int  * _iaWhichDose;
int  * _iaWhichReset;
double* _daYPrediction;
double* _daYObservation;
double* _daVarianceFunction;
int  * _iaWhichObs;

double* _daYdK;
double* _daSdK;
int _nEta;

void SetPredArrays(
	int* __nPrediction
	, double* __daXPrediction
	, double* __taLastDose
	, int  * __iaWhichDose
	, int  * __iaWhichReset
	, double* __daYPrediction
	, double* __daYObservation
	, double* __daVarianceFunction
	, int  * __iaWhichObs
	)
{
	_nPrediction			= __nPrediction;
	_daXPrediction			= __daXPrediction;
	_taLastDose				= __taLastDose;
	_iaWhichDose			= __iaWhichDose;
	_iaWhichReset			= __iaWhichReset;
	_daYPrediction			= __daYPrediction;
	_daYObservation			= __daYObservation;
	_daVarianceFunction		= __daVarianceFunction;
	_iaWhichObs		= __iaWhichObs;
	// these are null unless set by SetGradArrays
	_nEta = 0;
	_daYdK = NULL;
	_daSdK = NULL;
}

void SetGradArrays(int __nEta, double* __daYdK, double* __daSdK){
	_nEta = __nEta;
	_daYdK = __daYdK;
	_daSdK = __daSdK;
}

void RecordObservation(
	double dXPred
	, double tLastDose
	, int iWhichDose
	, int iWhichReset
	, double dYPred
	, double dYObs
	, double dVFunc
	, int iWhichObs
	)
{
	int j = * _nPrediction;
	_daXPrediction[j] = dXPred;
	_taLastDose[j] = tLastDose;
	_iaWhichDose[j] = iWhichDose;
	_iaWhichReset[j] = iWhichReset;
	_daYPrediction[j] = dYPred;
	_daYObservation[j] = dYObs;
	_daVarianceFunction[j] = dVFunc;
	_iaWhichObs[j] = iWhichObs;
	(* _nPrediction)++;
}

void RecordObservationGradient(double dYdK, double dSdK, int iWhichK){
	int j = (* _nPrediction) - 1;
	if (_nEta > 0){
		if (_daYdK != NULL){
			_daYdK[j * _nEta + iWhichK] = dYdK;
		}
		if (_daSdK != NULL){
			_daSdK[j * _nEta + iWhichK] = dSdK;
		}
	}
}

#define MAX_HIST 10000
double hist[MAX_HIST];
int iHist = 0;
int bEnableHistBreak = 0;
void InitHist(){iHist = 0;}
void EnableHistBreak(int *pb){bEnableHistBreak = *pb;}
void RecordHist(double d){
	if (iHist >= MAX_HIST) return;
	if (bEnableHistBreak && d != hist[iHist]){
		int breakhere = 1;
	}
	hist[iHist] = d;
	iHist++;
}

#define _NA 3.3e-151

#define pc (*ppc)
void ScanOverRP(const unsigned char* pc){
	unsigned char c;
	int iLevel = 1;
	while(iLevel && (c = *pc)){
		pc++;
		switch(c){
		case 255:;
		break;	case OP_LP: iLevel++;
		break;	case OP_RP: iLevel--;
		break;	case OP_NUM: pc += sizeof(double);

		break;
				case OP_BOLUS1:
				case OP_BOLUS2:
				case OP_INF1:
				case OP_INF2:
				case OP_OBS:
				case OP_OBS_BQL:
				case OP_COVR:
				case OP_DCOVR:
				case OP_FCOVR:
				case OP_DEF:
				case OP_SET:
				case OP_REF:
					while(*pc++);

		break;
				case OP_CLR:
				case OP_DT:
				case OP_RESET:
				case OP_RPT:
				case OP_SS:
				case OP_SS1:
				case OP_SS2:
				case OP_SSEND:
				case OP_SSNM:
				case OP_ADDL:
				case OP_ADDLNM:
				case OP_DUP:
				case OP_COMMA:
				case OP_ADD:
				case OP_SUB:
				case OP_MUL:
				case OP_DIV:
				case OP_POW:
				case OP_QUEST:

		break; default:
			ASSERT(0);
		}
	}
}
#undef  pc

#define MAXSTACK 100

int ndStack;
double dStack[MAXSTACK];
int npcStack;
const unsigned char* pcStack[MAXSTACK];

#define FLAG_POW 16
#define FLAG_DIV 8
#define FLAG_MUL 4
#define FLAG_SUB 2
#define FLAG_ADD 1
int flags;

void SaveEventQueue(){
	zzNAction_Bak = zzNAction;
	memcpy(zzDT_Bak, zzDT, zzNAction * sizeof(zzDT[0]));
	memcpy(zzPriority_Bak, zzPriority, zzNAction * sizeof(zzPriority[0]));
	memcpy(zzActionCode_Bak, zzActionCode, zzNAction * sizeof(zzActionCode[0]));
	memcpy(zzActionIArg_Bak, zzActionIArg, zzNAction * sizeof(zzActionIArg[0]));
	memcpy(zzActionDArg1_Bak, zzActionDArg1, zzNAction * sizeof(zzActionDArg1[0]));
	memcpy(zzActionDArg2_Bak, zzActionDArg2, zzNAction * sizeof(zzActionDArg2[0]));
	memcpy(zzActionPArg_Bak, zzActionPArg, zzNAction * sizeof(zzActionPArg[0]));
	zzNAction = 0;
}

void RestoreEventQueue(){
	zzNAction = zzNAction_Bak;
	memcpy(zzDT, zzDT_Bak, zzNAction * sizeof(zzDT[0]));
	memcpy(zzPriority, zzPriority_Bak, zzNAction * sizeof(zzPriority[0]));
	memcpy(zzActionCode, zzActionCode_Bak, zzNAction * sizeof(zzActionCode[0]));
	memcpy(zzActionIArg, zzActionIArg_Bak, zzNAction * sizeof(zzActionIArg[0]));
	memcpy(zzActionDArg1, zzActionDArg1_Bak, zzNAction * sizeof(zzActionDArg1[0]));
	memcpy(zzActionDArg2, zzActionDArg2_Bak, zzNAction * sizeof(zzActionDArg2[0]));
	memcpy(zzActionPArg, zzActionPArg_Bak, zzNAction * sizeof(zzActionPArg[0]));
	zzNAction_Bak = 0;
}

void ClearActions(){
	zzNAction = 0;
}

void InsertAction(double dt, int priority, int code, int iarg, double darg1, double darg2, void* parg){
	int i, j;
	if (zzNAction >= ZZMAX_ACTION){
		return;	// TODO: handle error
	}
	// find where to insert this action
	for (i=0; i<zzNAction; i++){
		if (zzDT[i] < dt) continue;
		if (zzDT[i] == dt && zzPriority[i] <= priority) continue;
		break;
	}
	// i is the index to hold the new action
	// make space for it
	for (j=zzNAction; --j >= i;){
		zzDT[j+1] = zzDT[j];
		zzPriority[j+1] = zzPriority[j];
		zzActionCode[j+1] = zzActionCode[j];
		zzActionIArg[j+1] = zzActionIArg[j];
		zzActionDArg1[j+1] = zzActionDArg1[j];
		zzActionDArg2[j+1] = zzActionDArg2[j];
		zzActionPArg[j+1] = zzActionPArg[j];
	}
	// insert it
	zzDT[i] = dt;
	zzPriority[i] = priority;
	zzActionCode[i] = code;
	zzActionIArg[i] = iarg;
	zzActionDArg1[i] = darg1;
	zzActionDArg2[i] = darg2;
	zzActionPArg[i] = parg;
	zzNAction++;
}

void SetFlag(int flg){
	if (flags==0 && flg==0) return;
	if ((flags >= flg) && (flags & FLAG_POW)){
		dStack[ndStack-2] = pow(dStack[ndStack-2], dStack[ndStack-1]);
		ndStack--;
		flags &= ~FLAG_POW;
	}
	if ((flags >= flg) && (flags & FLAG_DIV)){
		dStack[ndStack-2] = dStack[ndStack-2] / dStack[ndStack-1];
		ndStack--;
		flags &= ~FLAG_DIV;
	}
	if ((flags >= flg) && (flags & FLAG_MUL)){
		dStack[ndStack-2] = dStack[ndStack-2] * dStack[ndStack-1];
		ndStack--;
		flags &= ~FLAG_MUL;
	}
	if ((flags >= flg) && (flags & FLAG_SUB)){
		dStack[ndStack-2] = dStack[ndStack-2] - dStack[ndStack-1];
		ndStack--;
		flags &= ~FLAG_SUB;
	}
	if ((flags >= flg) && (flags & FLAG_ADD)){
		dStack[ndStack-2] = dStack[ndStack-2] + dStack[ndStack-1];
		ndStack--;
		flags &= ~FLAG_ADD;
	}
	flags |= flg;
}

double curTime, curTimeLastDose;
int iCurWhichDose;
int iCurWhichReset;
double _dMaxTime = -1;
BOOL _bOnTrajectory;

void SetMaxTime(double d){
	_dMaxTime = d;
}
double GetMaxTime(){return _dMaxTime;}

int _bDroppingOut;
int _bInSS;
int  GetInSS(){return _bInSS;}

void RemoveActionZero(){
	int i;
	for (i=0; i<zzNAction-1; i++){
		zzDT[i] = zzDT[i+1];
		zzPriority[i] = zzPriority[i+1];
		zzActionCode[i] = zzActionCode[i+1];
		zzActionIArg[i] = zzActionIArg[i+1];
		zzActionDArg1[i] = zzActionDArg1[i+1];
		zzActionDArg2[i] = zzActionDArg2[i+1];
		zzActionPArg[i] = zzActionPArg[i+1];
	}
	zzNAction--;
}

void AdvanceWithActions(int bGenGradients, double *pTimeOuter, double dt, int bTimeAdvances, int bFinishUpSubject){
	int i, iCpt;
	int bDoEndRow = 0;
	int bDidSomething = 1;
	char szCompartmentName[512];
	// while there is more time to expire
	SetModelTime(*pTimeOuter);
	while(_finite(dt) && (dt > 0 || bDidSomething) && !_bDroppingOut){
		double drmin = dt;
		SetModelTime(*pTimeOuter);
		bDidSomething = 0;
		// get time to next action
		if (zzNAction > 0 && drmin > zzDT[0]) drmin = zzDT[0];
		// advance for the minimum delta time
		ModelBreakPoint();
		
		if (drmin > 0){
			double tim = *pTimeOuter;
//			SetupJacobian(bGenGradients);
			Advance(bGenGradients, iODELevel, &tim, drmin, bTimeAdvances);
			if (bTimeAdvances) *pTimeOuter += drmin;
			SetModelTime(*pTimeOuter);
		}
		ModelBreakPoint();
		// subtract it from the remaining drip and dose times
		if (drmin != 0){
			for (i=0; i<zzNAction; i++){
				zzDT[i] -= drmin;
				if (zzDT[i] < 0) zzDT[i] = 0;	// try to correct for roundoff
			}
		}

		// for each action to be done at this time
		// (stop short of actions at the end with priority > 1)
		// This is so that TIME actions don't get done too soon.
		bDoEndRow = 0;
		while(zzNAction > 0
			&& zzDT[0] == 0
			&& (dt > 0 || zzPriority[0] <= 1 || bFinishUpSubject)
			)
		{
			switch (zzActionCode[0]){
			break; case ZZACTION_DOSETIME:;
				curTimeLastDose = curTime;
				iCurWhichDose++;
			// if bolus, do so
			break; case ZZACTION_BOLUS1:;
				iCpt = zzActionIArg[0];
				PerformBolus1(&curTime, zzY, zzR, iCpt, zzActionDArg1[0], zzActionDArg2[0]);	// routine defined in model
			break; case   ZZMARK_BOLUS1:;
				iCpt = zzActionIArg[0];
				GetCompartmentName(&iCpt, szCompartmentName);
				if (_pcb) (*_pcb->_pfOnBolus)(curTime, curTimeLastDose, 1, szCompartmentName, zzActionDArg1[0]);
			break; case ZZACTION_BOLUS2:;
				iCpt = zzActionIArg[0];
				PerformBolus2(&curTime, zzY, zzR, iCpt, zzActionDArg1[0], zzActionDArg2[0]);	// routine defined in model
			break; case   ZZMARK_BOLUS2:;
				iCpt = zzActionIArg[0];
				GetCompartmentName(&iCpt, szCompartmentName);
				if (_pcb) (*_pcb->_pfOnBolus)(curTime, curTimeLastDose, 2, szCompartmentName, zzActionDArg1[0]);
			// if change in infusion rate, do so
			break; case ZZACTION_INFSTRT1:;
				iCpt = zzActionIArg[0];
				PerformInfStrt1(&curTime, zzY, zzR, iCpt, zzActionDArg1[0]);	// routine defined in model
			break; case   ZZMARK_INFSTRT1:;
				iCpt = zzActionIArg[0];
				GetCompartmentName(&iCpt, szCompartmentName);
				if (_pcb) (*_pcb->_pfOnInfusion)(curTime, curTimeLastDose, 1, szCompartmentName, zzActionDArg1[0], zzActionDArg2[0]);
			break; case ZZACTION_INFSTRT2:;
				iCpt = zzActionIArg[0];
				PerformInfStrt2(&curTime, zzY, zzR, iCpt, zzActionDArg1[0]);	// routine defined in model
			break; case   ZZMARK_INFSTRT2:;
				iCpt = zzActionIArg[0];
				GetCompartmentName(&iCpt, szCompartmentName);
				if (_pcb) (*_pcb->_pfOnInfusion)(curTime, curTimeLastDose, 2, szCompartmentName, zzActionDArg1[0], zzActionDArg2[0]);
			// if change in infusion rate, do so
			break; case ZZACTION_INFEND1:;
				iCpt = zzActionIArg[0];
				PerformInfEnd1(&curTime, zzY, zzR, iCpt, zzActionDArg1[0]);	// routine defined in model
			break; case   ZZMARK_INFEND1:;
			break; case ZZACTION_INFEND2:;
				iCpt = zzActionIArg[0];
				PerformInfEnd2(&curTime, zzY, zzR, iCpt, zzActionDArg1[0]);	// routine defined in model
			break; case   ZZMARK_INFEND2:;
			break; case   ZZMARK_ENDROW:;
				bDoEndRow = 1;
			// if wakeup from sleep, do so
			break; case ZZACTION_SLEEP:{
					_statemachine* p = (_statemachine*)zzActionPArg[0];
					if (p->_iState >= 0){
						p->_func(p, curTime, zzY);
					}
				}
			// if time callback, do so
			break; case ZZACTION_TIME:;
				if (_pcb && _pcb->_pfOnTime){
					_pcb->_pfOnTime(curTime, curTimeLastDose);
				}
			} // end switch
			// remove this action
			RemoveActionZero();
			bDidSomething = 1;
		} // end for each action at this time
		if (bDoEndRow){
			bDoEndRow = 0;
			if (_pcb && _pcb->_pfOnEndRow)(*_pcb->_pfOnEndRow)();
		}
		// subtract it from the total time
		dt -= drmin;
		if (dt < 0) dt = 0;	// try to correct for roundoff
	} // end while there is more time to expire
	// if dropping out, delete all actions
	if (_bDroppingOut){
		zzNAction = 0;
		memset(zzDT,          0, sizeof(zzDT         ));
		memset(zzPriority,    0, sizeof(zzPriority   ));
		memset(zzActionCode,  0, sizeof(zzActionCode ));
		memset(zzActionIArg,  0, sizeof(zzActionIArg ));
		memset(zzActionDArg1, 0, sizeof(zzActionDArg1));
		memset(zzActionDArg2, 0, sizeof(zzActionDArg2));
		memset(zzActionPArg,  0, sizeof(zzActionPArg ));
	}
	ModelBreakPoint();
}

void InterpSS(int bGenGradients, const unsigned char* pc, int* pbOK, int bTimeAdvances, int bFirstTimeThrough){
	int iCpt = -1;
	double v1, v2, dNum;
	//int bOK = 1;
	char idbuf[4096];
	char* q;
	const unsigned char* pc1;
	const double *pdTemp = NULL;
	while (*pc){
		ModelBreakPoint();
		switch(*pc++){
			case 255:;
			break; case OP_SS1:{
				if (_pcb && bFirstTimeThrough) (*_pcb->_pfOnSSi)(1);
			}
			break; case OP_SS2:{
				if (_pcb && bFirstTimeThrough) (*_pcb->_pfOnSSi)(2);
			}
			break; case OP_SSEND:{
				if (_pcb && bFirstTimeThrough) (*_pcb->_pfOnSSi)(0);
			}
			break; case OP_RP:{
				SetFlag(0);
				return;
			}
			break; case OP_QUEST:{
				dStack[ndStack++] = _UNK;	// unknown value
			}
			break; case OP_LP:{
				pc1 = pc;
				ScanOverRP(&pc);
				// save pc1
				pcStack[npcStack++] = pc1;
			}
			break; case OP_DT:{
				SetFlag(0);
				v1 = dStack[--ndStack];
				// delay by v1
	// ??		EvalSParms(zzY);
//				SetupJacobian(bGenGradients);
				if (_finite(v1)){
					AdvanceWithActions(bGenGradients, &curTime, v1, bTimeAdvances, 0);
				}
			}
			break; case OP_BOLUS1:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v1 = dStack[--ndStack];
				// bolus to variable
				iCpt = GetCompartmentNumber(idbuf);
				ScheduleBolus1(bFirstTimeThrough, 0, iCpt, v1, 0);
//				SetupJacobian(bGenGradients);
				AdvanceWithActions(bGenGradients, &curTime, 0, bTimeAdvances, 0);
			}
			break; case OP_BOLUS2:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v1 = dStack[--ndStack];
				// bolus to variable
				iCpt = GetCompartmentNumber(idbuf);
				ScheduleBolus2(bFirstTimeThrough, 0, iCpt, v1, 0);
//				SetupJacobian(bGenGradients);
				AdvanceWithActions(bGenGradients, &curTime, 0, bTimeAdvances, 0);
			}
			break; case OP_INF1:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v2 = dStack[--ndStack];	// rate
				v1 = dStack[--ndStack];	// amt
				iCpt = GetCompartmentNumber(idbuf);
				ScheduleIV1(bFirstTimeThrough, 0, iCpt, v1, v2, 0);
//				SetupJacobian(bGenGradients);
				AdvanceWithActions(bGenGradients, &curTime, 0, bTimeAdvances, 0);
			}
			break; case OP_INF2:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v2 = dStack[--ndStack];	// rate
				v1 = dStack[--ndStack];	// amt
				iCpt = GetCompartmentNumber(idbuf);
				ScheduleIV2(bFirstTimeThrough, 0, iCpt, v1, v2, 0);
//				SetupJacobian(bGenGradients);
				AdvanceWithActions(bGenGradients, &curTime, 0, bTimeAdvances, 0);
			}
			break; case OP_RPT:{
				SetFlag(0);
				if (dStack[ndStack-1] <= 0){
					ndStack--;
					npcStack--;
				} else {
					pc -= 1;	// will redo the op
					dStack[ndStack-1] -= 1;
					InterpSS(bGenGradients, pcStack[npcStack-1], pbOK, bTimeAdvances, bFirstTimeThrough);
				}
			}
			break; case OP_NUM:{
				//dNum = *((double*)pc)++;
				pdTemp = (const double*)pc; dNum = *pdTemp++; pc = (const unsigned char*)pdTemp;
				dStack[ndStack++] = dNum;
			}
			break; case OP_DUP:{
				dNum = dStack[ndStack-1];
				dStack[ndStack++] = dNum;
			}
			break; case OP_COMMA: SetFlag(0);
			break; case OP_ADD: SetFlag(FLAG_ADD);
			break; case OP_SUB: SetFlag(FLAG_SUB);
			break; case OP_MUL: SetFlag(FLAG_MUL);
			break; case OP_DIV: SetFlag(FLAG_DIV);
			break; case OP_POW: SetFlag(FLAG_POW);
			break; default:
			{
				int breakhere = 1;
			}
		}
	}
}

int iWhichSSCall;

void GetSS(int bGenGradients, const unsigned char* pc, int* pbOK, double* params){
	double zvec[NINTEG_ALL+NCFMACRO*6+1];
	double y0[NINTEG_ALL+NCFMACRO*6+1];
	double y1[NINTEG_ALL+NCFMACRO*6+1];
	double y2[NINTEG_ALL+NCFMACRO*6+1];
	double y3[NINTEG_ALL+NCFMACRO*6+1];
	double dy01, dy02, dy1z, vec[NINTEG_ALL+NCFMACRO*6+1], _a, _absa;
	int i, j, bTrySimpleWay = 0, bSSFailed = 0;
	double _tol = 1e-5;
	iWhichSSCall++;
	memset(zvec, 0, sizeof(zvec));
//	double temp = 0, maxtemp = 0;
	SaveEventQueue();
	// should be able to start from any state
	// Serge had a model where if you start from state 0 it won't reach SS
	//ResetInteg();
	// first estimate a reasonable tolerance
//		for (i=0; i<NINTEG_ALL; i++) y0[i] = zzY[i];
//		for (i=0; i<NINTEG_ALL; i++){temp = fabs(y0[i]); if (maxtemp < temp) maxtemp = temp;}
//		if (maxtemp > 1e-7) _tol = maxtemp * 1e-7;

	// first run two cycles to get in the ballpark
	if (pbOK && *pbOK == 0) return;
	InterpSS(bGenGradients, pc, pbOK, FALSE, TRUE);
	if (pbOK && *pbOK == 0) return;
	InterpSS(bGenGradients, pc, pbOK, FALSE, FALSE);
	if (pbOK && *pbOK == 0) return;
	for(j=0; j<20 && !bTrySimpleWay; j++){
		for (i=0; i<NINTEG_ALL; i++) y0[i] = zzY[i];
		GetAllCFState(y0+NINTEG_ALL);
		ClearSSIrrelevantState(y0);

		InterpSS(bGenGradients, pc, pbOK, FALSE, FALSE);
		if (pbOK && *pbOK == 0) return;
		for (i=0; i<NINTEG_ALL; i++) y1[i] = zzY[i];
		GetAllCFState(y1+NINTEG_ALL);
		ClearSSIrrelevantState(y1);

		dy01 = vdis(NINTEG_ALL+NCFMACRO*6, y0, y1);
		dy1z = vdis(NINTEG_ALL+NCFMACRO*6, y1, zvec);

		// check for convergence
		if (dy01 < _tol * dy1z) break;

		InterpSS(bGenGradients, pc, pbOK, FALSE, FALSE);
		if (pbOK && *pbOK == 0) return;
		for (i=0; i<NINTEG_ALL; i++) y2[i] = zzY[i];
		GetAllCFState(y2+NINTEG_ALL);
		ClearSSIrrelevantState(y2);

		dy02 = vdis(NINTEG_ALL+NCFMACRO*6, y0, y2);
		// guard against zero-divide
		if (2*dy01 - dy02 == 0){
			bTrySimpleWay = 1;
			break;
		}
		// get vector y2 - y0
		for (i=0; i<NINTEG_ALL+NCFMACRO*6; i++) vec[i] = (y2[i] - y0[i]);

		// extrapolate
		_a = (dy01 * dy01) / (2 * dy01 - dy02);
		// if it's suspiciously large, give up extrapolation
		_absa = fabs(_a);
		if (_absa > 5 * dy02){
			bTrySimpleWay = 1;
			break;
		}
		// do the extrapolation
		for (i=0; i<NINTEG_ALL+NCFMACRO*6; i++) y3[i] = y0[i] + vec[i]/dy02 * _a;
		// if any closed-form component is negative, give up extrapolation
		for (i=NINTEG_ALL; i<NINTEG_ALL+NCFMACRO*6; i++){
			if (y3[i] < 0){
				bTrySimpleWay = 1;
				break;
			}
		}

		for (i=0; i<NINTEG_ALL; i++) zzY[i] = y3[i]; SetAllCFState(y3+NINTEG_ALL);
	}
	if (j >= 20) bTrySimpleWay = 1;
	if (bTrySimpleWay){
		bSSFailed = 0;
		for (i=0; i<NINTEG_ALL; i++) y0[i] = zzY[i];
		GetAllCFState(y0+NINTEG_ALL);
		ClearSSIrrelevantState(y0);

		// allow it to cycle up to this number of times before giving up
		// in case it's a really bad but still possible case
		for (j = 1000; --j>=0;){
			if (pbOK && *pbOK == 0) return;
			InterpSS(bGenGradients, pc, pbOK, FALSE, FALSE);
			if (pbOK && *pbOK == 0) return;
			for (i=0; i<NINTEG_ALL; i++) y1[i] = zzY[i];
			GetAllCFState(y1+NINTEG_ALL);
			ClearSSIrrelevantState(y1);

			dy01 = vdis(NINTEG_ALL+NCFMACRO*6, y0, y1);
			dy1z = vdis(NINTEG_ALL+NCFMACRO*6, y1, zvec);

			// check for convergence
			if (dy01 < _tol * dy1z) break;

			for (i=0; i<NINTEG_ALL + NCFMACRO*6; i++) y0[i] = y1[i];
		}
		if (j < 0){
			bSSFailed = 1;
		}
	}
	// it's a choice between failing and stopping with a result
	// that is pretty good but doesn't meet the tolerance
	// here I'm choosing to accept the pretty-good result

	if (bSSFailed){
		SetSSFailedMessage();

		if (pbOK != 0L) (*pbOK) = 0;
	}
	RestoreEventQueue();
}

// this is used to schedule additional doses into the future
void InterpADDL(int bGenGradients, const unsigned char* pc, int* pbOK, double* pdTime){
	int iCpt = -1;
	double v1, v2, dNum;
	int bOK = 1;
	char idbuf[4096];
	char* q;
	const unsigned char* pc1;
	const double *pdTemp = NULL;
	while (*pc){
		ModelBreakPoint();
		switch(*pc++){
			case 255:;
			break; case OP_RP:{
				SetFlag(0);
				return;
			}
			break; case OP_QUEST:{
				dStack[ndStack++] = _UNK;	// unknown value
			}
			break; case OP_LP:{
				pc1 = pc;
				ScanOverRP(&pc);
				// save pc1
				pcStack[npcStack++] = pc1;
			}
			break; case OP_DT:{
				SetFlag(0);
				v1 = dStack[--ndStack];
				// delay by v1
				*pdTime += v1;
			}
			break; case OP_BOLUS1:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v1 = dStack[--ndStack];
				// bolus to variable
				iCpt = GetCompartmentNumber(idbuf);
				ScheduleBolus1(1, *pdTime - curTime, iCpt, v1, 1);
			}
			break; case OP_INF1:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v2 = dStack[--ndStack];	// rate
				v1 = dStack[--ndStack];	// amt
				iCpt = GetCompartmentNumber(idbuf);
				ScheduleIV1(1, *pdTime - curTime, iCpt, v1, v2, 1);
			}
			break; case OP_BOLUS2:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v1 = dStack[--ndStack];
				// bolus to variable
				iCpt = GetCompartmentNumber(idbuf);
				ScheduleBolus2(1, *pdTime - curTime, iCpt, v1, 1);
			}
			break; case OP_INF2:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v2 = dStack[--ndStack];	// rate
				v1 = dStack[--ndStack];	// amt
				iCpt = GetCompartmentNumber(idbuf);
				ScheduleIV2(1, *pdTime - curTime, iCpt, v1, v2, 1);
			}
			break; case OP_RPT:{
				SetFlag(0);
				if (dStack[ndStack-1] <= 0){
					ndStack--;
					npcStack--;
				} else {
					pc -= 1;	// will redo the op
					dStack[ndStack-1] -= 1;
					InterpADDL(bGenGradients, pcStack[npcStack-1], pbOK, pdTime);
				}
			}
			break; case OP_NUM:{
				//dNum = *((double*)pc)++;
				pdTemp = (const double*)pc; dNum = *pdTemp++; pc = (const unsigned char*)pdTemp;
				dStack[ndStack++] = dNum;
			}
			break; case OP_DUP:{
				dNum = dStack[ndStack-1];
				dStack[ndStack++] = dNum;
			}
			break; case OP_COMMA: SetFlag(0);
			break; case OP_ADD: SetFlag(FLAG_ADD);
			break; case OP_SUB: SetFlag(FLAG_SUB);
			break; case OP_MUL: SetFlag(FLAG_MUL);
			break; case OP_DIV: SetFlag(FLAG_DIV);
			break; case OP_POW: SetFlag(FLAG_POW);
			break; default:
			{
				int breakhere = 1;
			}
		}
	}
}

void GetADDL(int bGenGradients, const unsigned char* pc, int* pbOK, double dOffset, int nAddl){
	int i;
	// if there's an offset, apply that
	double dLocalTime = curTime + dOffset;
	for (i = 0; i < nAddl; i++){
		InterpADDL(bGenGradients, pc, pbOK, &dLocalTime);
	}
}

static double v1, v2, v3;
static double dNum;
static char idbuf[256];
//static char numbuf[256];
static const unsigned char* pc1;

static int nRunStack;
static const unsigned char* pcRunStack[MAXSTACK];

static const unsigned char* pc;
static const unsigned char* pcLast;

void StartDroppingOut(){
	_bDroppingOut = 1;
}

void CleanOutActionList(int bGenGradients){
	// clean out the action list
	// this allows the model to run past the last event, such as when simulating
	while(zzNAction > 0
		&& _finite(zzDT[0])
		&& (_dMaxTime < 0 || curTime <= _dMaxTime)
		)
	{
		v1 = zzDT[0];
		// delay by v1
		EvalSParms(zzY);
//		SetupJacobian(bGenGradients);
		// last argument allows us to finish up subject
		AdvanceWithActions(bGenGradients, &curTime, v1, TRUE, TRUE);
	}
}

void Interp(
	  int bGenGradients
	, double dLL[]
	, double dGrad[]
	, double dId1, double dId2, double dId3, double dId4, double dId5
	, int* pnObs
	, double** args
	, int* pbOK
	, int iInterpMode
	, int bSampleEpsilon
	, int bForSimTbl
	, int bForVPC
	)
{
	int iCpt = -1;
	char* q;
	char c;
	BOOL bDone = FALSE;
	const double *pdTemp = NULL;
	_bDroppingOut = FALSE;
	_bInSS = FALSE;
//	zzNAction = 0;
////	curTime = 0;
	//testGammaDistr();	// temp test code
	//testTrap();
	if (_pcb) (*_pcb->_pfOnId)(dId1, dId2, dId3, dId4, dId5);
	while(!bDone && (pbOK==0L || *pbOK) && (pcLast = pc, *pc)){
		SetTimeOfStateCF(curTime);
		SetTimeOfGetValCF(curTime);
		c = *pc++;
		switch(c){
			case 127:;
			break; case OP_NUL: case OP_ID1: case OP_ID2: case OP_ID3: case OP_ID4: case OP_ID5:
				pc--;
				bDone = TRUE;
			break; case OP_CLR:{
				if (!_bDroppingOut){ndStack = nRunStack = npcStack = flags = 0;}
			}
			break; case OP_CLR_RESET:{
				if (!_bDroppingOut){iCurWhichReset = 0;}
			}
			break; case OP_INIT1:{
				if (iInterpMode==INTERP_MODE_INIT1){
					bDone = TRUE;
					if (_pcb)(*_pcb->_pfOnInit1)();
				} else if (!_bDroppingOut){
					EvalSParms(zzY);
					InitDelays(curTime);
					InitCF(curTime);
					RestartSubjectSequences();
					if (iInterpMode==INTERP_MODE_SIM){
						RestartSubjectEventSims();
					}
					if (_pcb)(*_pcb->_pfOnInit1)();
				}
			}
			break; case OP_ENDROW:{
				if (_pcb)(*_pcb->_pfOnEndRow)();
			}
			break; case OP_RP:{
				if (!_bDroppingOut){
					SetFlag(0);
					if (nRunStack <= 0) return;
					pc = pcRunStack[--nRunStack];
				}
			}
			break; case OP_QUEST:{
				if (!_bDroppingOut){
					dStack[ndStack++] = _UNK;	// unknown value
				}
			}
			break; case OP_LP:{
				pc1 = pc;
				ScanOverRP(&pc);
				if (!_bDroppingOut){
					// save pc1
					pcStack[npcStack++] = pc1;
				}
			}
			break; case OP_DT:{
				if (!_bDroppingOut){
					SetFlag(0);
					v1 = dStack[--ndStack];
					// delay by v1
					EvalSParms(zzY);
//					SetupJacobian(bGenGradients);
					AdvanceWithActions(bGenGradients, &curTime, v1, TRUE, 0);
				}
			}
			break; case OP_BOLUS1:{
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					SetFlag(0);
					v1 = dStack[--ndStack];
					// bolus to variable
					iCpt = GetCompartmentNumber(idbuf);
					EvalSParms(zzY);
					ScheduleBolus1(1, 0, iCpt, v1, 0);
//					SetupJacobian(bGenGradients);
					AdvanceWithActions(bGenGradients, &curTime, 0, TRUE, 0);
				}
			}
			break; case OP_INF1:{
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					SetFlag(0);
					v2 = dStack[--ndStack];	// rate
					v1 = dStack[--ndStack];	// amt
					// start infusion to variable
					iCpt = GetCompartmentNumber(idbuf);
					EvalSParms(zzY);
					ScheduleIV1(1, 0, iCpt, v1, v2, 0);
//					SetupJacobian(bGenGradients);
					AdvanceWithActions(bGenGradients, &curTime, 0, TRUE, 0);
				}
			}
			break; case OP_BOLUS2:{
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					SetFlag(0);
					v1 = dStack[--ndStack];
					// bolus to variable
					iCpt = GetCompartmentNumber(idbuf);
					EvalSParms(zzY);
					ScheduleBolus2(1, 0, iCpt, v1, 0);
//					SetupJacobian(bGenGradients);
					AdvanceWithActions(bGenGradients, &curTime, 0, TRUE, 0);
				}
			}
			break; case OP_INF2:{
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					SetFlag(0);
					v2 = dStack[--ndStack];	// rate
					v1 = dStack[--ndStack];	// amt
					// start infusion to variable
					iCpt = GetCompartmentNumber(idbuf);
					EvalSParms(zzY);
					ScheduleIV2(1, 0, iCpt, v1, v2, 0);
//					SetupJacobian(bGenGradients);
					AdvanceWithActions(bGenGradients, &curTime, 0, TRUE, 0);
				}
			}
			break; case OP_RESET:{
				if (!_bDroppingOut){
					CleanOutActionList(bGenGradients);
					ResetSubject();
					iCurWhichReset ++;
					if (_pcb) (*_pcb->_pfOnReset)();
				}
			}
			break; case OP_EVENT:{
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					SetFlag(0);
					v1 = dStack[--ndStack];
					// observed value is v1
					EvalSParms(zzY);
					if (0){
					} else if (iInterpMode==INTERP_MODE_LL){
						if (bGenGradients){
							NewGetLL(bGenGradients, dLL, dGrad, pnObs, curTime, idbuf, v1, 0);
						} else {
							GetLL(bGenGradients, dLL, dGrad, pnObs, curTime, idbuf, v1, 0);
						}
					} else if (iInterpMode==INTERP_MODE_SIM){
					}
				}
			}
			break; case OP_STRTOBS:{
			}
			break; case OP_OBS: case OP_OBS_BQL: {
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					SetFlag(0);
					v1 = dStack[--ndStack];
					// observed value is v1
					EvalSParms(zzY);
					if (0){
					} else if (iInterpMode==INTERP_MODE_PRED){
						if (c==OP_OBS){
							NewGetPred(bGenGradients, pnObs, curTime, idbuf, v1);
						}
					} else if (iInterpMode==INTERP_MODE_LL){
						if (bGenGradients){
							NewGetLL(bGenGradients, dLL, dGrad, pnObs, curTime, idbuf, v1, c==OP_OBS_BQL);
						} else {
							GetLL(bGenGradients, dLL, dGrad, pnObs, curTime, idbuf, v1, c==OP_OBS_BQL);
						}
					} else if (iInterpMode==INTERP_MODE_SIM){
						if (!bForSimTbl){
							Simulate(curTime, idbuf, v1, c==OP_OBS_BQL, bSampleEpsilon, bForVPC);
						}
					}
				}
			}
			break; case OP_ENDOBS:{
				if (!_bDroppingOut){
					DoEndObsActions();
				}
			}
			break; case OP_COVR:{
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					v1 = dStack[--ndStack];	// val0
					// set covariate
					SetCovariate(curTime, idbuf, v1);
					if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
					if (GetStripDoseName(idbuf)){
						if (v1 == 0){
							SetErrorMsg("Error: stripping dose is zero.");
							if (pbOK != 0L) (*pbOK) = 0;
						}
						if (_pcb)(*_pcb->_pfOnStripDose)(curTime, curTimeLastDose, idbuf, v1);
					} else {
						//EvalSParms(zzY); // don't, because all the covariates may not have been set yet, causing a failure here
						if (_pcb)(*_pcb->_pfOnCovariate)(curTime, curTimeLastDose, idbuf, v1);
					}
				}
			}
			// val1 dt val2 dcovr(covname) // backward version
			break; case OP_DCOVR:{
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					SetFlag(0);
					v2 = dStack[--ndStack];	// val2
					v3 = dStack[--ndStack];	// dt
					v1 = dStack[--ndStack];	// val1
					// set covariate interpolation params
					dNum = 0; GetVarValue(idbuf, &dNum);
					SetCovariateInterp(curTime, idbuf, v1, v3, v2, FALSE);
					if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
					if (v2 != dNum){
						Advance(bGenGradients, iODELevel, &curTime, 0, TRUE);
						EvalSParms(zzY);
						ReInitCF(curTime);
					}
					if (_pcb){
						EvalSParms(zzY);
						(*_pcb->_pfOnCovariate)(curTime, curTimeLastDose, idbuf, v1);
					}
				}
			}
			// val1 dt val2 fcovr(covname) // forward version
			break; case OP_FCOVR:{
				for (q=idbuf; *q++ = *pc++;);
				if (!_bDroppingOut){
					SetFlag(0);
					v2 = dStack[--ndStack];	// val2
					v3 = dStack[--ndStack];	// dt
					v1 = dStack[--ndStack];	// val1
					// set covariate interpolation params
					dNum = 0; GetVarValue(idbuf, &dNum);
					SetCovariateInterp(curTime, idbuf, v1, v3, v2, TRUE);
					if (*_piState==ODESTATE_UNDERWAY) *_piState = ODESTATE_CHANGE;
					if (v1 != dNum){
						AdvanceWithActions(bGenGradients, &curTime, 0, TRUE, 0);
						EvalSParms(zzY);
						ReInitCF(curTime);
					}
					if (_pcb){
						EvalSParms(zzY);
						(*_pcb->_pfOnCovariate)(curTime, curTimeLastDose, idbuf, v1);
					}
				}
			}
			break; case OP_RPT:{
				if (!_bDroppingOut){
					SetFlag(0);
					if (dStack[ndStack-1] <= 0){
						ndStack--;
						npcStack--;
					} else {
						pc -= 1;	// will redo the op
						dStack[ndStack-1] -= 1;
						pcRunStack[nRunStack++] = pc;
						pc = pcStack[npcStack-1];
					}
				}
			}
			break; case OP_SS:{
				if (!_bDroppingOut){
					if (_pcb) (*_pcb->_pfOnSS)();
					SetFlag(0);
					_bInSS = TRUE;
					GetSS(bGenGradients, pcStack[npcStack-1], pbOK, args[0]);
					if (pbOK && *pbOK == 0) break;
					_bInSS = FALSE;
					npcStack--;
				}
			}
			break; case OP_SS1:{
			}
			break; case OP_SS2:{
			}
			break; case OP_SSEND:{
			}
			break; case OP_SSNM:{
				if (!_bDroppingOut){
					if (_pcb) (*_pcb->_pfOnSS)();
					SetFlag(0);
					_bInSS = TRUE;
					GetSS(bGenGradients, pcStack[npcStack-1], pbOK, args[0]);
					if (pbOK && *pbOK == 0) break;
					_bInSS = FALSE;
					// schedule an additional cycle starting at this time
					if (*pbOK && !_bDroppingOut){
						SetFlag(0);
						GetADDL(bGenGradients, pcStack[npcStack-1], pbOK, 0.0, 1);
					}
					npcStack--;
				}
			}
			break; case OP_ADDL:{
				if (!_bDroppingOut){
					int nAddl = 0;
					v1 = dStack[--ndStack];	// val0
					nAddl = (int)floor(v1 + 0.5);
					SetFlag(0);
					GetADDL(bGenGradients, pcStack[npcStack-1], pbOK, 0.0, nAddl);
					npcStack--;
				}
			}
			break; case OP_ADDLNM:{
				if (!_bDroppingOut){
					int nAddl = 0;
					double dAddlOffset = 0;
					v2 = dStack[--ndStack]; // offset
					dAddlOffset = v2;
					v1 = dStack[--ndStack];	// repetitions
					nAddl = (int)floor(v1 + 0.5);
					SetFlag(0);
					GetADDL(bGenGradients, pcStack[npcStack-1], pbOK, dAddlOffset, nAddl);
					npcStack--;
				}
			}
			break; case OP_NUM:{
				//dNum = *((double*)pc)++;
				pdTemp = (const double*)pc; dNum = *pdTemp++; pc = (const unsigned char*)pdTemp;
				if (!_bDroppingOut){
					dStack[ndStack++] = dNum;
				}
			}
			break; case OP_DUP:{
				if (!_bDroppingOut){
					dNum = dStack[ndStack-1];
					dStack[ndStack++] = dNum;
				}
			}
			break; case OP_COMMA: SetFlag(0);
			break; case OP_ADD: SetFlag(FLAG_ADD);
			break; case OP_SUB: SetFlag(FLAG_SUB);
			break; case OP_MUL: SetFlag(FLAG_MUL);
			break; case OP_DIV: SetFlag(FLAG_DIV);
			break; case OP_POW: SetFlag(FLAG_POW);
			break; default:
			{
				int breakhere = 1;
			}
		}
		ModelBreakPoint();
	}
	if (pbOK && *pbOK == 0) return;
	if (_pcb) (*_pcb->_pfAfterLastEvent)();
	CleanOutActionList(bGenGradients);
	if (_pcb) (*_pcb->_pfOnEndId)();
}

extern void InterpScanSubject(int* pnObs, double* pdMaxTime);	// forward
void InterpScanSubjectGetADDL(const unsigned char* pc1, int nAddl, double dOffset, double* pdMaxTime){
	int i;
	const unsigned char* pcSave = pc;
	int nObs = 0;
	for (i = 0; i < nAddl; i++){
		double dLocalMaxTime = 0;
		pc = pc1;
		InterpScanSubject(&nObs, &dLocalMaxTime);
		*pdMaxTime += dLocalMaxTime;
	}
	// if there's an offset, apply that
	*pdMaxTime += dOffset;
	pc = pcSave;
}

void InterpScanSubject(int* pnObs, double* pdMaxTime){
	int iCpt = -1;
	char* q;
	char c;
	BOOL bDone = FALSE;
	double dLocalTime = 0;
	const double *pdTemp = NULL;
	while(!bDone && (pcLast = pc, *pc)){
		c = *pc++;
		switch(c){
			case 127:;
			break; case OP_NUL: case OP_ID1: case OP_ID2: case OP_ID3: case OP_ID4: case OP_ID5:
				pc--;
				bDone = TRUE;
			break; case OP_CLR:{ndStack = nRunStack = npcStack = flags = 0;}
			break; case OP_CLR_RESET:{}
			break; case OP_INIT1:{}
			break; case OP_ENDROW:{}
			break; case OP_RP:{
				SetFlag(0);
				if (nRunStack <= 0) return;
				pc = pcRunStack[--nRunStack];
			}
			break; case OP_QUEST:{
				dStack[ndStack++] = _UNK;	// unknown value
			}
			break; case OP_LP:{
				pc1 = pc;
				ScanOverRP(&pc);
				// save pc1
				pcStack[npcStack++] = pc1;
			}
			break; case OP_DT:{
				SetFlag(0);
				v1 = dStack[--ndStack];
				dLocalTime += v1;
				if (*pdMaxTime < dLocalTime) *pdMaxTime = dLocalTime;
			}
			break; case OP_BOLUS1:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v1 = dStack[--ndStack];
			}
			break; case OP_INF1:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v2 = dStack[--ndStack];	// rate
				v1 = dStack[--ndStack];	// amt
				if (v2 != 0 && *pdMaxTime < dLocalTime + v1/v2) *pdMaxTime = dLocalTime + v1/v2;
			}
			break; case OP_BOLUS2:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v1 = dStack[--ndStack];
			}
			break; case OP_INF2:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v2 = dStack[--ndStack];	// rate
				v1 = dStack[--ndStack];	// amt
				if (v2 != 0 && *pdMaxTime < dLocalTime + v1/v2) *pdMaxTime = dLocalTime + v1/v2;
			}
			break; case OP_RESET:{}
			break; case OP_EVENT:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v1 = dStack[--ndStack];
			}
			break; case OP_STRTOBS:{}
			break; case OP_OBS: case OP_OBS_BQL: {
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v1 = dStack[--ndStack];
				if (c==OP_OBS){(*pnObs) ++;}
			}
			break; case OP_ENDOBS:{}
			break; case OP_COVR:{
				for (q=idbuf; *q++ = *pc++;);
				v1 = dStack[--ndStack];	// val0
			}
			// val1 dt val2 dcovr(covname) // backward version
			break; case OP_DCOVR:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v2 = dStack[--ndStack];	// val2
				v3 = dStack[--ndStack];	// dt
				v1 = dStack[--ndStack];	// val1
			}
			// val1 dt val2 fcovr(covname) // forward version
			break; case OP_FCOVR:{
				for (q=idbuf; *q++ = *pc++;);
				SetFlag(0);
				v2 = dStack[--ndStack];	// val2
				v3 = dStack[--ndStack];	// dt
				v1 = dStack[--ndStack];	// val1
			}
			break; case OP_RPT:{
				SetFlag(0);
				if (dStack[ndStack-1] <= 0){
					ndStack--;
					npcStack--;
				} else {
					pc -= 1;	// will redo the op
					dStack[ndStack-1] -= 1;
					pcRunStack[nRunStack++] = pc;
					pc = pcStack[npcStack-1];
				}
			}
			break; case OP_SS:{
				SetFlag(0);
				npcStack--;
			}
			break; case OP_SS1:{
			}
			break; case OP_SS2:{
			}
			break; case OP_SSEND:{
			}
			break; case OP_SSNM:{
				SetFlag(0);
				npcStack--;
			}
			break; case OP_ADDL:{
				int nAddl = 0;
				v1 = dStack[--ndStack];	// n repetitions
				nAddl = (int)floor(v1 + 0.5);
				SetFlag(0);
				InterpScanSubjectGetADDL(pcStack[npcStack-1], nAddl, 0.0, pdMaxTime);
				npcStack--;
			}
			break; case OP_ADDLNM:{
				int nAddl = 0;
				double dAddlOffset = 0;
				v2 = dStack[--ndStack];	// offset
				dAddlOffset = v2;
				v1 = dStack[--ndStack];	// n repetitions
				nAddl = (int)floor(v1 + 0.5);
				SetFlag(0);
				InterpScanSubjectGetADDL(pcStack[npcStack-1], nAddl, dAddlOffset, pdMaxTime);
				npcStack--;
			}
			break; case OP_NUM:{
				pdTemp = (const double*)pc; dNum = *pdTemp++; pc = (const unsigned char*)pdTemp;
				dStack[ndStack++] = dNum;
			}
			break; case OP_DUP:{
				dNum = dStack[ndStack-1];
				dStack[ndStack++] = dNum;
			}
			break; case OP_COMMA: SetFlag(0);
			break; case OP_ADD: SetFlag(FLAG_ADD);
			break; case OP_SUB: SetFlag(FLAG_SUB);
			break; case OP_MUL: SetFlag(FLAG_MUL);
			break; case OP_DIV: SetFlag(FLAG_DIV);
			break; case OP_POW: SetFlag(FLAG_POW);
			break; default:
			{
				int breakhere = 1;
			}
		}
	}
}

void ScanSubject(int* pnObs){
	BOOL bDone = FALSE;
	while(!bDone && (pcLast = pc, *pc)){
		int c = *pc++;
		switch(c){
			case 255:;
			break;
			case OP_NUL:
			case OP_ID1:
			case OP_ID2:
			case OP_ID3:
			case OP_ID4:
			case OP_ID5:
			{
				pc--;
				bDone = TRUE;
			}

			break;
			case OP_RP:
			{
				ASSERT(0);	// should not get here
			}

			break;
			case OP_INIT1:
			case OP_ENDROW:
			case OP_STRTOBS:
			case OP_ENDOBS:
			case OP_CLR:
			case OP_CLR_RESET:
			case OP_DT:
			case OP_RESET:
			case OP_RPT:
			case OP_SS:
			case OP_SS1:
			case OP_SS2:
			case OP_SSEND:
			case OP_SSNM:
			case OP_ADDL:
			case OP_ADDLNM:
			case OP_DUP:
			case OP_COMMA:
			case OP_ADD:
			case OP_SUB:
			case OP_MUL:
			case OP_DIV:
			case OP_POW:
			case OP_QUEST:

			break;
			case OP_LP:
			{
				ScanOverRP(&pc);
			}

			break;
			case OP_BOLUS1:
			case OP_INF1:
			case OP_BOLUS2:
			case OP_INF2:
			case OP_COVR:
			case OP_DCOVR:
			case OP_FCOVR:
			case OP_EVENT:
			{
				while(*pc++);
			}

			break;
			case OP_OBS:
			case OP_OBS_BQL:
			{
				if (c==OP_OBS){(*pnObs) ++;}
				while(*pc++);
			}

			break;
			case OP_NUM:
			{
				pc += sizeof(dNum);
			}

			break; default:
			{
				ASSERT(0);	// should not get here
			}
		}
	}
}
void ScanSubjectGetCovariate(const char* sCovName, double* pVal){
	int nCV = 0;
	char* q;
	double dCVSum = 0;
	const double *pdTemp = NULL;
	double dNum0 = 0, dNum1 = 0, dNum2 = 0;
	BOOL bDone = FALSE;
	while(!bDone && (pcLast = pc, *pc)){
		int c = *pc++;
		switch(c){
			case 255:;
			break;
			case OP_NUL:
			case OP_ID1:
			case OP_ID2:
			case OP_ID3:
			case OP_ID4:
			case OP_ID5:
			{
				pc--;
				bDone = TRUE;
			}

			break;
			case OP_RP:
			{
				ASSERT(0);	// should not get here
			}

			break;
			case OP_INIT1:
			case OP_ENDROW:
			case OP_STRTOBS:
			case OP_ENDOBS:
			case OP_CLR:
			case OP_CLR_RESET:
			case OP_DT:
			case OP_RESET:
			case OP_RPT:
			case OP_SS:
			case OP_SS1:
			case OP_SS2:
			case OP_SSEND:
			case OP_SSNM:
			case OP_ADDL:
			case OP_ADDLNM:
			case OP_DUP:
			case OP_COMMA:
			case OP_ADD:
			case OP_SUB:
			case OP_MUL:
			case OP_DIV:
			case OP_POW:
			case OP_QUEST:

			break;
			case OP_LP:
			{
				ScanOverRP(&pc);
			}

			break;
			case OP_BOLUS1:
			case OP_INF1:
			case OP_BOLUS2:
			case OP_INF2:
			case OP_COVR:
			case OP_DCOVR:
			case OP_FCOVR:
			case OP_EVENT:
			{
				for (q=idbuf; *q++ = *pc++;);
				if (strcmp(idbuf, sCovName)==0){
					if (c==OP_COVR){
						double d = dNum0;
						dCVSum += d;
						nCV++;
					} else if (c==OP_DCOVR || c==OP_FCOVR){
						double d = (dNum0 + dNum2) / 2;
						dCVSum += d;
						nCV++;
					}
				}
			}

			break;
			case OP_OBS:
			case OP_OBS_BQL:
			{
				while(*pc++);
			}

			break;
			case OP_NUM:
			{
				dNum2 = dNum1;
				dNum1 = dNum0;
				//dNum0 = *((double*)pc)++;
				pdTemp = (const double*)pc; dNum0 = *pdTemp++; pc = (const unsigned char*)pdTemp;
			}

			break; default:
			{
				ASSERT(0);	// should not get here
			}
		}
	}
	if (nCV > 0){
		*pVal = dCVSum / nCV;
	}
}

void GetFixefName(int *pi, char* nm);
void SetArgsOK(double** _args, int *_pbOK, int* _piState);

void LLOneSubject1(
	  int bGenGradients
	, int nEta
	, int* pnObs
	, const unsigned char* *pb
	, double* pdId1, double* pdId2, double* pdId3, double* pdId4, double* pdId5
	, double _dLL[]
	, double _dGrad[]
	, double** args
	, int* pbOK
	)
{
	int iState = 1;
	pc = *pb;
	SetArgsOK(args, pbOK, &iState);
	InitSubject();
	Interp(bGenGradients, _dLL, _dGrad, *pdId1, *pdId2, *pdId3, *pdId4, *pdId5, pnObs, args, pbOK, INTERP_MODE_LL, TRUE, FALSE, FALSE);
	EndSubject(0);
	*pb = pc;
}

void LLOneSubject(
	  int bGenGradients
	, int* pnObs
	, const unsigned char* *pb
	, double* pdId1
	, double* pdId2
	, double* pdId3
	, double* pdId4
	, double* pdId5
	, double** args
	, int* pbOK
	, double dLL[]
	, double dGrad[]
	)
{
	int iState = 1;
	pc = *pb;
	SetArgsOK(args, pbOK, &iState);
	if (bGenGradients){
		GetLLGradientEta(pnObs, pb, pdId1, pdId2, pdId3, pdId4, pdId5, dLL, dGrad);
		*pb = pc;
	} else {
		InitSubject();
		Interp(FALSE, dLL, dGrad, *pdId1, *pdId2, *pdId3, *pdId4, *pdId5, pnObs, args, pbOK, INTERP_MODE_LL, TRUE, FALSE, FALSE);
		EndSubject(0);
		*pb = pc;
	}
}
void PredOneSubject(
	  int* pnObs
	, const unsigned char* *pb
	, double* pdId1, double* pdId2, double* pdId3, double* pdId4, double* pdId5
	, double** args
	, int* pbOK
	, int* __nPrediction
	, double* __daXPrediction, double* __taLastDose, int* __iaWhichDose, int* __iaWhichReset, double* __daYPrediction, double* __daYObservation, double* __daVarianceFunction
	, int* __iaWhichObs
	)
{
	int iState = 1;
	double dLL[1], dGrad[1];
	dLL[0] = 0;
	pc = *pb;
	SetArgsOK(args, pbOK, &iState);
	SetPredArrays(__nPrediction
		, __daXPrediction
		, __taLastDose
		, __iaWhichDose
		, __iaWhichReset
		, __daYPrediction
		, __daYObservation
		, __daVarianceFunction
		, __iaWhichObs
		);
	InitSubject();
	Interp(FALSE, dLL, dGrad
		, *pdId1, *pdId2, *pdId3, *pdId4, *pdId5
		, pnObs
		, args
		, pbOK
		, INTERP_MODE_PRED
		, TRUE
		, FALSE
		, FALSE
		);
	EndSubject(0);
	*pb = pc;
}
void NewPredOneSubject1(
	  int bGenGradients
	, int nEta
	, int* pnObs
	, const unsigned char* *pb
	, double* pdId1, double* pdId2, double* pdId3, double* pdId4, double* pdId5
	, double** args
	, int* pbOK
	, int* __nPrediction
	, double* __daXPrediction, double* __taLastDose, int* __iaWhichDose, int* __iaWhichReset, double* __daYPrediction, double* __daYObservation, double* __daVarianceFunction
	, double* __daYdK
	, double* __daSdK
	, int* __iaWhichObs
	)
{
	int iState = 1;
	pc = *pb;
	SetArgsOK(args, pbOK, &iState);
	SetPredArrays(__nPrediction
		, __daXPrediction
		, __taLastDose
		, __iaWhichDose
		, __iaWhichReset
		, __daYPrediction
		, __daYObservation
		, __daVarianceFunction
		, __iaWhichObs
		);
	if (bGenGradients){
		SetGradArrays(nEta, __daYdK, __daSdK);
	}
	InitSubject();
	Interp(bGenGradients, NULL, NULL
		, *pdId1, *pdId2, *pdId3, *pdId4, *pdId5
		, pnObs
		, args
		, pbOK
		, INTERP_MODE_PRED
		, TRUE
		, FALSE
		, FALSE
		);
	EndSubject(0);
	*pb = pc;
}
void SimOneSubject(
	const unsigned char* *pb
	, double* pdId1
	, double* pdId2
	, double* pdId3
	, double* pdId4
	, double* pdId5
	, double** args
	, int* pbOK
	, int bSampleEpsilon
	, int bForSimTbl
	, int bForVPC
	)
{
	//testGammaDistr();	// temp test code
	int iState = 1;
	pc = *pb;
	SetArgsOK(args, pbOK, &iState);
	InitSubject();
	Interp(FALSE, NULL, NULL
		, *pdId1
		, *pdId2
		, *pdId3
		, *pdId4
		, *pdId5
		, NULL
		, args
		, pbOK
		, INTERP_MODE_SIM
		, bSampleEpsilon
		, bForSimTbl
		, bForVPC
		);
	EndSubject(1);
	*pb = pc;
}
void Init1OneSubject(
	const unsigned char* *pb
	, double* pdId1
	, double* pdId2
	, double* pdId3
	, double* pdId4
	, double* pdId5
	, double** args
	, int* pbOK
	)
{
	int iState = 1;
	pc = *pb;
	SetArgsOK(args, pbOK, &iState);
	InitSubject();
	Interp(FALSE, NULL, NULL
		, *pdId1
		, *pdId2
		, *pdId3
		, *pdId4
		, *pdId5
		, NULL
		, args
		, pbOK
		, INTERP_MODE_INIT1
		, TRUE
		, FALSE
		, FALSE
		);
	EndSubject(0);
	*pb = pc;
}

void ScanOneSubject(
	const unsigned char* *pb
	)
{
	int n = 0;
	pc = *pb;
	ScanSubject(&n);
	*pb = pc;
}
void ScanOneSubjectCountObs(
	const unsigned char* *pb
	, int* pnObs
	, double* pdMaxTime
	)
{
	pc = *pb;
	InterpScanSubject(pnObs, pdMaxTime);
	*pb = pc;
}
void ScanOneSubjectGetCovariate(
	const unsigned char* *pb
	, const char* sCovName
	, double* pVal
	)
{
	pc = *pb;
	ScanSubjectGetCovariate(sCovName, pVal);
	*pb = pc;
}
#undef pb

/******************************************************************************/
/*
  Purpose:

    POLY_VALUE evaluates a double precision polynomial.

  Discussion:

    For sanity's sake, the value of N indicates the NUMBER of 
    coefficients, or more precisely, the ORDER of the polynomial,
    rather than the DEGREE of the polynomial.  The two quantities
    differ by 1, but cause a great deal of confusion.

    Given N and A, the form of the polynomial is:

      p(x) = a[0] + a[1] * x + ... + a[n-2] * x^(n-2) + a[n-1] * x^(n-1)

  Licensing:

    This code is distributed under the GNU LGPL license. 

  Modified:

    19 March 2010

  Author:

    John Burkardt

  Parameters:

    Input, int N, the order of the polynomial.

    Input, double A[N], the coefficients of the polynomial.
    A[0] is the constant term.

    Input, double X, the point at which the polynomial is to be evaluated.

    Output, double POLY_VALUE, the value of the polynomial at X.
*/
double poly_value(int n, double a[], double x){
	int i;
	double value = 0;
	for (i = n - 1; 0 <= i; i-- ){
		value = value * x + a[i];
	}
	return value;
}

/******************************************************************************/
/*
  Purpose:

    probit or original asa241 name R8_NORMAL_01_CDF_INVERSE inverts the standard normal CDF.

  Discussion:

    The result is accurate to about 1 part in 10^16.

  Licensing:

    This code is distributed under the GNU LGPL license. 

  Modified:

    19 March 2010

  Author:

    Original FORTRAN77 version by Michael Wichura.
    C version by John Burkardt.

  Reference:

    Michael Wichura,
    The Percentage Points of the Normal Distribution,
    Algorithm AS 241,
    Applied Statistics,
    Volume 37, Number 3, pages 477-484, 1988.

  Parameters:

    Input, double P, the value of the cumulative probability 
    densitity function.  0 < P < 1.  If P is outside this range, an "infinite"
    value is returned.

    Output, double R8_NORMAL_01_CDF_INVERSE, the normal deviate value 
    with the property that the probability of a standard normal deviate being 
    less than or equal to this value is P.
*/
double probit(double p){
	static double a[8] = { 
		3.3871328727963666080,     1.3314166789178437745e+2,
		1.9715909503065514427e+3,  1.3731693765509461125e+4,
		4.5921953931549871457e+4,  6.7265770927008700853e+4,
		3.3430575583588128105e+4,  2.5090809287301226727e+3 };
	static double b[8] = {
		1.0,                       4.2313330701600911252e+1,
		6.8718700749205790830e+2,  5.3941960214247511077e+3,
		2.1213794301586595867e+4,  3.9307895800092710610e+4,
		2.8729085735721942674e+4,  5.2264952788528545610e+3 };
	static double c[8] = {
		1.42343711074968357734,     4.63033784615654529590,
		5.76949722146069140550,     3.64784832476320460504,
		1.27045825245236838258,     2.41780725177450611770e-1,
		2.27238449892691845833e-2,  7.74545014278341407640e-4 };
	static double const1 = 0.180625;
	static double const2 = 1.6;
	static double d[8] = {
		1.0,                        2.05319162663775882187,
		1.67638483018380384940,     6.89767334985100004550e-1,
		1.48103976427480074590e-1,  1.51986665636164571966e-2,
		5.47593808499534494600e-4,  1.05075007164441684324e-9 };
	static double e[8] = {
		6.65790464350110377720,     5.46378491116411436990,
		1.78482653991729133580,     2.96560571828504891230e-1,
		2.65321895265761230930e-2,  1.24266094738807843860e-3,
		2.71155556874348757815e-5,  2.01033439929228813265e-7 };
	static double f[8] = {
		1.0,                        5.99832206555887937690e-1,
		1.36929880922735805310e-1,  1.48753612908506148525e-2,
		7.86869131145613259100e-4,  1.84631831751005468180e-5,
		1.42151175831644588870e-7,  2.04426310338993978564e-15 };
	double q;
	double r;
	static double split1 = 0.425;
	static double split2 = 5.0;
	double value;

	if (p <= 0) return -1e30;

	if (p >= 1) return 1e30;

	q = p - 0.5;

	if (q >= -split1 && q <= split1){
		r = const1 - q * q;
		value = q * poly_value(8, a, r) / poly_value(8, b, r);
	} else {
		r = (q < 0 ? p : 1-p);

		// this should not be possible, and even if it is, exit() is a no-no
		//if (r <= 0){
		//	value = - 1;
		//	exit(1);
		//}

		r = sqrt(-log(r));

		if (r <= split2){
			r = r - const2;
			value = poly_value(8, c, r) / poly_value(8, d, r); 
		} else {
			r = r - split2;
			value = poly_value(8, e, r) / poly_value(8, f, r);
		}

		if (q < 0) value = - value;
	}
	return value;
}


